;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/defs.lisp *-*
;;;; *-* Last-edit: Monday, March 8, 1993  17:34:45; Edited-By: carlson *-* 
;;;; *-* Machine: Lester (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                               Definitions                              *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  07-08-92 File Created.  (Carlson)
;;;  02-13-93 Added (named-object remember-instances object-with-properties)
;;;           superclass to `data'.
;;;           Added  *dataset-class* and *variable-class* so the interface can 
;;;           control the class of the objects created.
;;;           Altered `dataset' and `variable' accessors to use slots from 
;;;           `named-object'.  (Westy)
;;;  06-21-94 Began changes to remove dependency on RTM. Bascially this
;;;           storing the values as a sequence in a slot of the variable.  (Westy)
;;;  08-25-94 Added code to `make-clasp-name' to parse the numerical suffix so
;;;           that we extract the correct prefix.  (Westy)
;;;  08-30-94 Moved all the variable-value code to VARIABLE-VALUE.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp)

;;; --*--
;;; ***************************************************************************


;;; ***************************************************************************
;;;; Data set definition
;;; ***************************************************************************

(defclass data (named-object remember-instances object-with-properties)
  ((value :initarg :value :accessor value)
   #+ignore
   (id
     :type symbol
     :documentation "Unique data identifier"
     :initarg :id
     :accessor id)
   (source
     :type t
     :documentation "Where did this data come from?"
     :initarg :source
     :accessor source))
  (:documentation
  "The basic data class for clasp.  Dataset, variable etc.  are all subclasses
  of data"))

(defmethod description ((self data))
  (with-slots (description) self
     (setf description (or description (name self) (id self)))))

(defmethod initialize-instance :after ((self data) &rest ignore)
  (declare (ignore ignore))
  (with-slots (name) self
    (unless (and name
                 (stringp name)
                 (> (length name) 0))
      (setf name (make-clasp-name (class-name (class-of self)) :data)))))

;;; ***************************************************************************
;;;; Data set definition
;;; ***************************************************************************

;;; There are two basic data structures in clasp.  The variable holds a set of
;;; data and is intended to represent a variable in an experiment.  In most
;;; descriptions of data, the variable will correspond to a column of data.
;;; The dataset is a collection of variables.  Its rows represent tuples of
;;; data which could be particular trials in an experiment.  Its columns are
;;; variables.

;;; A dataset represents a group of related variables, and some extra
;;; information about them

#+:franz-inc
(setf excl:*enable-package-locked-errors* nil)

(defvar *dataset-class* 'dataset)

(defclass dataset (data)
  (
   #+OLD-VARS
   (write-data
     :type symbol
     :documentation
     "Storage for the actual value of the variables.  This is the name of the
     rtm table."
     :initarg :write-data
     :accessor dataset-write-data)
   #+OLD-VARS
   (read-data
    :type symbol
    :documentation
    "Name of views for cases where data must be read and written in different
    ways"
    :initarg :read-data
    :accessor dataset-read-data)
   ;; Get rid of this someday?
   (where-clause
    :type list
    :documentation "Restrictions of range of data within rtm table"
    :initarg :where-clause
    :accessor dataset-where-clause)
   (variables
     :type list
     :documentation
     "List of the variables in a dataset"
     :initarg :variables
     :accessor dataset-variables)
   (variable-count
     :type hash-table
     :documentation
     "Used to create unique names for variables by adding a counter to
     conflicting names"
     :initarg :variable-count
     :accessor dataset-variable-count)
   (rows
     :type integer
     :documentation "The number of rows in the dataset"
     :initarg :rows
     :accessor dataset-rows)
   #+NOT-USED
   (by-row-cache
    :initform nil
    :type list
    :documentation "Stores the dataset in row-major-order"
    )
   )
  
  (:documentation
  "The dataset is the top-level user class for data representation in CLASP.
  Datasets are collections of variables"))

(defmethod dataset-variable-count ((the-dataset t))
  (declare (special *clasp-variable-count*))
  *clasp-variable-count*)

;;; A variable consists of information about a particular variable in a dataset

(defvar *variable-class*  'variable)

(defclass variable (data)
  ((value
     :type t
     :documentation "The value of a variable, expressed as a list"
     :initarg :value
     :initform nil)
   (type
    :type (member :numerical :categorical :ordinal)
    :documentation "The statistical type of the variable."
    :initform :numerical ;; assume it is numbers, relax it if necessary
    )
   (order-by-array-cache
    :type t
    :documentation "Cache of ordered values as an array"
    :initform nil ; later we can make it a hash table
    )
   (order-by-list-cache
    :type t
    :documentation "Cache of ordered values as a list"
    :initform nil ; later we can make it a hash table
    )
   (dataset
     :type dataset
     :documentation "The dataset that the variable belongs to"
     :initarg :dataset
     :accessor variable-dataset)
   #+OLD-VARS
   (attribute
     ;;:type (I don't know what types attributes are)
     :documentation
     "The attribute information for the particular column of the dataset rtm
     table"
     :initarg :attribute
     :accessor variable-attribute))
  (:documentation
  "Variables are ordered collections of things.  Valid data types for variables
  are numbers (integers, rationals and floats) and symbols and strings (for
  categorical data)."))

(defmethod categorical-variable-p ((self variable))
  (eq (slot-value self 'type) :categorical))

#+:franz-inc
(setf excl:*enable-package-locked-errors* t)

(defmethod description ((self variable))
  (with-slots (description name dataset) self
    (if (null description)
	(setf description (format nil "~@(~a from ~a~)" name (description dataset)))
      description)))

(defvar *row-number-variable-class* 'row-number-variable)

(defclass row-number-variable (variable)
  ((indirect-array)
   (sorted? :initform t :reader sorted-p)))

(defmethod row-number-p ((self variable))
  nil)

(defmethod row-number-p ((self row-number-variable))
  t)

(defmethod row-number-variable ((self symbol))
  (get-variable 'row-number (get-dataset self)))

(defmethod row-number-variable ((self dataset))
  (get-variable 'row-number self))

;;; ***************************************************************************
;;;; Support functions
;;; ***************************************************************************

;;; These are low-level functions used by the dataset and variable functions to
;;; do such tasks as create new symbols for internally stored objects like
;;; rtm-tables which the user never sees.

#+OLD
(defmethod print-object ((the-variable variable) stream) 
  (if (and *print-pretty* (not *print-escape*))
      (format stream "~&id: ~a~%name: ~a~%dataset: ~a~%attribute: ~a
~%sample values: ~a"
	      (id the-variable)
	      (name the-variable)
	      (name (variable-dataset the-variable))
	      #+OLD-VARS
	      (variable-attribute the-variable)
              :CRAP
	      (subseq (variable-value the-variable) 0 4))
    (print-unreadable-object (the-variable stream :type t)
      (write-string (id-string the-variable) stream))))

(defmethod print-object ((object row-number-variable) stream)
  (if (and *print-escape* (not *override-print-escape*))
    (format stream "#.(~a '~a)" (class-name (class-of object)) (name (variable-dataset object)))
    (let ((name (name object)))
      (format stream "~:[(unnamed)~;~a~]" name name))))

#+OLD
(defmethod print-object ((the-dataset dataset) stream)
  (if (and *print-pretty* (not *print-escape*))
      (format stream "~&id: ~a~%name: ~a~%data: ~a
~%variables: (~{~a~^, ~})~%rows: ~a"
	      (id the-dataset)
	      (name the-dataset)
	      #+OLD-VARS
	      (dataset-write-data the-dataset)
              :CRAP
	      (mapcar #'name (dataset-variables the-dataset))
	      (dataset-rows the-dataset))
    (print-unreadable-object (the-dataset stream :type t)
      (write-string (name the-dataset) stream))))

;;; The following variables hold numbers which make allow for unique naming of
;;; various clasp internally stored objects.

(defvar *clasp-dataset-count*
	(make-hash-table :test #'equal)
  "Used to create unique names for clasp datasets")

(defvar *clasp-variable-count*
	(make-hash-table :test #'equal)
  "Used to create unique names for clasp variables")

(defvar *clasp-data-count*
	(make-hash-table :test #'equal)
  "Used to create unique names for objects not having their own counter")

(defun make-unique-string (prefix &optional (count 0))
  "Uppercases `prefix' and replaces all the whitespace with dashes.  Also
appends -`count' to the end of the string for non-zero `count's"
  (format nil "~:@(~A~@[-~A~]~)" 
	  (substitute-if #\- 
			 #'(lambda (char) 
			     (member char
				     '(#\SPACE #\TAB #\NEWLINE
				       #\LINEFEED #\PAGE #\RETURN)
				     :test #'char=))
			 (remove-if #'(lambda (char)
					(member char
						'(#\( #\) #\' #\|)
						:test #'char=))
				    prefix))
	  (and (not (= 0 count)) count)))

(defun strip-suffix (string)
  (let ((position (position #\- string :from-end t :test #'char=)))
    (or (when position
          (multiple-value-bind (integer end-position)
              (parse-integer string :start position :junk-allowed t)
            (when (and integer (= end-position (length string)))
              (subseq string 0 position))))
	string)))

(defun make-clasp-name (name type &optional dataset (strip-suffix t))
  
  "Generate a unique title given a title and a type.  Valid types are
:dataset, :variable and :data.  If the type is variable, then dataset
will be the dataset which the variable is created (a variable
name that is unique in one dataset may not be in another).  The first
occurence of a given prefix produces a string the same as the prefix
(capitalized and with dashes substitued for whitespace).  Subsequent
repetitions of a prefix will cause a number to be appended to force
uniqueness."
  (let* ((prefix (if strip-suffix
		     (strip-suffix (string name))
                     (string name)))
	 (key (string-upcase prefix))
	 count)
    
    (when (= 0 (length key)) (setf key (format nil "~a" type)))
    
    (cond
     ((eq type :dataset)
      (setf count (incf (gethash key *clasp-dataset-count* -1))))
     ((eq type :variable)
      (setf count (incf (gethash key (dataset-variable-count dataset) -1))))
     ((eq type :data)
      (setf count (incf (gethash key *clasp-data-count* -1))))
     (t (error 'invalid-data-type)))
    (make-unique-string prefix count)))

;;; ***************************************************************************
;;; Special clasp variables
;;; ***************************************************************************

;;; The preferences variables contain options which the user can adjust which
;;; effect the running of the program and allow user customization

(defvar *clasp-preferences* nil
  "Holds settings by the user which control global behavior of the program")

(defvar *preferences-activate-automatically* t
  "Should newly loaded/created datasets be automatically added to the *clasp-active-datasets* list.")

(defun get-preferences (preference-name &optional (preferences *clasp-preferences*))
  "Get a preference value from a preference list"
  (assoc preference-name preferences))

(defun add-preference (preference-name preference-value &optional (preference-list *clasp-preferences*))
  (nconc preference-list (acons preference-name preference-value nil)))

(defun remove-preference (preference preference-list)
  (delete preference preference-list :key #'car))

;;; The variables *clasp-datasets* and *clasp-active-datasets* contain datasets
;;; which are currently extant within a clasp session.  *clasp-datasets*
;;; contains all datasets which have been loaded or created and not
;;; subsequently removed.  *clasp-active-datasets* contains a subset of
;;; *clasp-datasets* which are the choices presented the user when a dataset
;;; selection must occur.  (This is basically for the convenience of the user.
;;; It allows many datasets to be open, without cluttering up the menus.)

(defvar *clasp-datasets* nil
  "A list of the datasets available in the current session of clasp")
(defvar *clasp-active-datasets* nil
  "A list of the datasets which are currently active in clasp")

(defvar *clasp-version* nil)

(defvar *clasp-package* (find-package 'clasp))

;;; ***************************************************************************
;;; EOF
