#|

patch to add standardize variable to the MacCLASP release 1.3.3

09-03-94  ms Created.

FIX:: place in clasp/shortcuts.lisp

|#

(in-package "CLASP")

;;;
;;; standardize variable
;;;

;;; THESE 2 SHOULD GO IN SHORTCUTS.LISP

(defun standardize-variable (variable)
  (let* ((data (variable-value variable))
         (m (mean data))
         (sd (standard-deviation data)))
    (add-variable-to-dataset
     (variable-dataset variable)
     (mapcar #'(lambda (number) (/ (- number m) sd))
             data)
     (format nil "Standardized ~a" (name variable)))))

(defun mul-by-constant (variable constant)
  (add-variable-to-dataset
   (variable-dataset variable)
   (mapcar #'(lambda (datum) (* datum constant))
           (variable-value variable))
   (format nil "* ~F ~a" constant (name variable))))


;;;
;;; correlation table
;;;

;;; MAYBE THIS SHOULD GO IN STATISTICS.LISP ??

(defun make-correlation-table (&rest number-lists)
  (let (r-list ir-list)
    (dolist (ox number-lists)
      (setf ir-list '())
      (dolist (ix number-lists)
        (push (r-score ox ix) ir-list))
      (push ir-list r-list))
    (values r-list)))

;;
;; more sampling functions for bootstrapping & randomization
;;

(defun sample-no-replace-from-joint-sequence (seq1 seq2)
  (let ((sample (sample-without-replacement-from-sequence (append seq1 seq2))))
    (values (subseq sample 0 (length seq1))
            (subseq sample (length seq1)))))

#+TEST
(sample-no-replace-from-joint-sequence '(1 2 3 4 5) '(10 20 30 40 50))

(defun swap-random-pairs (seq1 seq2 &key (p 0.5))
  (let ((s1 (copy-list seq1))
        (s2 (copy-list seq2)))
    (dotimes (i (min (length s1) (length s2)))
      (if (< (random 1.0) p)
        (let ((tt (elt s1 i)))
          (setf (elt s1 i) (elt s2 i))
          (setf (elt s2 i) tt))))
;    (prin1 s1) (fresh-line) (prin1 s2) (fresh-line)
    (values s1 s2)))

#+TEST
(swap-random-pairs '(1 2 3 4 5) '(10 20 30 40 50))

(defun sample-with-replace-from-joint-sequence (seq1 seq2)
  (let ((sample (sample-with-replacement-from-sequence (append seq1 seq2))))
    (values (subseq sample 0 (length seq1))
            (subseq sample (length seq1)))))

#+TEST
(sample-with-replace-from-joint-sequence '(1 2 3 4 5) '(10 20 30 40 50))

;;
;; general-bootstrapping
;;
;; compute-bootstrap is a more generalized boot function which can handle multivariate 
;; bootstraps. generalization to multivariate expressions comes at a price, however.
;; the keyword arguments are pretty much required -- sampling-function arguments at least.
;; the way these arguments work is that they are mapped lists of functions and the arguments.
;; so, you might say :sampling-functions '(sample-with-replacement sample-with-replacement)
;;                   :sampling-function-arguments '((( <data1> )) (( <data2>)))
;; and the function would map each sample-with.. to the corresponding arg list (list of single data list)
;;


(defun compute-bootstrap (boot-function &rest boot-function-arguments &key 
		                           (sampling-functions #'clasp::sample-with-replacement-from-sequence)
		                           sampling-function-arguments
		                           (number-of-iterations 500))
  (remf boot-function-arguments :sampling-functions)
  (remf boot-function-arguments :sampling-function-arguments)
  (remf boot-function-arguments :number-of-iterations)
  (let (temp)
    (loop for i from 1 to number-of-iterations
          collect (apply boot-function
                         (car (setq temp (mapcar #'(lambda (function arguments)
                                                     (apply function (car arguments) (cdr arguments)))
                                                 sampling-functions
                                                 sampling-function-arguments)))
                         (append (cdr temp)
                                 boot-function-arguments)))))

(defun compute-bootstrap-special (boot-function sample-1 sample-2 resamples &key (mode :joint-with-replace))
  (loop for i from 1 to resamples
        collect (let ((funcall-args 
                       (ecase mode
                         (:joint-with-replace (multiple-value-list
                                               (sample-with-replace-from-joint-sequence sample-1 sample-2)))
                         (:joint-without-replace (multiple-value-list
                                                  (sample-no-replace-from-joint-sequence sample-1 sample-2)))
                         (:swap-in-place (multiple-value-list
                                          (swap-random-pairs sample-1 sample-2)))
                         (:disjoint-with-replace (list (sample-with-replacement-from-sequence sample-1)
                                                       (sample-with-replacement-from-sequence sample-2)))
                         (:disjoint-without-replace (list (sample-without-replacement-from-sequence sample-1)
                                                          (sample-without-replacement-from-sequence sample-2))))))
                  (apply boot-function funcall-args))))

#+TEST
(compute-bootstrap-special #'correlation '(1 2 3 4) '(10 20 30 40) 10 :mode :joint-without-replace)

;; two variables to bootstrap
#+TEST
(compute-bootstrap #'clasp::correlation 
                   :sampling-functions (list #'clasp::sample-without-replacement-from-sequence
                                             #'clasp::sample-without-replacement-from-sequence)
                   :sampling-function-arguments '(((1 2 3 4 5)) ((10 20 30 40 50)))
                   :number-of-iterations 5)
;; one variable to bootstrap
#+TEST
(compute-bootstrap #'clasp::mean
                   :sampling-functions (list 'clasp::sample-without-replacement-from-sequence)
                   :sampling-function-arguments '(((1 2 3 4 5 6)))
                   :number-of-iterations 5)
