;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/translation-fns.lisp *-*
;;;; *-* Last-Edit: Friday, November 4, 1994  12:16:34; Edited-By: westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                          Translation Functions                         *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  06-30-92 File Created.  (Carlson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp)

;;; --*--
;;; ***************************************************************************


;;; ***************************************************************************
;;; dataset translation functions
;;; ***************************************************************************

(defmethod dataset-to-columns (data &key #+OLD-VARS rows columns where where-variables)
  "Takes a dataset `data' and extracts tuples representing the columns of data
in the dataset.  `rows' is a where-clause selecting which rows of the dataset
to select. `columns' is a list of variable names to return.  By default, all
rows and all columns are returned."
  (transpose (dataset-to-rows (get-dataset data) :columns columns :where where :where-variables where-variables)))

(defmethod dataset-to-rows (dataset &key columns where where-variables &aux result)
  "Takes a dataset `data' and extracts tuples representing the rows of
data in the dataset.  `rows' is a where-clause selecting which rows of
the dataset to select. `columns' is a list of variable names to
return.  By default, all rows and all columns are returned."
  (setf dataset (get-dataset dataset))
  (setf columns (mapcar #'(lambda (c) (get-variable c dataset))
			(or columns (dataset-variables dataset))))
  (selecting-rows (row dataset columns (or where #'(lambda () t)) where-variables)
		  (push row result))
  (nreverse result))

#+OLD-VARS
(defun dataset-to-rows (data &key rows columns)
  "Takes a dataset `data' and extracts tuples representing the rows of
data in the dataset.  `rows' is a where-clause selecting which rows of
the dataset to select. `columns' is a list of variable names to
return.  By default, all rows and all columns are returned."
  (setf data (get-dataset data))
  (when columns
    (setf columns (mapcar #'(lambda (x)
			      (car (variable-attribute
				    (get-variable x 
						  (get-dataset data)))))
			  columns)))
  (unless columns
    (setf columns (mapcar #'(lambda (x)
			      (car (variable-attribute x)))
			  (dataset-variables data))))
  (reverse
   (list-selection :attrs columns
		   :from (dataset-read-data data)
		   :where (process-where-clause rows data))))

;;; ***************************************************************************
;;; array & matrix translation functions
;;; ***************************************************************************


;;; ***************************************************************************
;;; other translation functions
;;; ***************************************************************************

(defun transpose (data)
  "Takes a list of lists, `data', and makes a list of tuples t where t[i]
contains all the i'th elements of the lists in `data'."
  (if data
      #+(or EXPLORER Lispworks)
      (let ((retval (make-list (length (car data)) :initial-element '())))
	(dolist (datum (reverse data))
	  (dotimes (n (length datum))
	    (push (nth n datum) (nth n retval))))
	retval)
      #-(or EXPLORER Lispworks)
      (apply #'mapcar #'list data)
      nil))

;;; ***************************************************************************
;;; EOF
