;;;; -*- Mode: Common-Lisp; Package: CLASP; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/variable-value.lisp *-*
;;;; *-* Last-Edit: Tuesday, November 29, 1994  16:48:18; Edited-By: westy *-*
;;;; *-* Machine: Lindy (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                             VARIABLE VALUE                             *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  08-30-94 File Created.  (Westy)
;;;  11-14-94 Made `order-by' argument to `variable-value' handle sorting of
;;;           categorical data. (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP)

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(defvar *variable-caching-status* :DEFERRED)

(defun set-variable-caching-status (new-status)
  (check-type new-status (member :ENABLED :DEFERRED :DISABLED))
  (setf *variable-caching-status* new-status))

;; Returns the value of `self'.  If included, `order-by' is the name of
;; another variable which `self' should be sorted by, and `where' is a
;; predicate accepting `where-variables' for selecting a subset of the variable.
;; If distinct is non-nil then duplicate values are removed."
(defmethod variable-value ((self t) &key (order-by 'row-number) where distinct where-args)
  (variable-value (get-variable self) :order-by order-by :where where :distinct distinct :where-args where-args))

(defmethod variable-value ((self cons) &key (order-by 'row-number) where distinct where-args)
  (variable-value (get-variable (first self) (second self)) :order-by order-by :where where
                  :distinct distinct :where-args where-args))

(defmethod variable-value ((self row-number-variable)
                           &key (order-by 'row-number)
			   where
			   distinct where-args)
  (declare (ignore where-args))
  (cond ((and (eq order-by 'row-number)
              (or (null where) (numberp where))
	      (null distinct))
         (cond ((numberp where)
                (unless (slot-value self 'indirect-array)
                  (update-variable-cache self))
	        (aref (slot-value self 'indirect-array) where))
               (t
	        (slot-value self 'value))))
        (t
         (variable-value-internal self order-by where where-args distinct))))

(defmethod variable-value ((self variable) &key
			   (order-by 'row-number)
			   where
			   distinct where-args)
  ;; Cache if necessary:
  ;; If the caches are empty and this is a simple, by row-number access
  ;; then we update the cache.
  (when (and  (not (eq *variable-caching-status* :DISABLED))
              (null (slot-value self 'order-by-array-cache))
	      (eq order-by 'row-number)
	      (or (null where) (numberp where))
	      (null distinct))
    
    (update-variable-cache self))
  
  ;; Get the value:
  (if (and (eq order-by 'row-number)
	   (or (null where) (numberp where))
	   (null distinct))
      
      ;; Simple access by row-number
      (cond ((numberp where)
             (if (eq *variable-caching-status* :DISABLED)
                 (elt (slot-value self 'value) where)
                 (aref (slot-value self 'order-by-array-cache) where))
             ;; If we use this style we do not have to update the array cache
             #+LATER
             (aref (slot-value self 'order-by-array-cache)
                   (aref (slot-value
		          ;; replace this with a slot in the dataset?  
                          (row-number-variable (variable-dataset self)) 'indirect-array)
                         where)))
            (t
             (if (eq *variable-caching-status* :DISABLED)
                 (slot-value self 'value)
	         (slot-value self 'order-by-list-cache))))
      
      ;; Standard case of a complex access
      (variable-value-internal self order-by where where-args distinct)))

(defmethod update-variable-cache ((variable variable))
  (assert (not (eq *variable-caching-status* :DISABLED)) () "Not allowed when caching is disabled.")
  
  (setf (slot-value variable 'order-by-array-cache)
	(make-array (dataset-rows (variable-dataset variable)) 
		    :element-type t
		    :initial-contents 
		    (setf (slot-value variable 'order-by-list-cache)
			  (variable-value-internal variable 'row-number nil nil nil)))))

(defmethod update-variable-cache ((self row-number-variable))
  
  (setf (slot-value self 'indirect-array)
        (make-indirect-array
	 (slot-value self 'value) #'<
	 (sorted-p self))))


(defun variable-value-internal  (self
				 order-by
				 where where-args
				 distinct &aux predicate)
  ;; Parse the order by arg into a list that `select-tuples' can use to order the results.
  ;; Note: We do not need to do this for row-number variables since they are by definition in row-number order.
  (if (and order-by (not (row-number-p self)))
      (let* ((dataset (variable-dataset self))
	     (order-by-variable (get-variable order-by dataset))
             (row-number-variable (row-number-variable dataset)))
        (cond
         ;; Check this special case for ordering by the row number variable
         ((and (eq order-by-variable row-number-variable) (sorted-p row-number-variable))
	  (setf order-by (slot-value order-by-variable 'value)
		predicate :SORTED))
              (t
               (setf order-by
		     (variable-value-internal order-by-variable 'row-number nil nil nil)
                     predicate (ecase (slot-value order-by-variable 'type)
                                 (:categorical #'alphalessp)
                                 ((:numerical :ordinal) #'<))))))
      ;; otherwise...
      (setf order-by nil predicate nil))
  
  (select-tuples (variable-dataset self) (slot-value self 'value) order-by predicate distinct where where-args))

;;; Helper functions for `variable-value-internal' that could probably be made into internal functions.

(defun select-tuples (dataset tuple-set order-by predicate distinct where where-args)
  ;; `tuples' and `order-by' are both a sequence of numbers. 
  (let ((tuple-set
         (if order-by
             (if where 
                 (handle-where dataset tuple-set
			       where where-args
                               (sort-using-sequence-order tuple-set order-by
							  predicate
							  t))
               (sort-using-sequence-order tuple-set order-by
                                          predicate))
           (if where
	       (handle-where dataset tuple-set where where-args nil) 
             (values tuple-set)))))
    (if distinct
        (extract-unique-values tuple-set)
      (values tuple-set))))

(defmacro map-rows ((row values dataset variables &optional indirect-array) &body body)
  "Map over the rows of `dataset' binding `row' to the row number and `values' to the values
of the `variables' in that row."
  `(loop with variables = (mapcar #'(lambda (v) (get-variable v ,dataset)) ,variables)
    for cnt from 0 to (1- (if ,indirect-array (length ,indirect-array) (dataset-rows ,dataset)))
         for ,row = (if ,indirect-array (aref ,indirect-array cnt) cnt) do
         (let ((,values
                (loop for var in variables
                      collect
		      (variable-value var :where ,row))))
           (progn . ,body))))
           
(defun handle-where (dataset tuples where where-args indirect-array)
  (if (typep where 'number)
      ;; Special case for row numbers
      (elt tuples (if indirect-array (aref indirect-array where) where))
    (let (result)
      (map-rows (row-number variable-values dataset where-args indirect-array)
        (when (apply where variable-values)
	  (push (elt tuples row-number)
                result)))
      (nreverse result))))

(defun make-indirect-array (y predicate &optional presorted)
  (let ((length (length y)))
    ;; copy y into an array 
    (with-temp-vector (tmp length)
      (replace tmp y)
      (flet ((make-indirect-array-helper (length)
          (let ((array (make-array length :element-type 'fixnum)))
	    (dotimes (cnt length)
	      (setf (aref array cnt) cnt))
	    (values array))))
     (let* ((array (make-indirect-array-helper length)))
       (values (if presorted
                   array
                   (sort array predicate :key #'(lambda (el)
	                                      (aref tmp el))))))))))

(defun sort-using-index-array (x array)
  (map (etypecase x (array 'array) (list 'list)) #'(lambda (index) (elt x index)) array))

;; We want to augment this so that we can reconstruct the orignal order of a list after we
;; have removed an element. (maybe mark the missing elements with indices of -1).
(defun unsort-using-index-array (x array)
  (let ((result (etypecase x
                  (array (make-array (length x)))
                  (list (make-list (length x))))))
    (map nil #'(lambda (elt index)
                 (setf (elt result index) elt))
         x array)
    (values result)))

(defun sort-using-sequence-order (x y predicate &optional return-indirect)
  (if (and (not return-indirect) (eq predicate :SORTED))
      x
      (let* ((array (make-indirect-array y predicate (eq predicate :SORTED))))
        (if return-indirect
            array
            (sort-using-index-array x array)))))
  
(defmethod add-data-point ((self variable) new-data push-on-front)
  (if push-on-front
      (push new-data (slot-value self 'value))
      ;; This is very time inefficient if we are dealing with large datasets.
      (setf (slot-value self 'value)
	    (nconc (slot-value self 'value) (list new-data))))
  ;; For now we just deal with :numerical and :categorical.
  (unless (numberp new-data)
    (setf (slot-value self 'type) :categorical)))

#+BROKEN
(defmethod delete-data-point ((self variable) row-number)
  (assert (not (eq *variable-caching-status* :DISABLED)) () "Not allowed when caching is disabled.")
  (with-slots (order-by-list-cache value) self
    ;; We need things sorted by row number.
    (unless order-by-list-cache (update-variable-cache self))
    (if (zerop row-number)
        (setf order-by-list-cache (rest order-by-list-cache))
        (setf (rest (nthcdr (1- row-number) order-by-list-cache))
              (rest (nthcdr (- row-number 2) order-by-list-cache))))
    ;; THIS WILL NOT WORK IF ROWS ARE NOT SORTED BY ROW NUMBER ORDER. If this is
    ;; the case we must resort the value by the ordering of of the row number.
    (setf value order-by-list-cache)
    (clear-cache (variable-dataset self) self)))

;; I really want atom here...
(defmethod (setf variable-value) (new-value (self row-number-variable)
                                            &key
				            (order-by 'row-number)
				            where
				            distinct)
  (declare (ignore order-by where distinct))
  
  (error "Setting a single row of a row-number-variable is not supported."))

;; I really want atom here...
(defmethod (setf variable-value) (new-value (self variable) &key
				  (order-by 'row-number)
				  where
				  distinct)

  (assert (not (eq *variable-caching-status* :DISABLED)) () "Not allowed when caching is disabled.")
  (assert (numberp where))
  (assert (eq order-by 'row-number))
  (assert (null distinct))

  (with-slots (order-by-list-cache order-by-array-cache value) self
    ;; update the caches since we need to have acorrect ordering anyway
    (unless (and order-by-list-cache order-by-array-cache)
      (update-variable-cache self))

    (setf (aref order-by-array-cache where) new-value)
    (setf (nth where order-by-list-cache) new-value)
    ;; Since the value is by definition in any order this one will be as good as any, NOT!!!!!!!
    ;; THIS WILL NOT WORK IF ROWS ARE NOT SORTED BY ROW NUMBER ORDER. If this is
    ;; the case we must resort the value by the ordering of of the row number.
    (setf value order-by-list-cache)))

;; No error checking is done here so you can totally puck ober your dataset
;; by changing the value of the row-number variable.
(defmethod (setf variable-value) ((list cons) (self row-number-variable) &key
				  (order-by 'row-number)
				  where
				  distinct)

  (assert (null where))
  (assert (eq order-by 'row-number))
  (assert (null distinct))

  ;; Change the 'VALUE slot
  (setf (slot-value self 'value) list)
  (setf (slot-value self 'indirect-array) nil)
  (when (eq *variable-caching-status* :ENABLED)
     (update-variable-cache self))
  )

(defmethod (setf variable-value) ((value (eql nil)) (self variable)
				  &key
				  (order-by 'row-number)
				  where
				  distinct)
  
  (assert (eq order-by 'row-number))
  (assert (null distinct))

  (cond ((numberp where)
         (call-next-method))
        (t
	 (assert (null where))
         (setf (slot-value self 'value) nil)
         (setf (slot-value self 'order-by-list-cache) nil)))

  ;; Reset the cache slots, but do not worry about updating the cache.
  ;; Note: We still reset these even if caching is disabled in case they reenable it.
  (setf (slot-value self 'order-by-list-cache) nil)
  (setf (slot-value self 'order-by-array-cache) nil))
  
(defmethod (setf variable-value) ((list cons) (self variable) &key
				  (order-by 'row-number)
				  where
				  distinct)
  
  (assert (null where))
  (assert (eq order-by 'row-number))
  (assert (null distinct))

  ;; Change the 'VALUE slot
  (setf (slot-value self 'value) list)
  ;; Change the `row-number' slot and the row-number variable of the dataset if necessary.
  (update-row-number-for-changed-variable-value (variable-dataset self) (length list))

  ;; Note: We still reset these even if caching is disabled in case they reenable it.
  (setf (slot-value self 'order-by-list-cache) nil)
  (setf (slot-value self 'order-by-array-cache) nil)

  (when (eq *variable-caching-status* :ENABLED)
    (update-variable-cache self)))

;; Could actually make this an after method on updating the datasets rows variable
;; Need to think about error checking for differing numbers of rows.
(defmethod update-row-number-for-changed-variable-value (dataset new-length)
  "Change the `row-number' slot and the row-number variable of the dataset if necessary."
  (let* ((row-number-variable (get-variable 'row-number dataset))
         (old-length (data-length-internal (variable-value row-number-variable))))
    
    (when (> new-length old-length)
      (setf (dataset-rows dataset) new-length)
      (setf (variable-value row-number-variable) 
            (loop for cnt from 0 to (1- new-length) collect cnt)))))

(defmethod distinct-values ((self variable) &rest args)
  "Returns all the distinct values of `variable'."
  (apply #'variable-value self :distinct t args))

(defun clear-cache (dataset &optional variable[s])
  "Clears the variable-value caches of `variable[s]' (defaults to all variables in `dataset').
Should be called whenever a new value or values are added to a variable."
  ;; We still reset these even if caching is disabled in case they reenable it.
  (unless variable[s]
    (setf variable[s] (dataset-variables dataset)))
  (flet ((do-it (variable)
  	   (setf (slot-value variable 'order-by-list-cache) nil)
           (setf (slot-value variable 'order-by-array-cache) nil)))
  (if (consp variable[s])
      (dolist (variable variable[s])
        (do-it variable))
      (do-it variable[s])))
  #+UNUSED
  (setf (slot-value dataset 'by-row-cache) nil))

;;; ***************************************************************************
;;; *                              End of File                                *
;;; ***************************************************************************