;;; process-switch-and-paste-play.lisp

#|
20-Aug-94 mc	Created. Works! You must wait for the process to
		 activate, I guess!

To Do:

Catch errors, such as

BUG: If there is no document to paste to we get this error:
 > Error: MacWrite Pro Demo got an error: No such document
 > While executing: #<STANDARD-METHOD ASTOOLS:EXECUTE-APPLESCRIPT (ASTOOLS:APPLESCRIPT-OBJECT)>

and

BUG: If there is no process named *str-name-log-application* we get:
 > Error: value NIL is not of the expected type NUMBER.
 > While executing: LOG-STR-OR-H-PICT


|#

(in-package "CLASP-INTERFACE")


#| ; These are all loaded elsewhere now
(require "APPLEEVENT-TOOLKIT")
(require "APPLESCRIPT-LOADER"
         "ccl:applescript/appleevents;applescript-loader.lisp")
(require "PICT-SCRAP")
|#

;;;

(defvar *str-name-log-application* nil
  "The name of the application process that the user has selected for
logging textual and graphical CLASP output. Initially nil until
choose-and-set-log-application is evaluated.")


(defvar *ao-tell-log-app-paste* nil
  "An as:applescript-object that tells *str-name-log-application* to
paste. Initially nil until choose-and-set-log-application is evaluated.")


;;;

(defun str-choose-process-name ()
  "Returns a string naming a process the user selects from the PPC
chooser dialog."
  ;;
  (rlet ((p-portInfoRec :PortInfoRec)
         (p-TargetID :TargetID))
    (AS::choose-process-dialog :location-name-record p-TargetID
                           :port-info-record p-portInfoRec)
    (let* ((p-ppcportrec (pref p-portInfoRec :PortInfoRec.name))
           (str-name (pref p-ppcportrec :PPCPortRec.name)))
      str-name)))


(defun choose-and-set-log-application ()
  "Lets the user choose an active process, sets
*str-name-log-application* to its name, and sets
*ao-tell-log-app-paste* ."
  ;;
  (let* ((str-process-name (str-choose-process-name))
         str-script)
    ;; Check that str-process-name isn't nil or "" ?
    (setf *str-name-log-application* str-process-name
          ;; FIX: Need 	activate before paste in following script?
          str-script (format nil "
tell application ~S
	Paste
end tell"
                             str-process-name)
          *ao-tell-log-app-paste* (make-instance 'as::applescript-object
                                    :script str-script
                                    :application-name str-process-name))))


(defmethod set&wait-for-psn ((p-psn macptr))
  "Calls #_SetFrontProcess on p-psn then loops until #_GetFrontProcess
returns values equal to those in p-psn, which is a pointer to a
:ProcessSerialNumber."
  ;;
  (#_SetFrontProcess p-psn)
  (rlet ((p-psn-front :ProcessSerialNumber))
    (loop with int-psn-high-target = (rref p-psn :ProcessSerialNumber.highLongOfPSN)
          with int-psn-low-target = (rref p-psn :ProcessSerialNumber.lowLongOfPSN)
          do (#_GetFrontProcess p-psn-front)
          until (and (= (rref p-psn-front :ProcessSerialNumber.highLongOfPSN)
                        int-psn-high-target)
                     (= (rref p-psn-front :ProcessSerialNumber.lowLongOfPSN)
                        int-psn-low-target)))))


(defmethod h-copy-h ((h-source macptr))         ;this works
  "Copies the data in h-source into a new handle which is returned.
The returned handle must be deallocated via #_DisposHandle when done."
  ;;
  ;; Use #_BlockMove because I can't get #_HandToHand to work.
  ;;
  (let* ((int-size (#_GetHandleSize h-source))
         (h-dest (#_NewHandle int-size)))
    (with-dereferenced-handles ((p-source h-source)
                                (p-dest h-dest))
      (#_BlockMove p-source p-dest int-size))
    h-dest))


(defun copy-str-or-h-pict (str-or-h-pict)
  ;;
  (let* ((kw-scrap-type (etypecase str-or-h-pict
                          (string :TEXT)
                          (macptr :PICT)))
         (data (case kw-scrap-type
                 (:TEXT str-or-h-pict)
                 (:PICT (h-copy-h str-or-h-pict)))))
    ;; FIX: For macptrs verify they point to PICTs.
    (ccl::put-scrap kw-scrap-type data)         ;FIX: Should lock and unlock str-or-h-pict if it's a macptr
    (ccl::externalize-scrap (cdr (assoc kw-scrap-type *scrap-handler-alist*)))))


(defun log-str-or-h-pict (str-or-h-pict)
  "Copies str-or-h-pict to the desk scrap, switches to the process
identified by *str-name-log-application*, runs *ao-tell-log-app-paste*,
and switches back. Str-or-h-pict is a string or a handle to a PICT.
Errors if *str-name-log-application* and *ao-tell-log-app-paste* haven't
been set via choose-and-set-log-application."
  ;;
  (unless (and *str-name-log-application*
               *ao-tell-log-app-paste*)
    (error "Must call choose-and-set-log-application first"))
  ;;
  (rlet ((p-psn-mcl :ProcessSerialNumber))
    (#_GetFrontProcess p-psn-mcl)       ;for switching back
    (multiple-value-bind (int-psn-high-log-app int-psn-low-log-app)
                         (find-named-process *str-name-log-application*)
      (rlet ((p-psn-chosen :ProcessSerialNumber
                           :highLongOfPSN int-psn-high-log-app
                           :lowLongOfPSN int-psn-low-log-app))
        ;; Copy the data.
        (copy-str-or-h-pict str-or-h-pict)
        ;; Switch to the chosen process.
        (set&wait-for-psn p-psn-chosen)
        ;; Run the paste script.
        (as::execute-applescript *ao-tell-log-app-paste*)
        ;; Switch back to mcl.
        (set&wait-for-psn p-psn-mcl)))))


#|

(choose-and-set-log-application)

(log-str-or-h-pict "Test!")

(loop for int-count from 1 to 3
      for string = (format nil "~A~%" int-count)
      do (log-str-or-h-pict string))

(log-str-or-h-pict (#_GetIndResource :PICT 4))

(loop for string-or-h-pict in (list "This PICT brought to you by MCL!
"
                                    (#_GetIndResource :PICT 4)
                                    "
This one is nice, too:
"
                                    (#_GetIndResource :PICT 6))
      do (log-str-or-h-pict string-or-h-pict))

;;; Log all PICTs, printing a header before each:
(loop for int-pict-num from 1 to (#_CountResources :PICT)
      initially
      (log-str-or-h-pict (format nil "Logging ~A PICTs~%" (#_CountResources :PICT)))
      do
      (log-str-or-h-pict (format nil "~%PICT #~A:~%" int-pict-num))
      (log-str-or-h-pict (#_GetIndResource :PICT int-pict-num)))

|#