(in-package :clasp-interface)

;; symbologies stuff

;; here are some common colors and a menu class to diddle with them
(defvar *color-table* #+mcl(list (list "Yellow" *yellow-color*)
                                    (list "Gray" *gray-color*)
                                    (list "Tan" *tan-color*)
                                    (list "Orange" *orange-color*)
                                    (list "Black" *black-color*)
                                    (list "Purple" *purple-color*)
                                    (list "Pink" *pink-color*)
                                    (list "Brown" *brown-color*)
                                    (list "Blue" *blue-color*)
                                    (list "Red" *red-color*)
                                    (list "Green" *green-color*)
                                    ))

(defclass colors-pum (pop-up-menu)
  ()
  (:DOCUMENTATION "A pop up menu with several common choices for colors."))

(defmethod initialize-instance :after ((color-pum colors-pum) &key)
  "puts the colors into the pum"
  (add-menu-items color-pum (make-instance 'menu-item :menu-item-title "Custom"))
  (add-menu-items color-pum (make-instance 'menu-item :menu-item-title "-"
                                           :disabled t))
  (mapcar #'(lambda (item) (add-menu-items color-pum
                                           (make-instance 'menu-item 
                                             :menu-item-title (car item))))
             *color-table*))

(defmethod view-click-event-handler :after ((menu colors-pum) (pt-where fixnum))
  (if (string= (menu-item-title (selected-item menu)) "Custom")
    (dialog-item-enable (view-named 'picker (view-container menu)))
    (dialog-item-disable (view-named 'picker (view-container menu))))
  (set-rgb-items (view-container menu)))

(defun color-value-by-name (name)
  (let ((result 0))
    (loop for x in *color-table* do
          (if (equal (car x) name)
            (setq result (cadr x))))
    result))

;;; next is the plot-list item

(defvar *plot-list-font* '("Geneva" 9))

(defclass plot-list (sequence-dialog-item)
  ()
  (:documentation "A list of plots within a graph.")
  (:default-initargs
    :view-font *plot-list-font*
    :table-hscrollp nil :table-vscrollp t
    :cell-size #@(320 10)
    :table-print-function
    #'(lambda (plot stream)
        (declare (ignore stream))
        (let ((string (format nil "~:(~A~) : ~:(~A~)"
                              (caar plot) (cadar plot))))
          (multiple-value-bind (ff ms) (font-codes *plot-list-font*)
            (with-font-codes ff ms
              (with-pstrs ((p-str string))
                (#_DrawString p-str))))))))

(defmethod view-click-event-handler :after ((view plot-list) (pt-where fixnum))
  (let ((clicked-item (ignore-errors (elt (table-sequence view) 
                                          (cell-to-index view 0 (point-v (point-to-cell view pt-where)))))))
    #+IGNORE (if clicked-item 
      (ribbon-display (name-and-dataset clicked-item)))
    (set-ge-views (view-container view) clicked-item)))

;;; here is the actual class of graph editors

(defclass graph-editor (window)
  ((graph
    :initform nil
    :initarg :graph
    :type graph
    :accessor graph-editor-graph
    :documentation "The graph that is being operated on."))
  (:DOCUMENTATION "The window class of dialog boxes to edit graphs.")
  (:DEFAULT-INITARGS
    :window-type :document
    :window-show nil
    :VIEW-POSITION (best-spot 320 200)))

(defmethod initialize-instance :after ((graph-editor graph-editor) &key)
  ;; Get the subviews and the size which depend on the graph we are editing.
  (with-slots (graph) graph-editor
    ;; Compute the size first so we can use it to figure out where some of the subviews go.
    (set-view-size graph-editor (graph-edit-compute-size graph))
    (apply #'add-subviews graph-editor (graph-edit-compute-subviews graph)))
  ;; set up the x and y bound stuff
  (let ((xbc (view-named 'use-x-bounds graph-editor))
        (xmindi (view-named 'x-min graph-editor))
        (xmaxdi (view-named 'x-max graph-editor))
        (ybc (view-named 'use-y-bounds graph-editor))
        (ymindi (view-named 'y-min graph-editor))
        (ymaxdi (view-named 'y-max graph-editor))
        (graph (graph-editor-graph graph-editor)))
    (multiple-value-bind (xmin xmax xcp ymin ymax ycp)
                         (xy-bounds graph)
      (if xcp (progn (dialog-item-enable xmindi) (dialog-item-enable xmaxdi) 
                     (check-box-check xbc)))
      (if ycp (progn (dialog-item-enable ymindi) (dialog-item-enable ymaxdi) 
                     (check-box-check ybc)))
      (set-dialog-item-text xmindi (if xmin
                                     (format nil "~S" xmin)
                                     (dialog-item-disable xbc)))
      (set-dialog-item-text xmaxdi (if xmax
                                     (format nil "~S" xmax)
                                     (dialog-item-disable xbc)))
      (set-dialog-item-text ymindi (if ymin
                                     (format nil "~S" ymin)
                                     (dialog-item-disable ybc)))
      (set-dialog-item-text ymaxdi (if ymax
                                     (format nil "~S" ymax)
                                     (dialog-item-disable ybc)))))
  ;; set up the string crap
  (let ((dp (view-named 'data-points graph-editor))
        (pt (view-named 'plot-type graph-editor))
        (graph (graph-editor-graph graph-editor)))
        ;; set up string tags
    (set-window-title graph-editor (concatenate 'string (clasp::name graph) " Settings"))
    (set-dialog-item-text dp (format nil "~D data point~:P" (length (cdar (variable-plot-data
                                                                         graph)))))
    (set-dialog-item-text pt (format nil "~:(~A~)" (caaar (variable-plot-data graph)))))
  (window-show graph-editor))

(defmethod graph-edit-compute-subviews ((graph graph))
  (let* ((size (graph-edit-compute-size graph))
         (height (point-v size)))
    (list (make-instance 'plot-list :view-nick-name 'plots
                         :view-position #@(0 65) :view-size #@(320 57)
                         :table-sequence (variable-plot-data graph))
          (make-instance 'static-text-dialog-item :view-position #@(110 43)
                         :dialog-item-text (format nil "~D Plot Group~:p" (length (variable-plot-data graph))))
          (make-instance 'static-text-dialog-item
            :view-nick-name 'data-points :dialog-item-enabled-p NIL
            :view-position #@(165 123)
            :view-size #@(140 15))
          (make-instance 'static-text-dialog-item
            :view-nick-name 'plot-type :dialog-item-enabled-p NIL
            :view-position #@(10 123)
            :view-size #@(140 15))
          ;; color stuff
          (make-instance 'colors-pum #+CCL-3 :enabledp #-CCL-3 :dialog-item-enabled-p NIL
            :view-nick-name 'color-menu
            :view-position #@(90 140)
            #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Color")
          (make-instance 'button-dialog-item :dialog-item-text "Pick" :view-position #@(240 142) :view-nick-name 'picker
                         :dialog-item-enabled-p NIL
                         :dialog-item-action #'(lambda (item) (pick-color (view-container item))))
          (make-instance 'static-text-dialog-item :view-nick-name 'red-text
                         :dialog-item-text "Red" :dialog-item-enabled-p NIL
                         :view-position #@(5 170))
          (make-instance 'number-entry-item :view-nick-name 'red-value :dialog-item-enabled-p NIL
                         :allow-negatives nil :allow-floats nil
                         :view-position #@(40 170) :view-size #@(50 15))
          (make-instance 'static-text-dialog-item :view-nick-name 'green-text
                         :dialog-item-text "Green" :dialog-item-enabled-p NIL
                         :view-position #@(108 170))
          (make-instance 'number-entry-item :view-nick-name 'green-value :dialog-item-enabled-p NIL
                         :allow-negatives nil :allow-floats nil
                         :view-position #@(156 170) :view-size #@(50 15))
          (make-instance 'static-text-dialog-item :view-nick-name 'blue-text
                         :dialog-item-text "Blue" :dialog-item-enabled-p NIL
                         :view-position #@(222 170))
          (make-instance 'number-entry-item :view-nick-name 'blue-value :dialog-item-enabled-p NIL
                         :allow-negatives nil :allow-floats nil
                         :view-position #@(260 170) :view-size #@(50 15))
          ;; bounds stuf
          (make-instance 'disabler-checkbox-item :view-nick-name 'use-x-bounds
                         :dialog-item-text "Custom X bounds"
                         :view-position #@(15 1)
                         :check-box-checked-p nil
                         :watches-over '(x-min x-max))
          (make-instance 'number-entry-item :view-nick-name 'x-min
                         :dialog-item-enabled-p nil
                         :view-position #@(160 3) :view-size #@(59 15))
          (make-instance 'number-entry-item :view-nick-name 'x-max
                         :dialog-item-enabled-p nil
                         :view-position #@(240 3) :view-size #@(59 15))
          (make-instance 'disabler-checkbox-item :view-nick-name 'use-y-bounds
                         :dialog-item-text "Custom Y bounds"
                         :view-position #@(15 24)
                         :check-box-checked-p nil
                         :watches-over '(y-min y-max))
          (make-instance 'number-entry-item :view-nick-name 'y-min
                         :dialog-item-enabled-p nil
                         :view-position #@(160 24) :view-size #@(59 15))
          (make-instance 'number-entry-item :view-nick-name 'y-max
                         :dialog-item-enabled-p nil
                         :view-position #@(240 24) :view-size #@(59 15))
          (make-instance 'button-dialog-item :view-position #@(10 45) :dialog-item-enabled-p nil :view-nick-name 'plot-group
                         :dialog-item-text "Plot Group" :dialog-item-action #'(lambda (item)
                                                                          (plot-graph-group (view-container item))))
          (make-instance 'button-dialog-item :view-position #@(230 45) #+IGNORE (make-point 170 (- height 22))
                         :dialog-item-text "Plot Graph" :dialog-item-action #'(lambda (item)
                                                                          (redraw-graph-window (view-container item))))
          (make-instance 'default-button-dialog-item
            :view-nick-name 'change-button 
            :dialog-item-text "Change"
            :view-position (make-point 250 (- height 22))
            :dialog-item-action #'(lambda (item)
                                    (change-graph (view-container item))))
          )))

;;; stuff to tweak the dialog items

(defmethod set-rgb-items ((graph-editor graph-editor))
  (let* ((c-menu (view-named 'color-menu graph-editor))
         (c-picked (menu-item-title (selected-item c-menu)))
         (ri (view-named 'red-value graph-editor))
         (gi (view-named 'green-value graph-editor))
         (bi (view-named 'blue-value graph-editor))
         (rgb-state-set (if (equal c-picked "Custom") 'dialog-item-enable 'dialog-item-disable)))
    (funcall rgb-state-set ri)
    (funcall rgb-state-set gi)
    (funcall rgb-state-set bi)
    (if (equal rgb-state-set 'dialog-item-disable)
      (multiple-value-bind (red green blue)
                           (color-values (color-value-by-name c-picked))
        (progn
          (set-dialog-item-text bi (format nil "~D" blue))
          (set-dialog-item-text gi (format nil "~D" green))
          (set-dialog-item-text ri (format nil "~D" red)))))))

(defvar *need-plots-to-be-active* '(data-points plot-type red-text red-value green-text green-value picker
                                    blue-text blue-value color-menu plot-group))

(defmethod set-ge-views ((view graph-editor) (plot list))
  (let ((dp (view-named 'data-points view))
        (pt (view-named 'plot-type view)))
    ;; set up string tags @@
    (if plot
      (progn
        ;; enable standard items
        (mapcar #'(lambda (symbol) (ignore-errors (dialog-item-enable (view-named symbol view))))
                *need-plots-to-be-active*)
        ;; enable specific (mixin'd) items
        (enable-special-items view (graph-editor-graph view) plot)
        ;; text element
        (set-dialog-item-text dp (format nil "~D data points" (length (cdr plot))))
        (set-dialog-item-text pt (format nil "~:(~A~)" (caar plot)))
        ;; color elements
        (let ((color (plot-color plot))
              (ri (view-named 'red-value view))
              (gi (view-named 'green-value view))
              (bi (view-named 'blue-value view)))
          (multiple-value-bind (red green blue)
                               (color-values color)
            (progn
              (set-dialog-item-text bi (format nil "~D" blue))
              (set-dialog-item-text gi (format nil "~D" green))
              (set-dialog-item-text ri (format nil "~D" red))))))
      (progn
        (mapcar #'(lambda (symbol) (ignore-errors (dialog-item-disable (view-named symbol view))))
                *need-plots-to-be-active*)
        (enable-special-items view (graph-editor-graph view) plot :disable-p t)
        ))))

(defmethod plot-graph-group ((view graph-editor))
  (let* ((plot-list (view-named 'plots view))
         ;; find the individual plot if there is one selected
         (plot (ignore-errors
                (elt (table-sequence plot-list) 
                     (cell-to-index plot-list 0 (point-v (car (selected-cells plot-list)))))))
         (new-plot-graph (make-instance 'graph-group-graph :parent (graph-editor-graph view) :plot-data (list plot)
                                        :graph-keywords (graph-keywords (graph-editor-graph view)))))
    ;; 12-09-94 Slope intercept plots cannot be plotted alone without
    ;; specifying the x and y bounds. (Westy)
    (multiple-value-bind (xmin xmax cx ymin ymax cy)
                         (xy-bounds (graph-editor-graph view))
      (when (eq (first (first plot)) :slope-intercept-line)
        (unless (and cx cy)
          (gui-warn "You need to specify custom X and Y bounds.")
          (return-from plot-graph-group)))
      (open-clasp-object new-plot-graph)
      #+OLD (generate-graph-and-window (list plot) :name (cadar plot)
                                 :x-first (when cx xmin)
                                 :x-last  (when cx xmax)
                                 :y-first (when cy ymin)
                                 :y-last  (when cy ymax)))))


(defmethod pick-color ((view graph-editor))
  (let* ((bi (view-named 'blue-value view))
         (gi (view-named 'green-value view))
         (ri (view-named 'red-value view))
         (color (user-pick-color :color (make-color (number-item-value ri) (number-item-value gi) (number-item-value bi))
                                 :prompt "Select a color for this plot group.")))
    (multiple-value-bind (red green blue)
                           (color-values color)
        (progn
          (set-dialog-item-text bi (format nil "~D" blue))
          (set-dialog-item-text gi (format nil "~D" green))
          (set-dialog-item-text ri (format nil "~D" red))))))

(defmethod redraw-graph-window ((graph-editor graph-editor))
  (let* ((right-graph (find (clasp::name (graph-editor-graph graph-editor))
                            (windows) :test #'(lambda (item1 item2)
                                                     (string-equal item1 (window-title item2))))))
    (if right-graph (ignore-errors (window-close right-graph)))
    (open-clasp-object (graph-editor-graph graph-editor))))

(defmethod change-graph :around ((graph-editor graph-editor))
  (call-next-method)
  (set-table-sequence (view-named 'plots graph-editor)
                      (variable-plot-data (graph-editor-graph graph-editor))))

;; since the graph isn't special, there's nothing to enable
(defmethod enable-special-items ((view graph-editor) (graph graph) (plot list) &key &allow-other-keys)
  ())

(defmethod graph-edit-editor-class ((graph graph))
  'graph-editor)

(defmethod graph-edit-compute-size ((graph graph))
   (values #@(320 220)))

(defmethod make-graph-editor ((graph graph))
  (let ((edit-window (find-if #'(lambda (i2) (equal graph (ignore-errors (graph-editor-graph i2))))
                              (windows))))
    (if edit-window 
      (window-select edit-window)
      (make-instance (graph-edit-editor-class graph) :graph graph))))

;;;
;;; STUFF TO CHANGE THE PLOTS
;;;

;;
;; INDIVIDUAL PLOT GROUPS
;;

(defun install-plot-options (plot options &key (newname nil))
  "takes a SINGLE plot sequence and installs the new options, leaving plot type intact."
  (declare (ignore newname))
  (let* ((header (car plot))
         (old-options (car (last header)))
         (points (cdr plot)))
    (mapcar #'(lambda (option) (if (keywordp option) (remf old-options option)))
            options)
    (cons (list (car header) (cadr header) (append old-options options))
          points)))

(defun plot-color (plot)
  (let* ((header (car plot))
         (options (caddr header))
         (color-at (search (list :color) options))
         (color (if color-at (car (subseq options (1+ color-at))))))
    (if color
      color
      0)))

(defmethod change-graph ((view graph-editor))
  (let* ((plot-list (view-named 'plots view))
         ;; find the individual plot if there is one selected
         (plot (ignore-errors
                (elt (table-sequence plot-list) 
                     (cell-to-index plot-list 0 (point-v (car (selected-cells plot-list)))))))
         (uxb (view-named 'use-x-bounds view))
         (uyb (view-named 'use-y-bounds view))
         (xlb (number-item-value (view-named 'x-min view)))
         (xub (number-item-value (view-named 'x-max view)))
         (ylb (number-item-value (view-named 'y-min view)))
         (yub (number-item-value (view-named 'y-max view)))
         new-keyword-list
         (graph (graph-editor-graph view)))
    (if plot
      (progn
        (setf (elt (variable-plot-data graph) (position plot (variable-plot-data graph) :test #'equal))
              (install-plot-options plot `(:color ,(make-color
                                                    (number-item-value (view-named 'red-value view))
                                                    (number-item-value (view-named 'green-value view))
                                                    (number-item-value (view-named 'blue-value view))))))
        (ignore-errors (setf (dominant-color graph) (make-color
                                                     (number-item-value (view-named 'red-value view))
                                                     (number-item-value (view-named 'green-value view))
                                                     (number-item-value (view-named 'blue-value view)))))
        (set-table-sequence (view-named 'plots view) (variable-plot-data graph))))
    ;; set up the graph's keywords
    (if (check-box-checked-p uxb)
      (progn (push xlb new-keyword-list) (push :x-first new-keyword-list)
             (push xub new-keyword-list) (push :x-last new-keyword-list)))
    (if (check-box-checked-p uyb)
      (progn (push ylb new-keyword-list) (push :y-first new-keyword-list)
             (push yub new-keyword-list) (push :y-last new-keyword-list)))
    (setf (graph-keywords graph)
          (replace-keywords (graph-keywords graph)
                            new-keyword-list
                            *graph-editor-affects-keywords*))))
  
;;
;; KEYWORDS
;;

(defvar *graph-editor-affects-keywords* '(:X-FIRST :X-LAST :Y-FIRST :Y-LAST)
  "The keywords that the graph editor works on.")

(defmethod replace-keywords ((old-keywords LIST) (new-keywords LIST) (replace-keywords LIST))
  "replaces the keywords with the new ones."
  (let ((result '())
        marker)
    (loop for item in old-keywords do
          (if (typep item 'keyword)
            (if (not (or (find item replace-keywords)
                         (find item new-keywords)))
              (progn
                (push (elt old-keywords (setf marker (search (list item) old-keywords))) result)
                (push (elt old-keywords (1+ marker)) result)))))
    (append (reverse result) new-keywords)))

;;
;; EXTENSIONS TO THE GENERIC GRAPH EDITOR DIALOG -- HISTOGRAMS AND SCATTERPLOTS
;;

;;; -------------------------------------------------------------------------------------
;;; Histogram editor

(defclass histogram-editor (graph-editor)
  ()
  )

(defmethod graph-edit-editor-class ((graph histogram))
  'histogram-editor)

(defmethod graph-edit-compute-subviews ((graph histogram))
  ;; In lieu of a real append method combination we do this...
  (let* ((size (graph-edit-compute-size graph))
         (height (point-v size)))
    (append (call-next-method)
            (list (make-instance 'static-text-dialog-item :view-nick-name 'bin-name-tag
                    :dialog-item-text "Number of Bins" :dialog-item-enabled-p nil
                    :view-position (make-point 10 (- height 22)))
                  (make-instance 'number-entry-item 
                    :view-nick-name 'number-of-bins :dialog-item-enabled-p nil
                    :dialog-item-text (princ-to-string (slot-value graph 'number-of-bins))
                    :allow-negatives nil :allow-floats nil
                    :view-position (make-point 120 (- height 22))
                    :view-size #@(50 15))
                  ))))

(defmethod enable-special-items ((graph-editor graph-editor) (histogram histogram) (plot list) &key (disable-p nil))
  (let* ((bnt (view-named 'bin-name-tag graph-editor))
         (nob (view-named 'number-of-bins graph-editor)))
    (if disable-p
      (mapcar #'dialog-item-disable `(,bnt ,nob))
      (progn
        (set-dialog-item-text nob (princ-to-string (slot-value histogram 'number-of-bins)))
        (mapcar #'dialog-item-enable `(,bnt ,nob))))))

(defmethod graph-edit-compute-size ((graph histogram))
  ;; In lieu of a real add-sizes method combination we do this...
  (let* ((size (call-next-method))
         (width (point-h size))
         (height (point-v size)))
    (make-point width (+ height 30))))

(defmethod change-graph :after ((graph-editor histogram-editor))
  (let ((graph (graph-editor-graph graph-editor)))
    (setf (slot-value graph 'number-of-bins)
          (number-item-value (view-named 'number-of-bins graph-editor)))
    (setf (variable-plot-data graph) (generate-plots graph))
    (set-table-sequence (view-named 'plots graph-editor) (variable-plot-data (graph-editor-graph graph-editor)))))

;;; -------------------------------------------------------------------------------------
;; Scatter, row-line, line, regression editor

(defclass line-or-symbol-plot-editor (graph-editor)
  ()
  )

(defmethod graph-edit-editor-class ((graph line-or-symbol-plot-mixin))
  'line-or-symbol-plot-editor)

;; The above should be enough, but strangely does not work. This is a hack.
#+CRAP
(defmethod graph-edit-editor-class ((graph row-line-plot))
  'line-or-symbol-plot-editor)
#+CRAP
(defmethod graph-edit-editor-class ((graph scatterplot))
  'line-or-symbol-plot-editor)
#+CRAP
(defmethod graph-edit-editor-class ((graph line-plot))
  'line-or-symbol-plot-editor)
#+CRAP
(defmethod graph-edit-editor-class ((graph regression-plot))
  'line-or-symbol-plot-editor)

(defmethod initialize-instance :after ((graph-editor line-or-symbol-plot-editor) &key)
  '())

(defmethod enable-special-items ((graph-editor line-or-symbol-plot-editor) (graph line-or-symbol-plot-mixin) (plot list)
                                 &key (disable-p NIL))
  (let ((ucc (view-named 'use-custom-char graph-editor))
        (ccdi (view-named 'custom-symbol graph-editor))
        (xsh (view-named 'x-shift graph-editor))
        (xst (view-named 'xs-tag graph-editor))
        (special-options (car (last (car plot))))
        marker buff)
    ;; set up custom character thingy
    (if disable-p
      (mapcar #'dialog-item-disable `(,ucc ,ccdi ,xsh ,xst))
      (progn
        (if (member (caar plot) '(:POLYLINE-WITH-VERTICES :scatter))
          (progn
            (if (setf marker (search (list :symbol) special-options)) ;; if there's already one set up
              (progn
                (apply #'remove-menu-items ccdi (menu-items ccdi))
                (add-menu-items ccdi (make-instance 'valued-menu-item :value (setf buff (elt special-options (1+ marker)))
                                                    :menu-item-title (format nil (if (characterp buff) "~C" "~:(~A~)") buff))
                                (make-instance 'menu-item :menu-item-title "-" :disabled t))
                (apply #'add-menu-items ccdi
                       (map 'list #'(lambda (symbol) 
                                      (make-instance 'valued-menu-item :value symbol
                                                     :menu-item-title (format nil (cond ((characterp symbol) "~C")
                                                                                        (t "~:(~A~)")) symbol)))
                            *symbologies*))
                (invalidate-view ccdi t)))
            (dialog-item-enable ccdi)
            (dialog-item-enable ucc)
            (check-box-check ucc))
          (progn
            (dialog-item-disable ccdi)
            (dialog-item-disable ucc)))
        ;; set up x shift box
        (if (ignore-errors (shift-x graph))
          (progn (dialog-item-enable xsh)
                 (dialog-item-enable xst)
                 (set-dialog-item-text xsh (format nil "~S" (shift-x graph)))))))))

(defmethod graph-edit-compute-size ((graph line-or-symbol-plot-mixin))
  ;; In lieu of a real add-sizes method combination we do this...
  (let* ((size (call-next-method))
         (width (point-h size))
         (height (point-v size)))
    (make-point width (+ height 65))))

(defmethod graph-edit-compute-subviews ((graph line-or-symbol-plot-mixin))
  ;; In lieu of a real append method combination we do this...
  (let* ((size (graph-edit-compute-size graph))
         (height (point-v size)))
    (append (call-next-method)
            (list
             (make-instance 'disabler-checkbox-item :view-nick-name 'use-custom-char
                            :dialog-item-text "Use custom symbol" :dialog-item-enabled-p nil
                            :check-box-checked-p nil
                            :watches-over '(custom-symbol)
                            :view-position (make-point 32 (- height 60)))
             (make-instance 'pop-up-menu :view-position (make-point 215 (- height 60))
                            :view-size #@(80 18) :view-nick-name 'custom-symbol
                            #-CCL-3 :dialog-item-enabled-p #+CCL-3 :enabledp nil
                            :menu-items (map 'list #'(lambda (symbol) 
                                                       (make-instance 'valued-menu-item :value symbol
                                                                      :menu-item-title (format nil (cond ((characterp symbol) "~C")
                                                                                                         (t "~:(~A~)")) symbol)))
                                             *symbologies*))
             #+IGNORE (make-instance 'editable-text-dialog-item :view-nick-name 'custom-char
                            :dialog-item-enabled-p nil :dialog-item-text "o"
                            :view-position (make-point 215 (- height 60))
                            :view-size #@(59 15))
             ;; This might be taken out of here
             (make-instance 'static-text-dialog-item :dialog-item-text "X Shift" :dialog-item-enabled-p nil
                            :view-nick-name 'xs-tag :view-position (make-point 15 (- height 25)))
             (make-instance 'number-entry-item :view-nick-name 'x-shift  :dialog-item-enabled-p nil
                            :dialog-item-enabled-p nil :dialog-item-text "0"
                            :view-position (make-point 75 (- height 25))
                            :view-size #@(59 15))))))

(defmethod change-graph :after ((graph-editor line-or-symbol-plot-editor))
  (let* ((plot-list (view-named 'plots graph-editor))
         ;; find the individual plot if there is one selected
         (plot (ignore-errors
                (elt (table-sequence plot-list) 
                     (cell-to-index plot-list 0 (point-v (car (selected-cells plot-list)))))))
         (graph (graph-editor-graph graph-editor))
         (ucc (view-named 'use-custom-char graph-editor))
         (ccdi (view-named 'custom-symbol graph-editor))
         (xsh (number-item-value (view-named 'x-shift graph-editor)))
         new-plot-options
         (symbol (value (selected-item ccdi))))
    ;; do the x shift stuff
    (if (and (dialog-item-enabled-p (view-named 'x-shift graph-editor))
             (ignore-errors (/= xsh (shift-x graph))))
      (progn (setf (shift-x graph) xsh) (setf (variable-plot-data graph) (generate-plots graph))))
    ;; do the custom symbol stuff
    (if (check-box-checked-p ucc)
      (setf new-plot-options (append `(:SYMBOL ,symbol) new-plot-options)))
    ;; set the plot to reflect the changes
    (if (and plot new-plot-options)
      (progn
        (setf (elt (variable-plot-data graph) (position plot (variable-plot-data graph) :test #'equal))
              (install-plot-options plot new-plot-options))
        (set-table-sequence (view-named 'plots graph-editor) (variable-plot-data graph))))
    ))

;;; -------------------------------------------------------------------------------------
;;; Histogram editor

(defclass cluster-editor (line-or-symbol-plot-editor)
  ())

(defmethod graph-edit-editor-class ((graph cluster-plot))
  'cluster-editor)

(defmethod graph-edit-compute-size ((graph cluster-plot))
  ;; In lieu of a real add-sizes method combination we do this...
  (let* ((size (call-next-method))
         (width (point-h size))
         (height (point-v size)))
    (make-point width (+ height 65))))

(defmethod graph-edit-compute-subviews ((graph cluster-plot))
  ;; In lieu of a real append method combination we do this...
  (let* (;(size (graph-edit-compute-size graph))
         ;(height (point-v size))
         )
    (append 
     (call-next-method)
     (list 
      (MAKE-DIALOG-ITEM
       'CHECK-BOX-DIALOG-ITEM
       #@(32 199)
       #@(72 16)
       "Filled"
       'NIL
       :view-nick-name 'filled
       :VIEW-FONT
       '("Chicago" 12 :SRCOR :PLAIN)
       :DIALOG-ITEM-ENABLED-P NIL)
      (MAKE-DIALOG-ITEM
       'STATIC-TEXT-DIALOG-ITEM
       #@(5 229)
       #@(56 17)
       "Dot Size"
       'NIL
       :view-nick-name 'size-name
       :VIEW-FONT
       '("Chicago" 12 :SRCOR :PLAIN)
       :DIALOG-ITEM-ENABLED-P NIL)
      (MAKE-DIALOG-ITEM
       'STATIC-TEXT-DIALOG-ITEM
       #@(108 229)
       #@(40 17)
       "Scale"
       'NIL
       :view-nick-name 'scale-name
       :VIEW-FONT
       '("Chicago" 12 :SRCOR :PLAIN)
       :DIALOG-ITEM-ENABLED-P NIL)
      (MAKE-DIALOG-ITEM
       'STATIC-TEXT-DIALOG-ITEM
       #@(192 229)
       #@(81 17)
       "Cluster Size"
       'NIL
       :VIEW-NICK-NAME 'cluster-SIZE-NAME
       :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
       :DIALOG-ITEM-ENABLED-P NIL)
      (MAKE-DIALOG-ITEM
       'RADIO-BUTTON-DIALOG-ITEM
       #@(112 247)
       #@(62 17)
       "Linear"
       'NIL
       :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
       :view-nick-name 'linear
       :RADIO-BUTTON-PUSHED-P t
       :DIALOG-ITEM-ENABLED-P NIL)
      (MAKE-DIALOG-ITEM
       'RADIO-BUTTON-DIALOG-ITEM
       #@(112 265)
       #@(101 16)
       "Exponential"
       'NIL
       :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
       :view-nick-name 'exponential
       :DIALOG-ITEM-ENABLED-P NIL)
      (MAKE-DIALOG-ITEM
       'NUMBER-ENTRY-ITEM
       #@(68 228)
       #@(26 13)
       "1"
       'NIL
       :VIEW-NICK-NAME 'size
       :DIALOG-ITEM-ENABLED-P NIL
       :allow-floats nil
       :allow-negatives nil
       :allow-zero nil)
      (MAKE-DIALOG-ITEM
       'NUMBER-ENTRY-ITEM
       #@(149 228)
       #@(26 13)
       "1"
       'NIL
       :VIEW-NICK-NAME 'scale
       :DIALOG-ITEM-ENABLED-P NIL
       :allow-floats t
       :allow-negatives nil
       :allow-zero nil)
      (MAKE-DIALOG-ITEM
       'NUMBER-ENTRY-ITEM
       #@(276 228)
       #@(26 13)
       "1"
       'NIL
       :VIEW-NICK-NAME 'cluster-size
       :DIALOG-ITEM-ENABLED-P NIL
       :allow-floats nil
       :allow-negatives nil
       :allow-zero nil))
    )))

(defmethod enable-special-items :after ((graph-editor cluster-editor) (cluster cluster-plot) 
                                        (plot list) &key (disable-p nil))
  (let* ((scale (view-named 'scale graph-editor))
         (size (view-named 'size graph-editor))
         (cluster-size (view-named 'cluster-size graph-editor))
         (scale-name (view-named 'scale-name graph-editor))
         (size-name (view-named 'size-name graph-editor))
         (cluster-size-name (view-named 'cluster-size-name graph-editor))
         (filled (view-named 'filled graph-editor))
         (linear (view-named 'linear graph-editor))
         (exponential (view-named 'exponential graph-editor)))
    (cond (disable-p
           (dialog-item-disable scale)
           (dialog-item-disable size)
           (dialog-item-disable cluster-size)
           (dialog-item-disable scale-name)
           (dialog-item-disable size-name)
           (dialog-item-disable cluster-size-name)
           (dialog-item-disable filled)
           (dialog-item-disable linear)
           (dialog-item-disable exponential))
          (t
           (set-dialog-item-text scale (princ-to-string (slot-value cluster 'scale)))
           (set-dialog-item-text size (princ-to-string (slot-value cluster 'size)))
           (set-dialog-item-text cluster-size (princ-to-string (slot-value cluster 'cluster-size)))
           (if (slot-value cluster 'filled) 
             (check-box-check filled)
             (check-box-uncheck filled))
           (ecase (slot-value cluster 'scale-type)
             (:linear      (radio-button-push linear))
             (:exponential (radio-button-push exponential)))
           
           (dialog-item-enable scale)
           (dialog-item-enable size)
           (dialog-item-enable cluster-size)
           (dialog-item-enable scale-name)
           (dialog-item-enable size-name)
           (dialog-item-enable cluster-size-name)
           (dialog-item-enable filled)
           (dialog-item-enable linear)
           (dialog-item-enable exponential))
          )))

(defmethod change-graph ((graph-editor cluster-editor))
  (let* ((graph (graph-editor-graph graph-editor))
         (plot-list (view-named 'plots graph-editor))
         ;; find the individual plot if there is one selected
         (plot (ignore-errors
                (elt (table-sequence plot-list) 
                     (cell-to-index plot-list 0 (point-v (car (selected-cells plot-list)))))))
         )

    (setf (slot-value graph 'scale)
          (number-item-value (view-named 'scale graph-editor))
          (slot-value graph 'size)
          (number-item-value (view-named 'size graph-editor))
          (slot-value graph 'cluster-size)
          (number-item-value (view-named 'cluster-size graph-editor))
          (slot-value graph 'filled)
          (check-box-checked-p (view-named 'filled graph-editor))
          (slot-value graph 'scale-type)                               
          (cond ((radio-button-pushed-p (view-named 'exponential graph-editor))
                 :EXPONENTIAL)
                (t
                 :LINEAR)))

    (when plot
      (setf (elt (variable-plot-data graph) (position plot (variable-plot-data graph) :test #'equal))
            (install-plot-options 
             plot
             `(:SCALE (,(slot-value graph 'scale-type) ,(slot-value graph 'scale))
               :SIZE  ,(slot-value graph 'size)
               :GROUP-SIZE ,(slot-value graph 'cluster-size)    
               :FILLED ,(slot-value graph 'filled)))))
    (set-table-sequence (view-named 'plots graph-editor) (variable-plot-data graph))
    (call-next-method)))

;;;
;;;  OVERLAID GRAPH EDITOR (just spawns editors for the component graphs)
;;;

;;;
;;; overlaid graph editing. it's easy because we just spawn editors for each component graph.
;;;

(defclass overlaid-graph-editor (window)
  ((graph
    :initform nil
    :initarg :graph
    :type overlaid-graph
    :accessor overlaid-graph-editor-graph
    :documentation "The graph that is being operated on."))
  (:documentation "a way of launching overlaid graph component's editors.")
  (:DEFAULT-INITARGS
    :WINDOW-TYPE :DOCUMENT
    :VIEW-SIZE #@(300 115)
    :VIEW-POSITION (best-spot 300 115)
    :VIEW-SUBVIEWS (list (make-instance 'button-dialog-item 
                           :view-position #@(230 3) :dialog-item-text "edit"
                           :dialog-item-action #'(lambda (item) (launch-graph-editor (view-container item))))
                         (make-instance 'disabler-checkbox-item :view-nick-name 'use-x-bounds
                                        :dialog-item-text "Custom X bounds"
                                        :view-position #@(5 27)
                                        :check-box-checked-p nil
                                        :watches-over '(x-min x-max))
                         (make-instance 'number-entry-item :view-nick-name 'x-min
                                        :dialog-item-enabled-p nil
                                        :view-position #@(150 27) :view-size #@(59 15))
                         (make-instance 'number-entry-item :view-nick-name 'x-max
                                        :dialog-item-enabled-p nil
                                        :view-position #@(230 27) :view-size #@(59 15))
                         (make-instance 'disabler-checkbox-item :view-nick-name 'use-y-bounds
                                        :dialog-item-text "Custom Y bounds"
                                        :view-position #@(5 52)
                                        :check-box-checked-p nil
                                        :watches-over '(y-min y-max))
                         (make-instance 'number-entry-item :view-nick-name 'y-min
                                        :dialog-item-enabled-p nil
                                        :view-position #@(150 52) :view-size #@(59 15))
                         (make-instance 'number-entry-item :view-nick-name 'y-max
                                        :dialog-item-enabled-p nil
                                        :view-position #@(230 52) :view-size #@(59 15))
                         (make-instance 'button-dialog-item :view-position #@(95 80)
                                        :dialog-item-text "Plot"
                                        :dialog-item-action #'(lambda (item)
                                                                (redraw-graph-window (view-container item))))
                         (make-instance 'default-button-dialog-item
                           :view-nick-name 'change-button 
                           :dialog-item-text "Change"
                           :view-position #@(145 80)
                           :dialog-item-action #'(lambda (item)
                                                   (with-cursor *watch-cursor*
                                                     (change-graph (view-container item)))))
                         )))

(defmethod initialize-instance :after ((oge overlaid-graph-editor) &key)
  (set-window-title oge (clasp::name (overlaid-graph-editor-graph oge)))
  (add-subviews oge (make-instance 'pop-up-menu :view-position #@(10 1)
                                   #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Graph" :view-size #@(200 19)
                                   :view-nick-name 'component-pum
                                   :menu-items (mapcar #'(lambda (item) 
                                                           (make-instance 'menu-item 
                                                             :menu-item-title (clasp::name item)))
                                                       (graphs-overlaid-graphs (overlaid-graph-editor-graph oge)))))
  (let ((xbc (view-named 'use-x-bounds oge))
        (xmindi (view-named 'x-min oge))
        (xmaxdi (view-named 'x-max oge))
        (ybc (view-named 'use-y-bounds oge))
        (ymindi (view-named 'y-min oge))
        (ymaxdi (view-named 'y-max oge))
        (graph (overlaid-graph-editor-graph oge)))
    (multiple-value-bind (xmin xmax xcp ymin ymax ycp)
                         (xy-bounds graph)
      (if xcp (progn (dialog-item-enable xmindi) (dialog-item-enable xmaxdi) 
                     (check-box-check xbc)))
      (if ycp (progn (dialog-item-enable ymindi) (dialog-item-enable ymaxdi) 
                     (check-box-check ybc)))
      (set-dialog-item-text xmindi (if xmin
                                     (format nil "~S" xmin)
                                     (dialog-item-disable xbc)))
      (set-dialog-item-text xmaxdi (if xmax
                                     (format nil "~S" xmax)
                                     (dialog-item-disable xbc)))
      (set-dialog-item-text ymindi (if ymin
                                     (format nil "~S" ymin)
                                     (dialog-item-disable ybc)))
      (set-dialog-item-text ymaxdi (if ymax
                                     (format nil "~S" ymax)
                                     (dialog-item-disable ybc))))))

(defmethod redraw-graph-window ((oge overlaid-graph-editor))
  (let ((window (find-graph-window (overlaid-graph-editor-graph oge))))
    (when window
      (window-close window))
    (open-clasp-object (overlaid-graph-editor-graph oge))))

(defmethod launch-graph-editor ((oge overlaid-graph-editor))
  (let* ((graph-number (search (list (selected-item (view-named 'component-pum oge))) (menu-items (view-named 'component-pum oge))))
         (graph (elt (graphs-overlaid-graphs (overlaid-graph-editor-graph oge)) graph-number)))
    (make-graph-editor graph)))

(defmethod change-graph ((oge overlaid-graph-editor))
  (let ((uxb (view-named 'use-x-bounds oge))
         (uyb (view-named 'use-y-bounds oge))
         (xlb (number-item-value (view-named 'x-min oge)))
         (xub (number-item-value (view-named 'x-max oge)))
         (ylb (number-item-value (view-named 'y-min oge)))
         (yub (number-item-value (view-named 'y-max oge)))
         new-keyword-list)
    (if (check-box-checked-p uxb)
      (progn (push xlb new-keyword-list) (push :x-first new-keyword-list)
             (push xub new-keyword-list) (push :x-last new-keyword-list)))
    (if (check-box-checked-p uyb)
      (progn (push ylb new-keyword-list) (push :y-first new-keyword-list)
             (push yub new-keyword-list) (push :y-last new-keyword-list)))
    (setf (graph-keywords (overlaid-graph-editor-graph oge))
          (replace-keywords (graph-keywords (overlaid-graph-editor-graph oge))
                            new-keyword-list
                            *graph-editor-affects-keywords*))))
