;;;; -*- Mode:Common-Lisp; Package:CLASP-INTERFACE; Fonts:(MEDFNT); Base:10; Patch-file:T -*-
;;;
;;; New Menus
;;;

#| This is a new scheme of defining the CLASP menus that is amenable to patch-work |#

;;; 10-14-97 Added code to allow conditinalization of menu installation.
;;;          There is a newer version of this code in ACS and GAFS and it should
;;;          be migrated into here (or into EKSL Utils).  (Westy)

;;; 10-15-97 Updated to use new interface in EKSL Utilities.  (Westy)

;;;

(in-package :clasp-interface)

;; ================================================================
;; macros for defining menu items

#|

(defparameter *menu-defs* NIL)

(defclass conditional-menu-mixin ()
  ((predicate :initform nil :initarg :predicate :accessor predicate)))

(defclass tag-mixin ()
  ((tag :initform NIL :initarg :tag :accessor tag)))

(defclass menudef (tag-mixin conditional-menu-mixin)
  ((title :initform NIL :initarg :title :accessor menu-title)
   (help-spec :initform NIL :initarg :help-spec :accessor menu-help-spec)
   (class :initform NIL :initarg :class :accessor menu-class)
   (menu-items :initform NIL :initarg :menu-items :accessor menu-menu-items))
  :menu documentation "A definition for a menu."))

(defclass menu-itemdef (tag-mixin conditional-menu-mixin)
  ((class :initform NIL :initarg :class :accessor menu-item-class)
   (keyargs :initform NIL :initarg :keyargs :accessor menu-item-keyargs)
   (action :initform NIL :initarg :action :accessor menu-item-action))
  :menu documentation "A definition of a menu item."))

(defmethod tag ((item t))
  NIL)

(defun find-menudef (tag list)
  (if (null list)
    NIL
    (if (typep (car list) 'menudef) 
      (if (eql (tag (car list)) tag)
        (car list)
        (or (find-menudef tag (menu-menu-items (car list))) (find-menudef tag (cdr list))))
      (find-menudef tag (cdr list)))))

(defmethod weed-menudef ((menudef menudef) tag)
  (setf (menu-menu-items menudef) (delete tag (menu-menu-items menudef) :key #'tag))
  (mapc #'(lambda (md) (if (typep md 'menudef) (weed-menudef md tag))) (menu-menu-items menudef)))

(defmacro destroy-menudef (tag)
  `(progn (setf *menu-defs* (delete ,tag *menu-defs* :key #'tag))
          (mapc #'(lambda (md) (if (typep md 'menudef) (weed-menudef md ,tag))) *menu-defs*)))

(defmethod print-menu (list &optional (l 0))
  (if (null list)
    NIL
    (progn
      (format t "~A~S~%" (make-string (* l 2) :initial-element #\space) (tag (car list)))
      (when (typep (car list) 'menudef)
        (print-menu (menu-menu-items (car list)) (1+ l)))
      (print-menu (cdr list) l))))

(defmacro defmenu (menu-parent-name menu-name menu-class title help-spec &key predicate)
  (let ((menudef (gensym "MENUDEF"))
        (parent  (gensym "PARENT")))
  `(let ((,menudef (make-instance 'menudef :tag ,menu-name :class ,menu-class
                                 :title ,title :help-spec ,help-spec
                                 :predicate ,predicate))
         (,parent (find-menudef ,menu-parent-name *menu-defs*)))
     (if ,parent
       (push ,menudef (menu-menu-items ,parent))
       (push ,menudef *menu-defs*)))))

(defmacro defmenu-item (menu-item-name (menu-name menu-item-class &rest keys) &body action)
  (let ((menu (gensym "MENU"))
        (mitem (gensym "MITEM")))
    `(let ((,menu (find-menudef ,menu-name *menu-defs*))
           (,mitem (make-instance 'menu-itemdef :tag ,menu-item-name
                                  :class ,menu-item-class
                                  :keyargs ',keys
                                  :action #'(lambda () ,@action))))
       (destroy-menudef ,menu-item-name)
       (if ,menu
          (push ,mitem (menu-menu-items ,menu))
          (error "Illegal forward reference to menu ~S" ,menu-name)))))

(defmacro defmenu-raw (menu)
  `(push ,menu *menu-defs*))

(defmethod menumake :around ((menu conditional-menu-mixin))
  (when (or (null (predicate menu))
            (funcall (predicate menu)))
    (call-next-method)))

(defmethod menumake ((mid menu-itemdef))
  (apply #'make-instance (menu-item-class mid) :menu-item-action (menu-item-action mid)
         (menu-item-keyargs mid)))

(defmethod menumake ((md menudef))
  (make-instance (menu-class md) :menu-title (menu-title md) :help-spec (menu-help-spec md)
                 :menu-items (mapcar #'menumake (reverse (menu-menu-items md)))))

(defmethod menumake ((m menu))
  m)

(defmethod menumake ((fun function))
  (funcall fun))

(proclaim '(inline compute-menus))

(defun compute-menus (list)
  (mapcar #'menumake (reverse list)))

(defun install-as-menu (menu-name defs &optional (leaders NIL))
  (let ((oldmenu (find-menu menu-name)))
    (when oldmenu (menu-deinstall oldmenu))
    (menu-install (make-instance 'menu :menu-title menu-name
                                 :menu-items (append leaders (compute-menus defs))))))

(defun install-menubar (defs)
  (set-menubar (compute-menus defs)))

|#

;;; ================================================================
;;; CLASP Basic Menus

;;; ================================================================
;;; file menu

(initialize-menus)

(defmenu file :help-spec "File menu for I/O and general options.")

(defmenu-item (load-dataset :menu 'file :title "Load Dataset"
                            :help-spec "Prompts you to select a dataset file then loads it."
                            #-MCG-TEST :command-key #-MCG-TEST #\O
                            #+MCG-TEST :command-key #+MCG-TEST #\0)
  (eval-enqueue '(choose-and-load-dataset)))

(defmenu-item (import-dataset :menu 'file :title "Import Dataset"
                              :help-spec "Prompts you to select a data file then imports it. Use the Preferences commands to specify parameters such as file separator."
                              :command-key #\I)
  (eval-enqueue '(choose-and-import-dataset)))

(defmenu-item (save-dataset :menu 'file 
                            :menu-item-class 'one-dataset-menu-item
                            :title "Save Dataset"
                            :help-spec "Prompts you to name a data file in which to save the selected dataset. Enabled when exactly one dataset is selected in the frontmost clasp window."
                            :command-key #\S)
  (eval-enqueue '(save-selected-dataset)))

(defmenu-item (export-dataset 
               :menu 'file 
               :menu-item-class 'one-dataset-menu-item
               :title "Export Dataset"
               :help-spec "Prompts you to name a data file then exports the selected dataset to it. Use the Preferences commands to specify parameters such as file separator.")
  (eval-enqueue '(export-selected-dataset)))

(defmenu-item (close :menu 'file :title "Close"
                     :command-key #\W)
  (eval-enqueue '(window-close (front-window))))

(defmenu-item (close-all :menu 'file :title "Close All" :command-key #\-)
  (mapc #'(lambda (w) (unless (or (typep w 'listener) (typep w 'clasp-dock)
                                  (typep w 'fred-window) (typep w 'clasp-window))
                        (window-close w))) (windows)))

(defmenu-separator (:menu 'file))

(defmenu-item (clear-history :menu 'file :menu-item-class 'clasp-window-menu-item
              :title "Clear History"
              :command-key #\Backspace
              :help-spec "Clears the frontmost CLASP window's history view. Enabled when the front window is a CLASP window.")
  (eval-enqueue `(clear-history-view (clasp-window-front))))

(defmenu-item (update :menu 'file :menu-item-class 'clasp-window-menu-item
              :title "Update Dataset View"
              :help-spec "Updates the dataset view in the frontmost CLASP window. Enabled when the front window is a CLASP window.")
  (eval-enqueue `(update-dataset-view (clasp-window-front))))

(defmenu-item (log-sel :menu 'file :menu-item-class 'history-selection-menu-item
              :title "Log Selection"
              :command-key #\T
              :help-spec "Logs the frontmost CLASP window's selected history items. Enabled when the front window is a CLASP window.")
  (eval-enqueue `(log-selected-items)))

(defmenu-item (copy-clip :menu 'file :title  "Copy to Clipboard" 
              :command-key #\C
              :help-spec "Cuts the current window's selection to the clipboard.")
  #+DEMO (sorry-demo-version)
  #-DEMO (eval-enqueue `(cut-selection (front-window))))

(defmenu-item (copy-rib :menu 'file :menu-item-class 'ribbon-open-menu-item
                        :title "Copy Ribbon to Clipboard"
                        :command-key #\B
                        :help-spec "Cuts the ribbon window's contents to the clipboard.")
  (eval-enqueue `(cut-selection (find-window "gibbon-monkey" 'ribbon-window))))

(defmenu-item (paste :menu 'file :title  "Paste from Clipboard"
              :command-key #\V
              :help-spec "Pastes the current clipboard to the current window.")
  (eval-enqueue `(paste-into-clasp (front-window))))

(defmenu-separator (:menu 'file))

(defmenu-item (prefs :menu 'file :title  "Preferences"
                     :command-key #\`)
  (eval-enqueue '(edit-preferences)))

(defmenu-separator (:menu 'file))

(defmenu-item (quit :menu 'file :title  "Quit"
                    :COMMAND-KEY #\Q
                    :help-spec "Closes all windows and exits from CLASP. If the contents of any window have not been saved, you will loose that data.")
  (eval-enqueue '(quit-clasp)))

;;; ================================================================
;;; graph menu

(defmenu graph :help-spec "A menu full of graphing commands.")

(defmenu-item (histogram :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Histogram"
                         :command-key #\H)
  (eval-enqueue '(graph-ov-dialog-make 'histogram "Histogram" :help '*histogram-help*)))

(defmenu-item (scatterplot :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Scatter Plot"
                                   :command-key #\A)
  (eval-enqueue '(graph-tv-dialog-make 'scatterplot "Scatterplot" t 
                  :help '*scatterplot-help* :categorical-x nil :categorical-y nil)))

(defmenu-item (clusterplot :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Cluster Plot")
  (eval-enqueue '(graph-tv-dialog-make 'cluster-plot "Cluster Plot" t
                  :help '*cluster-plot-help* :categorical-x nil :categorical-y nil)))

(defmenu-item (lineplot :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Line Plot"
                                :command-key #\L)
  (eval-enqueue '(graph-tv-dialog-make 'line-plot "Line Plot" NIL 
                  :help '*line-plot-help* :categorical-x nil :categorical-y nil)))

(defmenu-item (rowplot :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Row Line Plot"
                               :command-key #\N )
  (eval-enqueue '(graph-ov-dialog-make 'row-line-plot "Row Line Plot" 
                  :help '*row-line-help* :categorical-ok nil)))

;;; 10/23/96 (MS) NO HELP !!
(defmenu-item (means-plot :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Means Plot")
    (eval-enqueue `(make-instance 'graph-ci-dialog)))

(defmenu-item (regplot :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Regression Plot"
                               :command-key #\G )
  (eval-enqueue '(graph-tv-dialog-make 'regression-plot "Regression Plot" t 
                  :help '*regression-plot-help* :categorical-x nil :categorical-y nil)))

(defmenu-item (tableplot :menu 'graph :menu-item-class 'table-window-menu-item :title "Table plot"
                                 :command-key #\P :slot-to-check nil)
  (eval-enqueue `(make-table-plot (front-window))))

(defmenu-item (overlay :menu 'graph :menu-item-class 'clasp-window-menu-item :title "Overlay Graphs"
                               :command-key #\=)
  (eval-enqueue '(make-instance 'overlaid-graph-dialog)))

(defmenu-separator (:menu 'graph))

(defmenu-item (editgraph :menu 'graph :menu-item-class 'graph-window-menu-item :title "Edit"
                                 :command-key #\E)
  (eval-enqueue `(make-graph-editor (window-graph (front-window)))))

(defmenu-item (key :menu 'graph :menu-item-class 'graph-window-menu-item :title "Graph Key"
                           :command-key #\K)
  (eval-enqueue `(plotter::pop-key (front-window))))

;;; ================================================================
;;; describe menu

(defmenu describe :help-spec "Descriptive stats")

(defmenu-item (length :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Data Length")
  (eval-enqueue `(one-variable-dialog-make 'clasp::data-length "Data Length"
                                           (make-line-mapper "~D is data length of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*data-length-help*)))

(defmenu-item (mean :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Mean")
  (eval-enqueue `(one-variable-dialog-make 'clasp::mean "Mean"
                                           (make-line-mapper "~,4F is mean of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*mean-help*
                                           :categorical-ok nil)))

(defmenu-item (variance :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Variance")
  (eval-enqueue `(one-variable-dialog-make 'clasp::variance "Variance"
                                           (make-line-mapper "~,4F is variance of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*variance-help* :categorical-ok nil)))

(defmenu-item (stddev :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Standard Deviation")
  (eval-enqueue `(one-variable-dialog-make 'clasp::standard-deviation "Standard Deviation"
                                           (make-line-mapper "~,4F is standard deviation of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*std-dev-help* :categorical-ok nil)))

(defmenu-item (minimum :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Minimum")
  (eval-enqueue `(one-variable-dialog-make 'clasp::minimum "Minimum"
                                           (make-line-mapper "~,4F is minimum of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*minimum-help* :categorical-ok nil)))

(defmenu-item (maximum :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Maximum")
  (eval-enqueue `(one-variable-dialog-make 'clasp::maximum "Maximum"
                                           (make-line-mapper "~,4F is maximum of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*maximum-help* :categorical-ok nil)))

(defmenu-item (range :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Range")
  (eval-enqueue `(one-variable-dialog-make 'clasp::range "Range"
                                           (make-line-mapper "~,4F is range of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*range-help* :categorical-ok nil)))

(defmenu-item (quantile :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Quantile")
  (eval-enqueue '(quantile-dialog-make)))

(defmenu-item (median :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Median")
  (eval-enqueue `(one-variable-dialog-make 'clasp::median "Median"
                                           (make-line-mapper "~,4F is median of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*median-help* :categorical-ok nil)))

(defmenu-item (trimmean :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Trimmed Mean")
  (eval-enqueue '(trimmed-mean-dialog-make)))

(defmenu-item (mode :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Mode")
  (eval-enqueue `(mode-dialog-make)))

(defmenu-item (iqrange :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Interquartile Range")
  (eval-enqueue `(one-variable-dialog-make 'clasp::interquartile-range "Interquartile Range"
                                           (make-line-mapper "~,4F is interquartile Range of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*iqr-help* :categorical-ok nil)))

(defmenu-item (skew :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Skew")
  (eval-enqueue `(one-variable-dialog-make 'clasp::skewness "Skew"
                                           (make-line-mapper "~,4F is skew of ~:(~A~)"
                                                             '(0 x-args))
                                           :help '*skew-help* :categorical-ok nil)))

(defmenu-item (summary :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Statistical Summary" :command-key #\M)
  (eval-enqueue `(one-variable-dialog-make 'clasp::statistical-summary "Statistical Summary" 
                                           (make-table-maker *statistical-summary-labels*
                                                             :table-type :column-table
                                                             :name-format "Statistical Summary of ~A"
                                                             :name-args '(X-ARGS))
                                           :help '*statistical-summary-help* :categorical-ok nil)))

(defmenu-item (covariance :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Covariance")
  (eval-enqueue `(two-variable-dialog-make 'clasp::covariance "Covariance"
                                           (make-line-mapper "~,4F is covariance of ~:(~A~) and ~:(~A~)"
                                                             '(0 X-ARGS Y-ARGS))
                                           :help '*covariance-help*
                                           :categorical-x NIL :categorical-y NIL)))

(defmenu-item (correlation :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Correlation")
  (eval-enqueue `(two-variable-dialog-make 'clasp::correlation "Correlation"
                  (make-line-mapper "~,4F is correlation of ~:(~A~) and ~:(~A~)"
                                    '(0 X-ARGS Y-ARGS))
                  :help '*correlation-help*
                  :categorical-x NIL :categorical-y NIL)))

(defmenu-item (parcorr :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Partial Correlation")
  (eval-enqueue `(partial-corr-dialog-make)))

(defmenu-item (crosscorr :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Cross Correlation")
  (eval-enqueue '(cross-correlation-dialog-make)))

(defmenu-item (autocorr :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Autocorrelation")
  (eval-enqueue '(autocorrelation-dialog-make)))

(defmenu-item (corrtable :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Correlation Table")
  (eval-enqueue `(corr-table-dialog-make)))

(defmenu-item (conttable :menu 'describe :menu-item-class 'clasp-window-menu-item :title "Contingency Table")
  (eval-enqueue `(two-variable-dialog-make 'clasp::make-contingency-table "Contingency Table"
                                           'ctable-format :help '*contingency-help*)))
#+FBD
(defmenu-item (fbd-model :menu 'describe :menu-item-class 'clasp-window-menu-item :title "FBD Causal Model")
  (eval-enqueue `(fbd-dialog-make)))

(defmenu-item (cdf :menu 'describe :menu-item-class 'clasp-window-menu-item :title "CDF Functions")
  (eval-enqueue `(cdf-dialog-make)))

;;; ================================================================
;;; manipulate menu

(defmenu manipulate :help-spec "Dataset manipulation commands.")

(defmenu-item (rename :menu 'manipulate :menu-item-class 'clasp-window-menu-item :title "Rename"
                                   :help-spec "Renames the selected dataset. Enabled when the front window is a CLASP window with one dataset selected."
                                   :command-key #\R)
  (eval-enqueue '(rename-selected-things)))

(defmenu-item (delete :menu 'manipulate :menu-item-class 'clasp-window-menu-item :title "Delete Result" :command-key #\D)
  (eval-enqueue '(delete-selected-items)))

(defmenu-item (partition :menu 'manipulate :menu-item-class 'clasp-window-menu-item :title "Partition Dataset")
  (eval-enqueue '(partition-dataset-dialog-make)))

(defmenu-item (partition-on :menu 'manipulate :menu-item-class 'clasp-window-menu-item :title "Partition On")
  (eval-enqueue '(partition-on-dialog-make)))

(defmenu-item (merge :menu 'manipulate :menu-item-class 'clasp-window-menu-item :title "Merge Datasets")
  (eval-enqueue '(merge-datasets-dialog-make)))

(defmenu-separator (:menu 'manipulate))

(defmenu tables :parent manipulate :help-spec "Table manipulation stuff.")

(defmenu-item (cclick-table :menu 'tables :menu-item-class 'table-window-menu-item :title "do nothing"
                            :help-spec "Control-click menu item for table."
                            :slot-to-check 'control)
  (eval-enqueue '(progn (control-click (table-view-table
                                        (view-named 'table-array-item (front-window))))
                  (revalidate-view (view-named 'table-array-item (front-window))))))

(defmenu-item (oclick-table :menu 'tables :menu-item-class 'table-window-menu-item :title "do nothing"
                            :help-spec "Option-click menu item for table."
                            :slot-to-check 'option)
  (eval-enqueue '(progn (option-click (table-view-table (view-named 'table-array-item (front-window))))
                  (revalidate-view (view-named 'table-array-item (front-window))))))

(defmenu-item (dclick-table :menu 'tables :menu-item-class 'table-window-menu-item :title "do nothing"
                            :help-spec "Double-click menu item for table."
                            :slot-to-check 'double)
  (eval-enqueue '(progn (dbl-click (table-view-table (view-named 'table-array-item (front-window))))
                  (revalidate-view (view-named 'table-array-item (front-window))))))

(defmenu variables :parent manipulate :help-spec "Variable/data manipulation.")

(defmenu-item (xdock :menu 'variables :menu-item-class 'dock-menu-item :title "X Dock" :dock-name "X Dock"
                     :command-key #-MCG-TEST #\X #+MCG-TEST #\- :affects x-var-item
                     :help-spec "Sends the selected variables to the x dock.")
  (eval-enqueue `(operate-on-dock (front-window) 'x-var-dock)))

(defmenu-item (ydock :menu 'variables :menu-item-class 'dock-menu-item :title "Y Dock" :dock-name "Y Dock"
                     :command-key #-MCG-TEST #\Y #+MCG-TEST #\= :affects y-var-item
                     :help-spec "Sends the selected variables to the y dock.")
  (eval-enqueue `(operate-on-dock (front-window) 'y-var-dock)))

(defmenu-item (zdock :menu 'variables :menu-item-class 'dock-menu-item :title "Cache" :Dock-name "Cache"
                     :command-key #\Z :affects z-var-item)
  (eval-enqueue `(operate-on-dock (front-window) 'gp-dock)))

(defmenu-separator (:menu 'variables))

(defmenu-item (lastx :menu 'variables :menu-item-class 'mrr-menu-item :title "X"
                     :command-key #\[ :affects x-var-item
                     :help-spec "Replaces the x arguments with the most recent result.")
  (eval-enqueue `(operate-on-mrr (front-window) 'x-var-item)))

(defmenu-item (lasty :menu 'variables :menu-item-class 'mrr-menu-item :title "Y"
                                 :command-key #\] :affects y-var-item
                                 :help-spec "Replaces the y arguments with the most recent result.")
  (eval-enqueue `(operate-on-mrr (front-window) 'y-var-item)))

(defmenu-item (lastz :menu 'variables :menu-item-class 'mrr-menu-item :title "Z"
                                 :command-key #\} :affects z-var-item
                                 :help-spec "Replaces the y arguments with the most recent result.")
  (eval-enqueue `(operate-on-mrr (front-window) 'z-var-item)))

(defmenu-separator (:menu 'variables))

(defmenu-item (changetype :menu 'variables :menu-item-class 'openable-selection-menu-item :title "Change Type"
                          :command-key #\J :help-spec "Changes the type to categorical or whatever.")
  (eval-enqueue '(change-selected-variable-types)))

(defmenu-item (edit :menu 'variables :menu-item-class 'openable-selection-menu-item :title "Edit"
                    :command-key #-MCG-TEST #\E #+MCG-TEST #\Tab
                    :help-spec "Opens the selected variables and datasets from the data view for editing/viewing.")
  (eval-enqueue '(open-openable-selection)))

;;; ================================================================
;;; transform menu

(defmenu transform :help-spec "Data transformations.")

(defmenu-item (userdefined :menu 'transform :menu-item-class 'clasp-window-menu-item :title "User Defined")
  (eval-enqueue '(make-user-defined-dialog)))

(defmenu-item (sort :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Sort")
  (eval-enqueue '(sort-dialog-make)))

(defmenu-item (sortindex :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Sort by Index")
  (eval-enqueue '(sort-index-dialog-make)))

(defmenu-item (addconst :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Add a Constant")
  (eval-enqueue '(add-constant-dialog-make)))

(defmenu-item (mulconst :menu 'transform  :menu-item-class 'clasp-window-menu-item :title "Multiply by Constant")
  (eval-enqueue '(mul-constant-dialog-make)))

(defmenu-item (natlog :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Natural Log")
  (eval-enqueue '(one-var-trans-dialog-make 'clasp::natural-log "Natural Log"
                  :help '*natural-log-help* :categorical-ok NIL)))

(defmenu-item (log10 :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Log10")
  (eval-enqueue '(one-var-trans-dialog-make 'clasp::log10 "Log Base 10"
                  :help '*log10-help* :categorical-ok NIL)))

(defmenu-item (recode :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Recode Categorical")
  (eval-enqueue '(make-instance 'recode-variable-dialog)))

(defmenu-item (reassign :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Reassign Bins")
  (eval-enqueue '(rebin-dialog-make)))

(defmenu-item (smooth :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Smooth")
  (eval-enqueue '(smooth-dialog-make)))

(defmenu-item (smooth4253h :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Smooth 4253h")
  (eval-enqueue '(one-var-trans-dialog-make 'clasp::smooth-variable-4253h "Smooth 4253h"
                  :help '*smooth-4253h-help* :categorical-ok NIL)))

(defmenu-item (derivative :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Discrete Derivative")
  (eval-enqueue '(one-var-trans-dialog-make 'clasp::Difference-a-column "Discrete Derivarive"
                  :help '*derivative-help* :categorical-ok NIL)))

(defmenu-item (standardize :menu 'transform :menu-item-class 'clasp-window-menu-item :title "Standardize")
  (eval-enqueue '(one-var-trans-dialog-make 'clasp::standardize-variable "Standardize"
                  :help '*standardize-help* :categorical-ok NIL)))

;;; ================================================================
;;; test menu

(defmenu test :help-spec "Hypothesis testing.")

(defmenu-item (zconfi :menu 'test :menu-item-class 'clasp-window-menu-item :title "Confidence Interval Using z Statistic")
  (eval-enqueue '(confidence-interval-data-dialog-make 'clasp::confidence-interval-z 
                  (make-table-maker *confidence-labels* :table-type :row-table
                                    :name-format (format nil "~A Z Confidence Interval on ~A" "~5F" "~:(~A~)")
                                    :name-args '(CONFIDENCE X-ARGS)) "Confidence Interval Using Z Statistic"
                  :help '*confi-z-help*)))

(defmenu-item (tconfi :menu 'test :menu-item-class 'clasp-window-menu-item :title "Confidence Interval Using t Statistic")
  (eval-enqueue '(confidence-interval-data-dialog-make 'clasp::confidence-interval-t 
                  'confidence-t-formatter "Confidence Interval Using T Statistic"
                  :help '*confi-t-help*)))

(defmenu-item (pconfi :menu 'test :menu-item-class 'clasp-window-menu-item :title "Confidence Interval of a Proportion")
  (eval-enqueue '(confidence-prop-dialog-make)))

(defmenu-item (ttest1 :menu 'test :menu-item-class 'clasp-window-menu-item :title "t-test One Sample")
  (eval-enqueue '(t-test-one-sample-dialog-make)))

(defmenu-item (ttest2 :menu 'test :menu-item-class 'clasp-window-menu-item :title "t-test Two Samples")
  (eval-enqueue '(t-test-dialog-make)))

(defmenu-item (ttestm :menu 'test :menu-item-class 'clasp-window-menu-item :title "t-test Matched Pairs")
  (eval-enqueue '(t-test-matched-dialog-make)))

(defmenu-item (dtest :menu 'test :menu-item-class 'clasp-window-menu-item :title "D test")
  (eval-enqueue '(d-test-dialog-make)))

(defmenu-item (contrasts :menu 'test :menu-item-class 'clasp-window-menu-item :title "Analysis of Contrasts")
  (eval-enqueue `(make-instance 'contrast-toolbox-dialog)))

(defmenu-item (anova-new :menu 'test :menu-item-class 'clasp-window-menu-item :title "Analysis of Variance")
  (eval-enqueue `(make-instance 'anova-toolbox-dialog)))
#+REPLACED-AS-OF-1.4.5
(defmenu-item (anova-ow :menu 'test :menu-item-class 'clasp-window-menu-item :title "Anova - One Way")
  (eval-enqueue '(anova-one-way-dialog-make)))
#+REPLACED-AS-OF-1.4.5
(defmenu-item (anova-tw :menu 'test :menu-item-class 'clasp-window-menu-item :title "Anova - Two Way")
  (eval-enqueue '(anova-two-way-dialog-make)))

(defmenu-item (chicount :menu 'test :menu-item-class 'clasp-window-menu-item :title "Chi-Square Counts")
  (eval-enqueue '(chi-square-2x2-counts-dialog-make)))

(defmenu-item (chi2x2 :menu 'test :menu-item-class 'clasp-window-menu-item :title "Chi-Square 2x2")
  (eval-enqueue `(two-variable-dialog-make 'clasp::chi-square-2x2 "Chi-Square 2x2"
                                           'chi-2x2-format :help '*chi-2x2-help*)))

(defmenu-item (chiRxC :menu 'test :menu-item-class 'clasp-window-menu-item :title "Chi-Square RxC")
  (eval-enqueue `(two-variable-dialog-make 'clasp::chi-square-rxc "Chi-Square RxC"
                                           'chi-rxc-format :help '*chi-rxc-help*)))

(defmenu-item (3danalyze :menu 'test :menu-item-class 'clasp-window-menu-item :title "3d Contingency Table Analysis")
  (eval-enqueue `(make-instance 'analyze-3d-dialog)))

(defmenu-item (regrbrief :menu 'test :menu-item-class 'clasp-window-menu-item :title "Linear Regression - brief")
  (eval-enqueue '(two-variable-dialog-make 'clasp::linear-regression-brief "Linear Regression - brief"
                  (make-table-maker *brief-lr-labels* :table-type :column-table
                                    :name-format "Linear Regression of ~:(~A~) against ~:(~A~)"
                                    :name-args '(X-ARGS Y-ARGS))
                  :help '*brief-lr-help* :categorical-x NIL :categorical-y NIL)))

(defmenu-item (regrverbose :menu 'test :menu-item-class 'clasp-window-menu-item :title "Linear Regression - verbose")
  (eval-enqueue '(two-variable-dialog-make 'clasp::linear-regression-verbose
                  "Linear Regression - verbose" 'verbose-lr-formatter
                  :help '*verbose-lr-help* :categorical-x NIL :categorical-y NIL)))

(defmenu-item (multiregr :menu 'test :menu-item-class 'clasp-window-menu-item :title "Multiple Linear Regression - verbose")
  (eval-enqueue '(multi-linear-regr-dialog-make)))

;;; ================================================================
;;; sample menu

(defmenu sample :help-spec "Commands that make datasets.")

(defmenu-item (uniform :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Uniform Distribution")
  (eval-enqueue '(uniform-dist-dialog-make)))

(defmenu-item (normal :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Normal Distribution")
  (eval-enqueue '(normal-dist-dialog-make)))

(defmenu-item (binomial :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Binomial Distribution")
  (eval-enqueue '(binomial-dist-dialog-make)))

(defmenu-item (poisson :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Poisson Distribution")
  (eval-enqueue '(poisson-dist-dialog-make)))

(defmenu-item (gamma :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Gamma Distribution")
  (eval-enqueue '(gamma-dist-dialog-make)))

(defmenu-item (monte-carlo :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Monte Carlo Sampling")
  (eval-enqueue '(monte-carlo-dialog-make)))

(defmenu-item (bootstrap :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Bootstrap/Randomize")
  (eval-enqueue '(data-sampling-dialog-make)))

(defmenu-item (mixed-resample :menu 'sample :menu-item-class 'clasp-window-menu-item :title "Mixed Resampling")
  (eval-enqueue `(bootstrap-dialog-make)))

;;; ================================================================
;;; windows menu
;;; THIS MENU USES THE FUNCTION FORMAT. THAT MAKES IT HARD TO PATCH, BUT HEY...

(defclass clasp-windows-menu (menu)
  ())

(defmethod menu-update ((item clasp-windows-menu))
  (declare (optimize (safety 3) (speed 1) (space 1)))
  (let ((front (front-window))
        (windows (windows)))
    (apply #'remove-menu-items *clasp-windows-menu* (menu-items *clasp-windows-menu*))
    (apply #'add-menu-items *clasp-windows-menu* 
           (append *clasp-windows-menu-default-items*
                   (loop for w in windows
                         collect (make-instance 'ccl::windows-menu-menu-item
                                   :disabled (eq w front)
                                   :command-key (when (eq w *top-listener*) #\L) 
                                   :menu-item-title (ccl::window-title w)
                                   :window w))))))

(defvar *clasp-windows-menu*)

(defvar *clasp-windows-menu-default-items*)

(defvar *rw-menu-item* nil) ; stores the ribbon window menu item so I can check it's box

(defun make-windows-menu ()
  (setq *clasp-windows-menu*
        (make-instance 'clasp-windows-menu :MENU-TITLE "Windows"
                       :help-spec "Window Menu
Use this select from the various windows available in the CLASP interface."
                       :MENU-ITEMS
                       (setf *clasp-windows-menu-default-items*
                             (list
                              ;; FIX: Put following somewhere else?
                              (MAKE-INSTANCE 'menu-item :menu-item-title  "CLASP Window"
                                             :help-spec "Selects the current CLASP window or creates a new one if there isn't one."
                                             :command-key #\1
                                             :menu-item-action
                                             #'(lambda ()
                                                 (eval-enqueue '(select-current-or-new-clasp-window))))
                              (MAKE-INSTANCE 'menu-item :menu-item-title  "CLASP Variable Palette"
                                             :help-spec "Selects the CLASP variable dock or creates one if there isn't one."
                                             :command-key #\2
                                             :menu-item-action
                                             #'(lambda ()
                                                 (eval-enqueue '(find-clasp-dock :force-access t))))
                              (make-instance 'menu-item :menu-item-title  "CLASP Notebook"
                                             :help-spec "Selects the CLASP Notebook or opens a new one if there isn't one."
                                             :command-key #\3
                                             :menu-item-action
                                             #+DEMO 'sorry-demo-version
                                             #-DEMO 'open-clasp-notebook)
                              (make-instance 'menu-item :menu-item-title  "Graph Palette"
                                             :help-spec "a list of generated graphs"
                                             :command-key #\4
                                             :menu-item-action 'open-graph-caddy)
                              (make-instance 'menu-item :menu-item-title  "Table Palette"
                                             :help-spec "a list of generated tables"
                                             :command-key #\5
                                             :menu-item-action 'open-table-caddy)
                              (setf *rw-menu-item* 
                                    (make-instance 'menu-item :menu-item-title  "Ribbon Window"
                                                   :help-spec "is the ribbon window open ?"
                                                   :command-key #\6
                                                   :menu-item-checked *use-ribbon-window*
                                                   :menu-item-action
                                                   #'(lambda ()
                                                       (set-menu-item-check-mark *rw-menu-item*
                                                                                 (setf *use-ribbon-window* (not *use-ribbon-window*)))
                                                       (if *use-ribbon-window*
                                                         (open-ribbon)
                                                         (ignore-errors (window-close (find-window "gibbon-monkey")))))))
                              (MAKE-INSTANCE 'menu-item :menu-item-title  "-" :DISABLED T))))))
  
(defmenu-raw #'make-windows-menu)

;;; ================================================================
;;; debug menu

(defparameter *enable-debug-menu* (member :CLASP-MCL-INTERFACE-DEBUG *features*))

(defmenu debug :help-spec "Debug Menu
This menu should be only be enabled in the development version."
  :predicate #'(lambda () *enable-debug-menu*))

(defmenu-item (clasp-menu :menu 'debug :title "Install CLASP Menu")
  (install-clasp-menu))

(defmenu-item (clasp-menubar :menu 'debug :title "Install CLASP Menubar")
  (install-clasp-menubar))

(defmenu-separator (:menu 'debug))

(defmenu-item (defaultmenu :menu 'debug :title  "Install Default Menubar")
  (install-default-menubar))

(defmenu-separator (:menu 'debug))

(defmenu-item (save-app :menu 'debug :title  "Save Application...")
  (eval-enqueue '(save-clasp-application)))

;;; =================================================================
;;; makes some menu items for the apple menu

(defun l-menu-item-clasp-apple-menu ()
  "Returns a list of CLASP menu items that are installed on the apple
menu, including the separator. See note in clasp-menubar-make
concerning binding this to a global variable."
  ;;
  (LIST (MAKE-INSTANCE 'menu-item :menu-item-title  "About CLASP"
                       :menu-item-action 'display-about-box)
        (MAKE-INSTANCE 'menu-item :menu-item-title  "Help"
                       :COMMAND-KEY #\?
                       :menu-item-action
                       #'(lambda ()
                           (display-help (front-window))))
        ;;
        (MAKE-INSTANCE 'menu-item :menu-item-title   "-" :DISABLED T)
        ;;
        ))

;;; =================================================================
;;; replacements for the old menu functions

(defun install-clasp-menu ()
  (install-default-menubar t)
  (install-as-menu "CLASP" *menu-defs* (l-menu-item-clasp-apple-menu)))

(defun install-clasp-menubar ()
  (LET ((APPLE-MENU *APPLE-MENU*))
    (APPLY #'REMOVE-MENU-ITEMS APPLE-MENU (MENU-ITEMS APPLE-MENU))
    (APPLY #'ADD-MENU-ITEMS APPLE-MENU (l-menu-item-clasp-apple-menu)))
  (install-menubar *menu-defs*))

(defun install-default-menubar (&optional no-query)
  (if no-query
    (set-menubar *default-menubar*)
    (eval-enqueue '(progn
                     (when (yes-or-no-p "Do you really want to install a menubar without an CLASP menus?")
                       (format *query-io* "To reinstall the CLASP menubar use (clasp-interface::install-clasp-menubar).")
                       (set-menubar *default-menubar*))))))

;;; =================================================================

;(install-clasp-menubar)

;;; *******************************************************************
;;; EOF