;;
;; partition-patch.lisp
;;

#| 
This file patches the replace-on-clauses from the clasp package
so that it will work on the Macintosh version. I don't know if any other
LISP flavors need it.

24-Aug-94 MS   Patched.

|#


(in-package "CLASP")

#+OLD-VARS
(defun replace-on-clauses (partition-clause dataset)
  "replaces .on. clauses of partition clauses with RTM supported connectives.
MCL patched version."
  (let ((operator (car partition-clause))
	new-partition-clause
	on-variables)
    (cond
     ((member operator *rtm-logical-connectives*)
      (dolist (clause (cdr partition-clause))
	(multiple-value-bind (new-clause new-on-variables)
	   (replace-on-clauses clause dataset)
	  (push new-clause new-partition-clause)
	  (setf on-variables (append new-on-variables on-variables))))
      (values (cons operator (reverse new-partition-clause))
	      (remove-duplicates on-variables :key #'second)))

;;  !!!! This is the line that blew everything !!!! I'm using a comparison on symbol-string now !!!!

     ((equal (symbol-name operator) (symbol-name '.on.))
      (if (> (length partition-clause) 2)
	  (values
	   `(.and.
	     ,@(mapcar #'(lambda (var)
			   (let ((sublist-entry
				  (get-sublist-entry var dataset)))
			     (push sublist-entry on-variables)
			     `(.==. ,(name (car sublist-entry))
				    ,(second sublist-entry))))
		       (cdr partition-clause)))
	   (remove-duplicates on-variables :key #'car))
	(let ((sublist-entry (get-sublist-entry (second partition-clause) dataset)))
	  (push sublist-entry on-variables)
	  (values
	   `(.==. ,(name (first sublist-entry)) ,(car (second sublist-entry)))
	   (remove-duplicates on-variables :key #'car))))) 
     (t (values partition-clause on-variables)))))

;;;
;;; Also a little description patch

(defmethod description ((object t))
  nil)

#|

patch to add standardize variable to the MacCLASP release 1.3.3

09-03-94  ms Created.

FIX:: place in clasp/shortcuts.lisp

|#

;;;
;;; standardize variable
;;;

(defun standardize-variable (variable)
  (let* ((data (variable-value variable))
         (m (mean data))
         (sd (standard-deviation data)))
    (add-variable-to-dataset
     (variable-dataset variable)
     (mapcar #'(lambda (number) (/ (- number m) sd))
             data)
     (format nil "Standardized ~a" (name variable)))))

(defun mul-by-constant (variable constant)
  (add-variable-to-dataset
   (variable-dataset variable)
   (mapcar #'(lambda (datum) (* datum constant))
           (variable-value variable))
   (format nil "* ~F ~a" constant (name variable))))