#|
result-labels.lisp

label lists for the clasp functions available via GUI
passed into make-clasp-command to provide informative output.

incomplete

19-Jul-94 ms     created.

|#

(in-package "CLASP-INTERFACE")

(defconstant *statistical-summary-labels*
  '("Data Length" "Minimum" "Maximum" "Range" "Median" "Mode" "Mean" "Variance" "Std. Deviation" "Interq. Range" "Skew"))
;;;
;;; test menu items
;;;

(defconstant *confidence-labels*
  '("Mean" "Lower Bound" "Upper Bound"))
(defconstant *t-test-labels*
  '("T Statistic" "Significance" "Standard Error" "DF"))
(defconstant *d-test-labels*
  '("D Statistic" "Significance"))
(defconstant *chi-counts-labels*
  '("Chi-square Statistic" "Significance"))
(defconstant *brief-lr-labels*
  '("Slope" "Intercept" "R^2" "Std Error of Slope" "P-Value"))


;;; The following two functions are provided for compatibility with older versions of the 
;;; output formatting. the guidelines for producing GUI elements should be followed and these items
;;; avoided because they suck.
#+LAME
(defun extract-variables (argument)
  "extracts the variables & datasets from the argument list given"
  (cond 
   ((typep argument 'clasp::variable) (list argument))
   ((typep argument 'clasp::dataset) (list argument))
   ((consp argument) (append (extract-variables (car argument))
                             (extract-variables (cdr argument))))
   ((null argument) '())
   (t '())))

#+LAME
(defun sexp-to-description (sexp)
  "makes a string from a sexp that describes the command"
  (let* ((command (car sexp))
         (args (extract-variables (cdr sexp)))
         (argnames (mapcar #'clasp::name args))
         (result (concatenate 'string (string-capitalize (symbol-name command))
                 " of ")))
    (loop for x in argnames do
          (setq result (concatenate 'string result x " ")))
    (string-trim " " result)))

;;;
;;; some support functions for use with clasp-commands
;;; 

(defun make-count-list (start finish)
  (let ((result '()))
    (dotimes (k (1+ (- finish start)) result)
      (setq result (append result (list (+ start k)))))))

(defun print-list-or-atom (list)
  "support function for pretty output formatting"
  (if (consp list)
    (if (>= (length list) 2)
      list
      (car list))
    list))

;;;
;;; finding row-column totals.
;;; 11/30 if you supply a contingency table, you get weighted averages ms
;;;

(defun r-totals (list &optional contingency-table)
  (if (null contingency-table)
    (mapcar #'(lambda (item) (append item (list (apply #'+ 0 item))))
            list)
    (mapcar #'(lambda (item ct-row) (append item (list (/ (reduce #'+ (mapcar #'(lambda (i1 i2) (* i1 i2)) item ct-row))
                                                          (reduce #'+ ct-row)))))
            list contingency-table)))

(defun c-totals (list &optional contingency-table)
  (if (member NIL list)
    '()
    (if (null contingency-table)
      (cons (reduce #'+ (mapcar #'car list))
            (c-totals (mapcar #'cdr list)))
      (cons (/ (reduce #'+ (mapcar #'(lambda (i1 i2) (* i1 i2)) (mapcar #'car list) (mapcar #'car contingency-table)))
               (reduce #'+ (mapcar #'car contingency-table)))
            (c-totals (mapcar #'cdr list) (if contingency-table (mapcar #'cdr contingency-table))))
      )))

(defun rc-totals (list &optional (contingency-table NIL))
  (let ((r-list (r-totals list contingency-table)))
    (if contingency-table
      (setf contingency-table (mapcar #'(lambda (row) (append row (list (reduce #'+ row))))
                                      contingency-table)))
    (append r-list (list (c-totals r-list contingency-table)))))

;; it makes me sooo mad to have to do this. MCL is a prick.

(defun remove-stupid-garbage (string)
  (let (nnl)
    (loop while (setf nnl (position #\NewLine string)) do
          (setf string (concatenate 'string
                                    (subseq string 0 nnl) " "
                                    (subseq string (position-if-not #'(lambda (e1)
                                                                        (eql #\space e1))
                                                                    string :start (1+ nnl))))))
    string))                                                                  

;;;
;;; the following functions are for making simple format functions
;;; the protocol for mac CLASP output is to prepare it using it's member
;;; named format-result-function. the simplest CLASP commands only need
;;; a simple template mapping. So, what you do is supply the make-instance
;;; format-function initarg with the currying functions, supplying the correct
;;; template & keywords, to produce a generic template mapping. there are three
;;; kinds :
;;;
;;; simple mapping (make-template-mapper template) which returns results as a annotated list
;;; table mapping  (make-table-maker template :table-type :name) which makes a table 
;;;       using the template as column or row headers depending on :table-type (:row-table or :column-table)
;;; sentence mapping (make-line-mapper template) which makes a formatted sentence describing results
;;;
;;; for more complex results, a custom format-function should be provided
;;; which takes the clasp command as an argument (and only that)
;;;
;;; a nil format function will do no result-preparation
;;; 
;;; see clasp-command.lisp for details
;;;
;;; IMPORTANT :: all formatting functions, regardless of results, must produce
;;; those results in a list, singleton or otherwise

;;SIMPLE MAPPING
(defun map-template (template arglist)
  (if (null template)
      arglist
      (let ((temp-buff template)
            (res-buff arglist)
            (prepared-results '()))
        (loop while res-buff do
              (setq prepared-results (append (list (car res-buff) 
                                                   (car temp-buff))
                                             prepared-results))
              (setq temp-buff (cdr temp-buff))
              (setq res-buff (cdr res-buff)))
        (reverse prepared-results))))

(defun make-template-mapper (template)
  #'(lambda (command) (map-template template (l-result-clasp-command command))))

;; using arg maps :
;; the arg-map is a list of descript-tags and numbers.
;; a number indicates to find the real arg in the arglist.
;; a descript-tag will find the real arg in the tagged command description and attempt to clasp::name it
;; otherwise, it will leave arg as is.
;;
;; example : "this ~A is a tag and ~5F is a result" '(tag-1 0)

(defun generate-args (arg-map clasp-command)
  "makes a list of arguments from an arg-map. see brochure for details."
  (let ((results (l-result-clasp-command clasp-command)))
    (mapcar #'(lambda (arg-id) (if (symbolp arg-id)
                                 (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command arg-id)))
                                 (if (integerp arg-id)
                                   (ignore-errors (elt results arg-id))
                                   arg-id)))
            arg-map)))

;; TABLE MAPPING
(defun map-to-table (template clasp-command &key (type :column-table) (name-format "~:(~A~)") (name-args '(COMMAND)))
  (let ((name (apply #'format nil name-format (generate-args name-args clasp-command))))
    (cond ((eq type :row-table)
           (list name
                 (make-instance 'statistical-table
                   :name name
                   :row-names '()
                   :column-names template
                   :table-data (list (l-result-clasp-command clasp-command))
                   :primary-table t)))
          ((eq type :column-table)
           (list name
                 (make-instance 'statistical-table
                   :name name
                   :row-names template
                   :column-names '()
                   :table-data (mapcar #'list (l-result-clasp-command clasp-command))
                   :primary-table t)))
          (t (list "Alert the GUI programmer that he/she screwed up."
                   (map-template template (l-result-clasp-command clasp-command)))))))

(defun make-table-maker (template &key (table-type :column-table) (name-format "~A") (name-args '(command)))
  #'(lambda (command) (map-to-table template command :type table-type :name-format name-format :name-args name-args)))

;; SENTENCE MAPPING

(defun make-result-line (clasp-command &key line-format arg-map)
  (let ((actual-args (generate-args arg-map clasp-command))
        )
    (list (make-instance 'clasp-result
            :result-text (remove-stupid-garbage (apply #'format nil line-format actual-args))
            :primary-result (car (l-result-clasp-command clasp-command))))))

(defun make-line-mapper (line-format arg-map)
  #'(lambda (command) (make-result-line command :line-format line-format :arg-map arg-map)))

;;;
;;; custom result-formatters for those hard-to-believe functions
;;;

(defun verbose-lr-formatter (clasp-command)
  "formats the results of the verbose linear regression."
  (let ((resultlist (l-result-clasp-command clasp-command))
        (xs (get-descript-tag clasp-command 'x-args))
        (ys (get-descript-tag clasp-command 'y-args))
        a-table s-table)
    (destructuring-bind 
      (slope intercept r-squared correlation sterr-slope sterr-intercept anova-table)
      resultlist
      (declare (ignore sterr-intercept))
      (list "Statistics"
            (setf s-table (make-instance 'statistical-table 
                            :name (format nil "Statistics for regression of ~:(~A~) against ~:(~A~)"
                                          (print-list-or-atom (mapcar #'name-and-dataset xs))
                                          (print-list-or-atom (mapcar #'name-and-dataset ys)))
                            :column-names '()
                            :row-names '("Slope" "Intercept" "R Squared" "Correlation" "Std. Error")
                            :table-data (mapcar #'list (list slope intercept r-squared correlation sterr-slope))))
            "Anova table"
            (setf a-table (make-instance 'statistical-table
                            :name (format nil "Anova Table for regression of ~:(~A~) against ~:(~A~)"
                                          (print-list-or-atom (mapcar #'name-and-dataset xs))
                                          (print-list-or-atom (mapcar #'name-and-dataset ys)))
                            :column-names '("DF" "Sum of Squares" "Mean Square" "F Ratio" "P")
                            :row-names    (list (clasp::name (car (get-descript-tag clasp-command 'x-args))) "Error" "Total")
                            :table-data anova-table))
            "Complete results"
            (make-instance 'composite-statistical-table
              :name (format nil "Results for regression of ~:(~A~) against ~:(~A~)"
                            (print-list-or-atom (mapcar #'name-and-dataset xs))
                            (print-list-or-atom (mapcar #'name-and-dataset ys)))
              :tables (list s-table a-table)
              :table-names '("Statistics" "Anova Table ")
              :primary-table t)))))

(defun list-to-correlations (list rows columns)
  (let ((result '())
        )
    (dotimes (x rows)
      (setq result (append result
                           (list (subseq list (* x columns) (+ (* x columns) columns))))))
    result))

(defun make-lr-stat-table (unstd betas t-stats ps)
  (if (null unstd)
    nil
    (cons (list (car unstd) (car betas) (car t-stats) (car ps)) (make-lr-stat-table (cdr unstd) (cdr betas) (cdr t-stats) (cdr ps)))))

(defun multi-lr-formatter (clasp-command)
  (let* ((resultlist (l-result-clasp-command clasp-command))
         (xs (car (get-descript-tag clasp-command 'x-args)))
         (y (car (get-descript-tag clasp-command 'y-arg)))
         (data-length (length (clasp::simplify y)))
         t1 t2 t3 cdt)
    (destructuring-bind 
      (intercept unstd-coeffs corr-table t-stats std-coeffs r-squared f-statistic unknown 
                 unused-corrs sum-reg sum-res ms-reg ms-res censored-column-indices)
      resultlist
      (declare (ignore unused-corrs unknown))
      (let* ((df-reg (length xs))
             (df-res (- data-length df-reg 1))
             (df-total (+ df-reg df-res))
             (f-stats (mapcar #'(lambda (item) (students-t-significance item df-res :both))
                              t-stats)))
        (when censored-column-indices
          (progn
            (gui-warn "There are columns which appear to be colinear in this regression. View the censored columns table for more details.")
            (setf cdt (list "Censored columns"
                            (make-instance 'censored-data-table
                               :name (format nil "Censored columns for multiple-linear-regression of ~:(~A~) against ~:(~A~)"
                                             (print-list-or-atom (name-and-dataset y))
                                             (format nil "~:(~A~)" (print-list-or-atom (mapcar #'name-and-dataset xs))))
                               :column-names (list (format nil "Censored variable~P" (length censored-column-indices)))
                               :row-names (mapcar #'princ-to-string censored-column-indices)
                               :table-data (mapcar #'list (mapcar #'(lambda (index) (clasp::name (elt xs index)))
                                                                  censored-column-indices)))))))
        (append 
         (list (setf t1 (make-instance 'statistical-table
                          :name (concatenate 'string "Correlations for regression of "
                                             (print-list-or-atom (name-and-dataset y))
                                             " against "
                                             (format nil "~:(~A~)" (print-list-or-atom (mapcar #'name-and-dataset xs))))
                         :column-names (mapcar #'clasp::name xs)
                         :row-names (cons (clasp::name y) (mapcar #'clasp::name xs))
                         :table-data (list-to-correlations corr-table (1+ (length xs)) (length xs))))
               "Regressions"
               (setf t3 (make-instance 'statistical-table
                          :name (format nil "Statistics for regression of ~:(~A~) against ~:(~A~)"
                                        (name-and-dataset y) (print-list-or-atom (mapcar #'name-and-dataset xs)))
                          :column-names '("Coefficient" "Beta" "t-Value" "p")
                          :row-names (append '("Intercept") (mapcar #'clasp::name xs))
                          :table-data (append (list (list intercept))
                                              (make-lr-stat-table unstd-coeffs std-coeffs t-stats f-stats))))
               "Anova table"
               (setf t2 (make-instance 'statistical-table
                          :name (format nil "Anova Table for regression of ~:(~A~) against ~:(~A~)"
                                        (name-and-dataset y) (print-list-or-atom (mapcar #'name-and-dataset xs)))
                          :column-names '("DF" "Sum" "Mean Square" "F" "P")
                          :row-names '("Regression" "Residual" "Total" "R Squared")
                          :table-data (list (list df-reg
                                                  sum-reg
                                                  ms-reg
                                                  f-statistic
                                                  ;; here comes P
                                                  (clasp::f-significance f-statistic df-reg df-res t))
                                            (list df-res
                                                  sum-res
                                                  ms-res)
                                            (list df-total (+ sum-reg sum-res))
                                            (list r-squared))))
               "Complete Results"
               (make-instance 'composite-statistical-table
                 :name (format nil "Results from regression on ~:(~A~) against ~:(~A~)"
                               (name-and-dataset y) (print-list-or-atom (mapcar #'name-and-dataset xs)))
                 :tables (list t1 t3 t2)
                 :table-names (list "Correlations" "Regression" "Anova Table")
                 :primary-table t))
         cdt)))))

(defun anova-one-way-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (xs (get-descript-tag clasp-command 'x-args))
        (ys (get-descript-tag clasp-command 'y-args))
        (con (car (get-descript-tag clasp-command 'confidence-intervals)))
        a-table gm-table st-table)
    (destructuring-bind 
      (anova-table group-means scheffe-table alt-sst confi-intervals groups)
      resultlist
      (declare (ignore #+INGORE confi-intervals alt-sst))
      (append 
       (if scheffe-table
         (list "Scheffe Table"
               (setf st-table 
                     (make-instance 'scheffe-table
                       :name (format nil "Scheffe tests on anova of ~:(~A~) on ~:(~A~)"
                                     (print-list-or-atom (mapcar #'name-and-dataset xs))
                                     (print-list-or-atom (mapcar #'name-and-dataset ys)))
                       :row-names (butlast (mapcar #'(lambda (numero) (format nil "~,4F" numero))
                                                   group-means))
                       :column-names (cdr (mapcar #'(lambda (numero) (format nil "~,4F" numero))
                                                  group-means))
                       :scheffe (mapcar #'(lambda (row)
                                            (append (make-list (- (length group-means) (length row) 1)
                                                               :initial-element " ")
                                                    (mapcar #'car row)))
                                        scheffe-table)
                       :significance (mapcar #'(lambda (row)
                                                 (append (make-list (- (length group-means) (length row) 1)
                                                                    :initial-element " ")
                                                         (mapcar #'cadr row)))
                                             scheffe-table)
                       :table-data (mapcar #'(lambda (row)
                                               (append (make-list (- (length group-means) (length row) 1)
                                                                  :initial-element " ")
                                                       (mapcar #'cadr row)))
                                           scheffe-table))))
         NIL)
       (list "Anova Table"
             (setf a-table (make-instance 'statistical-table
                             :name (format nil "One way anova table for ~A on ~A"
                                           (print-list-or-atom (mapcar #'name-and-dataset xs))
                                           (print-list-or-atom (mapcar #'name-and-dataset ys)))
                             :column-names '("DF" "Sum Squares" "Mean Square" "F Ratio" "P")
                             :row-names (list (print-list-or-atom (mapcar #'clasp::name xs))
                                              "Error" "Total")
                             :table-data anova-table))
             "Group Means"
             (setf gm-table (make-instance 'group-means 
                              :name (format nil "Group Means for one way anova of ~:(~A~) on ~:(~A~)"
                                            (print-list-or-atom (mapcar #'clasp::name xs))
                                            (print-list-or-atom (mapcar #'clasp::name ys)))
                              :row-names (append '("Group Mean")
                                                 (if confi-intervals
                                                   (mapcar #'(lambda (string) (format nil "~D ~A" con string))
                                                           '("Lower bound" "Upper bound"))
                                                   '()))
                              :column-names    (mapcar #'(lambda (item) (format nil "~:(~6@A~)=~5F" 
                                                                                (print-list-or-atom (mapcar #'clasp::name xs))
                                                                                item))
                                                       groups)
                              :table-data (if confi-intervals
                                            (clasp::transpose confi-intervals)
                                            (list group-means))
                              :confidence-intervals confi-intervals))
             "Complete Results"
             (make-instance 'composite-statistical-table
               :name (format nil "Results of anova of ~:(~A~) on ~:(~A~)"
                             (print-list-or-atom (mapcar #'name-and-dataset xs))
                             (print-list-or-atom (mapcar #'name-and-dataset ys)))
               :tables (append (if st-table (list st-table)) (list gm-table a-table))
               :table-names (append (if st-table '("Scheffe Table")) '("Group Means" "Anova Table"))
               :primary-table t)
             )))))

(defun anova-two-way-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (x1s (get-descript-tag clasp-command 'x-args))
        (x2s (get-descript-tag clasp-command 'x2-args))
        (ys (get-descript-tag clasp-command 'y-args))
        a-table gm-table)
    (destructuring-bind
      (anova-table group-means row-totals col-totals total col-names row-names)
      resultlist
      (declare (ignore row-totals col-totals total))
      (list "Anova Table"
            (setf a-table (make-instance 'statistical-table
                            :name (format nil "Two way anova table for ~:(~A~) on ~:(~A~) and ~:(~A~)"
                                          (print-list-or-atom (mapcar #'name-and-dataset ys))
                                          (print-list-or-atom (mapcar #'name-and-dataset x1s))
                                          (print-list-or-atom (mapcar #'name-and-dataset x2s)))
                            :column-names '("DF" "Sum Squares" "Mean Square" "F Ratio" "P")
                            :row-names (list "Interaction" (print-list-or-atom (mapcar #'clasp::name x1s))
                                             (print-list-or-atom (mapcar #'clasp::name x2s))
                                             "Error" "Total")
                            :table-data anova-table))
            "Group Means" ;; ||||
            (setf gm-table (make-instance 'group-means 
                             :name (format nil "Group Means for two way anova of ~:(~A~) on ~:(~A~) and ~:(~A~)"
                                           (print-list-or-atom (mapcar #'clasp::name ys))
                                           (print-list-or-atom (mapcar #'clasp::name x1s))
                                           (print-list-or-atom (mapcar #'clasp::name x2s)))
                             :row-names (append (mapcar #'(lambda (item) (format nil (if (integerp item)
                                                                                       "~D"
                                                                                       (if (numberp item) "~,4F" "~S"))
                                                                                 item)) 
                                                        row-names)
                                                '("Total"))
                             :column-names (append (mapcar #'(lambda (item) (format nil (if (integerp item)
                                                                                          "~D"
                                                                                          (if (numberp item) "~,4F" "~S"))
                                                                                    item))
                                                           col-names)
                                                   '("Total"))
                             ;; explanation - we need to pass a contingency table to tc-totals so we get averages
                             :table-data (rc-totals (array-to-table group-means
                                                                    (length col-names) 
                                                                    (length row-names))
                                                    ;; here is where we prep the cont. table. the hassle is that the rows
                                                    ;; and columns are different for the gm-table and the cont table we get
                                                    (multiple-value-bind (table x-list y-list)
                                                                         (clasp::make-contingency-table (clasp::simplify (car x1s))
                                                                                                        (clasp::simplify (car x2s)))
                                                      ;; convert to tabular list
                                                      (setq table (array-to-table table))
                                                      ;; sort the rows to match the column names (we will transpose later)
                                                      (setq table (sort-rows-to-match
                                                                   x-list table col-names))
                                                      ;; sort the columns to match the row names (we will transpose next)
                                                      (setq table (sort-cols-to-match
                                                                   y-list table row-names))
                                                      ;; now transpose the cont table and it will snap on top of the gm table
                                                      (clasp::transpose table)))))
            "Complete Results"
            (make-instance 'composite-statistical-table
              :name (format nil "Results of 2way ANOVA of ~:(~A~) on ~:(~A~) and ~:(~A~)"
                                          (print-list-or-atom (mapcar #'name-and-dataset ys))
                                          (print-list-or-atom (mapcar #'name-and-dataset x1s))
                                          (print-list-or-atom (mapcar #'name-and-dataset x2s)))
              :tables (list gm-table a-table)
              :table-names '("Group Means" "Anova Table")
              :primary-table t)))))

(defun anova-one-way-repeated-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (x1s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args))))
        (x2s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x2-args))))
        (ys  (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args)))))
    (list (make-instance 'statistical-table
            :name (format nil "One way RM ANOVA of ~A, ~A on ~A" x1s x2s ys)
            :column-names '("DF" "Sum Squares" "Mean Square" "F Ratio" "P")
            :row-names (list (format nil "Fixed(~A)" (subseq x1s 0 (min (length x1s) (or (position #\[ x1s) 15))))
                             (format nil "Random(~A)" (subseq x2s 0 (min (length x2s) (or (position #\[ x2s) 15))))
                             "Interaction" "Total")
            :table-data (first resultlist)
            :primary-table t))))

(defun anova-two-way-mixed-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (x1s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args))))
        (x2s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x2-args))))
        (ys  (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args)))))
    (list (make-instance 'statistical-table
            :name (format nil "Two way mixed ANOVA of ~A, ~A on ~A" x1s x2s ys)
            :column-names '("DF" "Sum Squares" "Mean Square" "F Ratio" "P")
            :row-names (list (format nil "F(~A)" (subseq x1s 0 (min (length x1s) (or (position #\[ x1s) 15))))
                             (format nil "R(~A)" (subseq x2s 0 (min (length x2s) (or (position #\[ x2s) 15))))
                             "F * R" "Within")
            :table-data (first resultlist)
            :primary-table t))))

(defun anova-two-way-repeated-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (x1s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args))))
        (x2s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x2-args))))
        (x3s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x3-args))))
        (ys  (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args)))))
    (list (make-instance 'statistical-table
            :name (format nil "Two way RM ANOVA of ~A, ~A, ~A on ~A" x1s x2s x3s ys)
            :column-names '("Sum Squares" "DF" "Mean Square" "F Ratio" "P")
            :row-names (list (format nil "F1(~A)" (subseq x1s 0 (min (length x1s) (or (position #\[ x1s) 15))))
                              "F1 * R"
                             (format nil "R(~A)" (subseq x3s 0 (min (length x3s) (or (position #\[ x3s) 15))))
                             (format nil "F2(~A)" (subseq x2s 0 (min (length x2s) (or (position #\[ x2s) 15))))
                             "F2 * R" "F1 * F2" "F1 * F2 * R" "Total")
            :table-data (first resultlist)
            :primary-table t))))

(defun anova-three-way-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (x1s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args))))
        (x2s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x2-args))))
        (x3s (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x3-args))))
        (ys  (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args)))))
    (list (make-instance 'statistical-table
            :name (format nil "Two way RM ANOVA of ~A, ~A, ~A on ~A" x1s x2s x3s ys)
            :column-names '("Sum Squares" "DF" "Mean Square" "F Ratio" "P")
            :row-names (list (format nil "X1(~A)" (subseq x1s 0 (min (length x1s) (or (position #\[ x1s) 15))))
                             (format nil "X2(~A)" (subseq x2s 0 (min (length x2s) (or (position #\[ x2s) 15))))
                             (format nil "X3(~A)" (subseq x3s 0 (min (length x3s) (or (position #\[ x3s) 15))))
                             "X1 * X2" "X1 * X3" "X2 * X3" "X1 * X2 * X3" "Within")
            :table-data (first resultlist)
            :primary-table t))))

(defun correlation-lag-formatter (clasp-command)
  "formats the results of a lagged-correlation command."
  (let ((resultlist (l-result-clasp-command clasp-command))
        (min-lag (car (get-descript-tag clasp-command 'min-lag)))
        (xs (get-descript-tag clasp-command 'x-args))
        (ys (ignore-errors (get-descript-tag clasp-command 'y-args)))
        (comm (get-descript-tag clasp-command 'command)))
    (list (make-instance 'lag-table
            :name (format nil "~:(~A~) of ~:(~A~)~:(~A~)" (print-list-or-atom comm)
                          (print-list-or-atom (mapcar #'name-and-dataset xs))
                          (if ys (format nil " vs " (print-list-or-atom (mapcar #'name-and-dataset ys))) ""))
            :column-names '("Correlation")
            :row-names (mapcar #'(lambda (item) (concatenate 'string
                                                             "Lag "
                                                             (format nil
                                                                     "~D"
                                                                     item)))
                               (make-count-list min-lag 
                                                (1- (+ min-lag (length (car resultlist))))))
            :table-data (mapcar #'list (car resultlist))
            :primary-table t))))

(defun chi-2x2-format (clasp-command)
  "format the results of a chi-square-2x2 test"
  (let ((resultlist (l-result-clasp-command clasp-command)))
    (list (make-instance 'statistical-table
            :name (format nil "Chi Square on ~A against ~A"
                          (ignore-errors (car (get-descript-tag clasp-command 'command)))
                          (ignore-errors (name-and-dataset (car (get-descript-tag clasp-command 'x-args))))
                          (ignore-errors (name-and-dataset (car (get-descript-tag clasp-command 'y-args)))))
            :row-names (append (mapcar #'(lambda (item) (format nil "~S" item)) (elt resultlist 3)) '("Totals" "Chi Square" "P"))
            :column-names (append (mapcar #'(lambda (item) (format nil "~S" item)) (elt resultlist 4)) '("Totals"))
            :table-data   (append (rc-totals (array-to-table (elt resultlist 2)
                                                             (length (elt resultlist 4)) 
                                                             (length (elt resultlist 3))))
                                  (list (list (elt resultlist 0)) (list (elt resultlist 1))))
            :primary-table t))))

(defun chi-rxc-format (clasp-command)
  "formats the results of a chi-square-rxc test"
  (let ((resultlist (l-result-clasp-command clasp-command))
        of-table st-table)
    (destructuring-bind (chi-square-stat significance g-test-stat g-test-sig g-test-dof table x-list y-list)
                        resultlist
      (multiple-value-setq (x-list table) (sort-rows x-list (array-to-table table (length y-list) (length x-list))))
      (multiple-value-setq (y-list table) (sort-cols y-list table))
      (list "Statistics"
            (setf st-table (make-instance 'statistical-table
                             :name (format nil "Statistics from Chi Square on ~A and ~A"
                                           (name-and-dataset (car (get-descript-tag clasp-command 'x-args)))
                                           (name-and-dataset (car (get-descript-tag clasp-command 'y-args))))
                             :column-names (list "Value" "P" "DF")
                             :row-names (list "Chi Square" "G Test")
                             :table-data (list (list chi-square-stat significance)
                                               (list (if (numberp g-test-stat) g-test-stat "cannot") (or g-test-sig "compute")
                                                     (or g-test-dof "g test")))))
            "Contingency Table"
            (setf of-table (make-instance 'contingency-table
                             :name (format nil "Observed freqs from Chi Square on ~A and ~A"
                                           (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args)))
                                           (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args))))
                             :y-variable (car (get-descript-tag clasp-command 'y-args))
                             :row-names (append (mapcar #'(lambda (item) (format nil "~S" item)) x-list) '("Totals"))
                             :column-names (append (mapcar #'(lambda (item) (format nil "~S" item)) y-list) '("Totals"))
                             :table-data   (rc-totals table)))
            "Complete Results"
            (make-instance 'composite-statistical-table 
              :name (format nil "Results of Chi Square on ~A against ~A"
                            (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args)))
                            (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args))))
              :tables (list of-table st-table)
              :table-names '("Frequencies" "Test Results")
              :primary-table t
              )))))

(defun ctable-format (clasp-command)
  (let ((result-list (l-result-clasp-command clasp-command)))
    (destructuring-bind (ctable v1s v2s)
                        result-list
      (multiple-value-setq (v1s ctable) (sort-rows v1s (array-to-table ctable (length v2s) (length v1s))))
      (multiple-value-setq (v2s ctable) (sort-cols v2s ctable))
      (list "Contingency Table"
            (make-instance 'contingency-table
              :name (format nil "Contingency table of ~:(~A~) against ~:(~A~)"
                                (name-and-dataset (car (get-descript-tag clasp-command 'x-args)))
                                (name-and-dataset (car (get-descript-tag clasp-command 'y-args))))
              :y-variable (car (get-descript-tag clasp-command 'y-args))
              :row-names (append (mapcar #'(lambda (item) (format nil "~S" item)) v1s) '("Totals"))
              :column-names (append (mapcar #'(lambda (item) (format nil "~S" item)) v2s) '("Totals"))
              :table-data (rc-totals ctable)
              :primary-table t)))))

(defun array-to-table (array &optional rows columns)
  "make an array into a formatted table. this is making clasp jump through hoops to be standard."
  (let ((result '())
        (buffer '()))
    (if (null rows) (setf rows (array-dimension array 1)))
    (if (null columns) (setf columns (array-dimension array 0)))
    (dotimes (x columns)
      (setq buffer '())
      (dotimes (y rows)
        (setq buffer (append buffer (list (aref array x y)))))
      (setq result (append result (list buffer))))
    result))

(defun trimmed-mean-format (clasp-command)
  "formats the results of a trimmed-mean"
  (let ((resultlist (l-result-clasp-command clasp-command)))
    (list (format nil "~S trimmed mean of ~S"
                  (car (get-descript-tag clasp-command 'percent))
                  (get-descript-tag clasp-command 'x-args))
          (car resultlist))))

(defun confidence-t-formatter (clasp-command)
  "formats the results of a confidence interval of a t statistic."
  (let ((resultlist (l-result-clasp-command clasp-command)))
    (list (make-instance 'statistical-table
            :name (format nil "~,4F Confidence Interval using t on ~:(~A~)" (print-list-or-atom (get-descript-tag clasp-command 'confidence))
                          (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args))))
            :column-names (cons "DF" *confidence-labels*)
            :row-names '()
            :table-data (list (cons (1- (clasp::data-length (clasp::simplify (car (get-descript-tag clasp-command 'x-args))))) resultlist))
            :primary-table t))))

(defun d-test-format (clasp-command)
  "formats the results of a d-test"
  (let ((resultlist (l-result-clasp-command clasp-command)))
    (list (print-list-or-atom (make-result-line clasp-command :line-format "D Test on ~:(~A~) and ~:(~A~) : D = ~5F, P = ~5F"
                                                :arg-map (list 'x-args 'y-args 0 (coerce (/ (elt resultlist 1) (elt resultlist 2)) 'float))))
;; note on the above line :
;; make-result-line is makes a great tag as well as result. Also note the float in the arg-map. any non-standard arg in the
;; arg-map is returned unmolested, so floats are printed verbatim !
          (make-instance 'statistical-table
            :name (format nil "D Test on ~:(~A~) and ~:(~A~)"
                          (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args)))
                          (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args))))
            :column-names '("D Statistic" "P")
            :row-names    '()
            :table-data   (list (list (elt resultlist 0) (coerce (/ (elt resultlist 1) (elt resultlist 2)) 'float)))
            :primary-table t))))

(defun correlation-table-format (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command))
        (varlist (car (get-descript-tag clasp-command 'X-ARGS))))
    (list (make-instance 'statistical-table
            :name (format nil "Correlation Table of ~:(~A~)" (mapcar #'name-and-dataset varlist))
            :column-names (mapcar #'(lambda (item) (format nil "~:(~A~)" (clasp::name item))) (reverse varlist))
            :row-names (mapcar #'(lambda (item) (format nil "~:(~A~)" (clasp::name item))) (reverse varlist))
            :table-data (elt resultlist 0)
            :primary-table t))))

(defun smart-mode-result (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command)))
    (if (= (length resultlist) 2)
      (make-result-line clasp-command :line-format  "~S is mode of ~:(~A~) and occurs ~D times"
                        :arg-map '(0 x-args 1))
      (make-result-line clasp-command :line-format  "~S is estimated mode of ~:(~A~)"
                        :arg-map '(0 x-args)))))

#+FBD
(defun fbd-formatter (clasp-command)
  (destructuring-bind (best-model models)
                      (l-result-clasp-command clasp-command)
    (list "Adjacency Mx"
          (make-instance 'statistical-table
            :name (format nil "FBD Adjacency Mx ~D" (slot-value best-model 'path-analysis::model-id))
            :column-names (mapcar #'cdr (slot-value (slot-value best-model 'path-analysis::model-data)
                                                    'path-analysis::names))
            :row-names (mapcar #'cdr (slot-value (slot-value best-model 'path-analysis::model-data)
                                                    'path-analysis::names))
            :table-data (make-adjacency-list (slot-value best-model 'path-analysis::matrix)))
          "Causal Model"
          best-model)))

#+FBD
(defun make-adjacency-list (adj-mx)
  (let (mx row)
    (dotimes (a (array-dimension adj-mx 0))
      (setf row NIL)
      (dotimes (b (array-dimension adj-mx 1))
        (push (if (= (aref adj-mx a b) 1) "<-'" " ") row))
      (push (reverse row) mx))
    (reverse mx)))