;;;
;;; preferences-window.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the Preferences dialog.


FIX: What other preferences do we need?

- Auto-open some results (use f-can-open-clasp-object and open-clasp-object)
  (Separate selection for graphs, data, )
- Error handling (use bin-hex example for catching)
- Copy graphs as: object, lisp, graph pict
- Copy data as: object, lisp, exportable table


================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
25-May-94 mc	Created.

|#

(in-package "CLASP-INTERFACE")

(export '(*f-confirm-quit*
          *char-file-delimiter*
          *f-include-file-labels*
          *f-auto-open-graphs*
          *font-spec-variables*
          *font-spec-datasets*
          *option-key-p*
          *x-key-p*
          *y-key-p*
          *cache-key-p*
          update-user-preferences-file
          load-user-preferences-file
          pn-user-preferences-file))


(defvar *clasp-interface-package* (find-package 'clasp-interface)) 

;;;
;;; Define variables.
;;;

(defvar *f-confirm-quit* t
  "Controls whether quit-clasp asks the user to confirm quitting
before doing so. Edited via edit-preferences .")

(defvar *char-file-delimiter* #\Tab
  "Used by the import and export menu functions to delimit non-clasp
data files. Either #\Tab or #\, .")

(defvar *f-include-file-labels* t
  "Used by the import and export menu functions to specify whether
file column labels are used.")

(defvar *f-auto-open-graphs* T
  "Used by initialize-instace of a graph type to determine whether or not
to automagically open the graph.")

(defvar *auto-open-primary-tables* T
  "The primary table may be opened on startup. The call to make-instance of a
table or composite table must specify that it is a primary table using the 
:primary-table keyword with a non-nil argument.")

(defvar *confirm-deletes* t
  "Confirm deleting variables & datasets ?")

(defvar *float-format-string* "~,5F"
  "The formatting string for printing floats into the history.")

(defvar *vars-openable* NIL
  "Double clicking will open variables & datasets.")

(defvar *log-type* #-MCG-TEST :event #+MCG-TEST :test
  "The destination for the logging feature, :FILE, :EVENT, or :TEST")

(defvar *bypass-dialogs* NIL
  "Whether or not to skip dialog boxes when there are sufficient arguments in the dock.")

(defvar *default-graph-size* (min (make-point *screen-width* *screen-height*) #@(400 320))
  "The default graph size for opening graph windows.")

(defvar *default-log-size* (make-point #@(320 200))
  "The default graph size for logging.")

(defvar *table-title-font*  '(:plain "Geneva" 10))
(defvar *table-header-font* '(:bold "Geneva" 10))
(defvar *table-entry-font*  '(:plain "Geneva" 10))
(defvar *data-view-font*    '(:plain "Geneva" 10))

(defvar *font-spec-history* '("Geneva" 10))
(defvar *font-spec-history-result* *font-spec-history*)
(defvar *font-spec-history-label*  (append '(:bold) *font-spec-history*))
(defvar *font-spec-history-command* (append '(:bold) *font-spec-history*))
(defvar *font-spec-history-comment* '("Courier" 10))

(defvar *font-spec-variables* '("Geneva" 9)
  "The font spec used by dataset-view to draw variables in.")

(defvar *font-spec-datasets* (append '(:bold) *font-spec-variables*)
  "The font spec used by dataset-view to draw datasets in.")

(defvar *caddy-font* '("Geneva" 9))

(defvar *use-ribbon-window* t)
(defvar *space-delimit-tables* NIL)

;;;
;;; The following preferences are NOT editable. I guess they're not really preferences.
;;;

(defvar *open-click-p* 'double-click-p
  "Used by the views to determine whether or not a click means to open that item.")

(defvar *x-key-p* 'control-key-p
  "Used to send an item to the CLASP x dock.")

(defvar *y-key-p* 'option-key-p
  "Used to send an item to the y dock.")

(defvar *cache-key-p* 'caps-lock-key-p
  "Used to automatically cache items moved to the dock.")

;;;
;;; Define make-preferences-dialog, used by edit-preferences .
;;;

;;; FIX: Add :help-spec to following!

(defvar *preferences-dialog* nil)

(defclass preferences-dialog (color-dialog)
  ()
  (:default-initargs
    :WINDOW-TYPE :DOUBLE-EDGE-BOX
    :VIEW-POSITION '(:TOP 60)
    :VIEW-SIZE #@(500 250)
    :CLOSE-BOX-P NIL
    :VIEW-FONT '("Palatino" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'radio-button-dialog-item :view-position #@(300 54)
                         :radio-button-cluster 1
                         :dialog-item-text "Tab"
                         :view-nick-name 'tab-delimits
                         :radio-button-pushed-p (eq *char-file-delimiter* #\Tab))
          (make-instance 'radio-button-dialog-item :view-position #@(300 70)
                         :radio-button-cluster 1
                         :dialog-item-text "Comma"
                         :view-nick-name 'comma-delimits
                         :radio-button-pushed-p (eq *char-file-delimiter* #\,))
          (make-instance 'static-text-dialog-item :view-position #@(190 54)
                         :dialog-item-text "File delimiter")
          (make-instance 'radio-button-dialog-item :view-position #@(300 6)
           :dialog-item-text "Log to file"
           :VIEW-NICK-NAME 'log-to-file 
           :dialog-item-enabled-p NIL
           :radio-button-pushed-p (eq *log-type* :file)
           )
          (make-instance 'radio-button-dialog-item :view-position #@(300 22)
                         :dialog-item-text "Log to application"
                         :view-nick-name 'log-to-event
                         :radio-button-pushed-p (eq *log-type* :event)
                         )
          #+MCG-TEST
          (make-instance 'radio-button-dialog-item :view-position #@(300 38)
                         :dialog-item-text "Log to listener"
                         :view-nick-name 'log-to-test
                         :radio-button-pushed-p (eq *log-type* :test))
          (make-instance 'static-text-dialog-item :view-position #@(190 6)
                         :dialog-item-text "Log output")
          (make-instance 'number-entry-item :view-position #@(300 90)
                         :view-nick-name 'dgs-x :view-size #@(50 16)
                         :allow-floats nil :allow-negatives nil
                         :dialog-item-text (format nil "~D" (point-h *default-graph-size*)))
          (make-instance 'number-entry-item :view-position #@(375 90)
                         :view-nick-name 'dgs-y :view-size #@(50 16)
                         :allow-floats nil :allow-negatives nil
                         :dialog-item-text (format nil "~D" (point-v *default-graph-size*)))
          (make-instance 'static-text-dialog-item :view-position #@(190 90)
                         :dialog-item-text "Def. Graph Size")
          (make-instance 'number-entry-item :view-position #@(300 115)
                         :view-nick-name 'dls-x :view-size #@(50 16)
                         :allow-floats nil :allow-negatives nil
                         :dialog-item-text (format nil "~D" (point-h *default-log-size*)))
          (make-instance 'number-entry-item :view-position #@(375 115)
                         :view-nick-name 'dls-y :view-size #@(50 16)
                         :allow-floats nil :allow-negatives nil
                         :dialog-item-text (format nil "~D" (point-v *default-log-size*)))
          (make-instance 'static-text-dialog-item :view-position #@(190 115)
                         :dialog-item-text "Def. Log Size")
          (make-instance 'font-pum :view-position #@(188 144) :view-size #@(160 18)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Table font" :proportional-p t
                         :view-nick-name 'table-font-name
                         :default (car (remove-if-not #'stringp *table-entry-font*)))
          (make-instance 'point-size-pum :view-position #@(355 144) :view-size #@(50 18)
                         :view-nick-name 'table-font-size
                         :default (car (last *table-entry-font*)))
          (make-instance 'font-style-pum :view-position #@(410 144) :view-size #@(85 18)
                         :view-nick-name 'table-font-style
                         :default (if (keywordp (car *table-entry-font*)) (car *table-entry-font*) nil))
          (make-instance 'font-pum :view-position #@(188 164) :view-size #@(160 18)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Data font" :proportional-p t
                         :view-nick-name 'dview-font-name
                         :default (car (remove-if-not #'stringp *data-view-font*)))
          (make-instance 'point-size-pum :view-position #@(355 164) :view-size #@(50 18)
                         :view-nick-name 'dview-font-size
                         :default (car (last *data-view-font*)))
          (make-instance 'font-style-pum :view-position #@(410 164) :view-size #@(85 18)
                         :view-nick-name 'dview-font-style
                         :default (if (keywordp (car *data-view-font*)) (car *data-view-font*) nil))
          (make-instance 'font-pum :view-position #@(188 184) :view-size #@(160 18)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "View font" :proportional-p t
                         :view-nick-name 'cw-font-name
                         :default (car (remove-if-not #'stringp *font-spec-variables*)))
          (make-instance 'point-size-pum :view-position #@(355 184) :view-size #@(50 18)
                         :view-nick-name 'cw-font-size
                         :default (car (last *font-spec-variables*)))
          (make-instance 'font-style-pum :view-position #@(410 184) :view-size #@(85 18)
                         :view-nick-name 'cw-font-style
                         :default (if (keywordp (car *font-spec-variables*)) 
                                    (car *font-spec-variables*) 
                                    nil))
          (make-instance 'static-text-dialog-item :dialog-item-text "Histogram bins"
                         :view-position #@(9 182))
          (make-instance 'number-entry-item :view-position #@(130 182) :view-size #@(53 16)
                         :dialog-item-text (format NIL "~D" *default-number-of-bins*)
                         :allow-floats NIL :allow-negatives NIL :view-nick-name 'hist-bins)
          (MAKE-DIALOG-ITEM 'CHECK-BOX-DIALOG-ITEM #@(9 6) #@(120 16) "Confirm Quit?"
                            'NIL
                            :VIEW-NICK-NAME 'confirm-quit-check-box-di
                            :CHECK-BOX-CHECKED-P *f-confirm-quit*)
          (MAKE-DIALOG-ITEM 'CHECK-BOX-DIALOG-ITEM #@(9 25) #@(120 16) "Include Labels?"
                            'NIL
                            :VIEW-NICK-NAME 'include-labels-check-box-di
                            :CHECK-BOX-CHECKED-P *f-include-file-labels*)
          (make-instance 'check-box-dialog-item :view-size #@(160 16)
                         :view-position #@(9 44) :dialog-item-text "Bypass dialogs?"
                         :view-nick-name 'bypass-dialogs
                         :check-box-checked-p *bypass-dialogs*)
          (MAKE-DIALOG-ITEM 'CHECK-BOX-DIALOG-ITEM #@(9 63) #@(140 16) "Auto Open Graphs?"
                        'NIL
                            :VIEW-NICK-NAME 'ao-graphs-check-box-di
                            :CHECK-BOX-CHECKED-P *f-auto-open-graphs*)
          (make-dialog-item 'check-box-dialog-item #@(9 82) #@(140 16) "Auto Open Tables?"
                            'NIL
                            :view-nick-name 'ao-tables-check-box-di
                            :check-box-checked-p *auto-open-primary-tables*)
          (make-dialog-item 'check-box-dialog-item #@(9 101) #@(160 16) "Easy Open Variables?"
                            'NIL
                            :view-nick-name 'openvars-check-box-di
                            :check-box-checked-p *vars-openable*)
          (make-dialog-item 'check-box-dialog-item #@(9 120) #@(160 16) "Confirm Deletes ?"
                            'NIL
                            :view-nick-name 'deletes-check-box-di
                            :check-box-checked-p *confirm-deletes*)
          (make-dialog-item 'check-box-dialog-item #@(9 139) #@(160 16) "Use Ribbon ?"
                            'NIL
                            :view-nick-name 'ribbon-check-box-di
                            :check-box-checked-p *use-ribbon-window*)
          (make-dialog-item 'check-box-dialog-item #@(9 158) #@(168 16) "Space delimit tables ?"
                            'NIL
                            :view-nick-name 'spaces-not-tabs
                            :check-box-checked-p *space-delimit-tables*)
          ;;
          (MAKE-DIALOG-ITEM 'BUTTON-DIALOG-ITEM #@(3 220) #@(62 16) "Cancel"
                            #'(lambda (cancel-button-di)
                                (window-hide (view-container cancel-button-di))
                                (return-from-modal-dialog nil))
                            :DEFAULT-BUTTON NIL)
          ;; Should make this only visible when there is a preferences file to load.
          (MAKE-DIALOG-ITEM 'BUTTON-DIALOG-ITEM #@(103 220) #@(62 16) "Load"
                            #'(lambda (load-button-di)
                                (load-user-preferences-file)
                                (window-hide (view-container load-button-di))
                                (return-from-modal-dialog t))
                            :view-nick-name 'load-button)
          (MAKE-DIALOG-ITEM 'BUTTON-DIALOG-ITEM #@(203 220) #@(62 16) "Use"
                            #'(lambda (ok-button-di)
                                (save-preferences-button-di ok-button-di nil)
                                (window-hide (view-container ok-button-di))
                                (return-from-modal-dialog t))
                            :DEFAULT-BUTTON T)
          (MAKE-DIALOG-ITEM 'BUTTON-DIALOG-ITEM #@(303 220) #@(94 16) "Use & Save"
                            #'(lambda (ok-button-di)
                                (save-preferences-button-di ok-button-di t)
                                (window-hide (view-container ok-button-di))
                                (return-from-modal-dialog t))
                            :DEFAULT-BUTTON NIL))))

(defmethod window-close :after ((view preferences-dialog))
  (declare (ignore view))
  (setf *preferences-dialog* nil))

(defun make-preferences-dialog ()
  "Returns a dialog to be used for editing. Call the dialog with
modal-dialog; it returns nil if cancelled and non-nil if preferences
changed."
  (if *preferences-dialog*
    (window-select *preferences-dialog*)
    (setf *preferences-dialog* 
          (with-cursor *watch-cursor*
            (MAKE-INSTANCE 'preferences-dialog)))))

(defmethod save-preferences-button-di ((ok-button-di button-dialog-item)
                                       &optional (f-update-prefs-file nil))
  "Saves the preferences specified in ok-button-di's container, which
must have been created via make-preferences-dialog . If
f-update-prefs-file is non-nil then the CLASP preferences file is
updated."
  ;;
  ;; Update memory.
  ;;
  (let* ((view-container (view-container ok-button-di))
         ;;
         (confirm-quit-check-box-di (view-named 'confirm-quit-check-box-di
                                                view-container))
         (include-labels-check-box-di (view-named 'include-labels-check-box-di
                                                  view-container))
         (ao-graphs-check-box-di (view-named 'ao-graphs-check-box-di
                                             view-container))
         (ov-check-box-di (view-named 'openvars-check-box-di view-container))
         (ao-tables-check-box-di (view-named 'ao-tables-check-box-di view-container))
         (bp-check-box-di (view-named 'bypass-dialogs view-container))
         (confirm-delete-di (view-named 'deletes-check-box-di view-container))
         (ribbon-di (view-named 'ribbon-check-box-di view-container))
         (spaces-di (view-named 'spaces-not-tabs view-container))
         ;;
         (file-button (view-named 'log-to-file view-container))
         (event-button (view-named 'log-to-event view-container))
;;         (test-button (view-named 'log-to-test view-container))  ;; unused
         (tab-button (view-named 'tab-delimits view-container))
         (comma-button (view-named 'comma-delimits view-container))
         ;;
         (dgs-x (number-item-value (view-named 'dgs-x view-container)))
         (dgs-y (number-item-value (view-named 'dgs-y view-container)))
         (dls-x (number-item-value (view-named 'dls-x view-container)))
         (dls-y (number-item-value (view-named 'dls-y view-container)))
         (hist-bins (number-item-value (view-named 'hist-bins view-container)))
         (font (menu-item-title (selected-item (view-named 'table-font-name view-container))))
         (font-size (menu-item-title (selected-item (view-named 'table-font-size view-container))))
         (font-style (menu-item-title (selected-item (view-named 'table-font-style view-container))))
         (dvfont (menu-item-title (selected-item (view-named 'dview-font-name view-container))))
         (dvfont-size (menu-item-title (selected-item (view-named 'dview-font-size view-container))))
         (dvfont-style (menu-item-title (selected-item (view-named 'dview-font-style view-container))))
         (cwfont (menu-item-title (selected-item (view-named 'cw-font-name view-container))))
         (cwfont-size (menu-item-title (selected-item (view-named 'cw-font-size view-container))))
         (cwfont-style (menu-item-title (selected-item (view-named 'cw-font-style view-container))))
         ;;
         )
    (setf *use-ribbon-window* (check-box-checked-p ribbon-di))
    (if *use-ribbon-window*
      (open-ribbon)
      (close-ribbon))
    (setf *space-delimit-tables* (check-box-checked-p spaces-di))
    (setf *default-number-of-bins* hist-bins)
    (setf *table-entry-font* (list (read-from-string font-style) font (read-from-string font-size)))
    (setf *table-header-font* (list :bold (read-from-string font-style) font (read-from-string font-size)))
    (setf *table-title-font* (list :bold (read-from-string font-style) font (1+ (read-from-string font-size))))
    (resize-cells-tb)
    (setf *data-view-font* (list (read-from-string dvfont-style) dvfont (read-from-string dvfont-size)))
    (setf *font-spec-variables* (list (read-from-string cwfont-style) cwfont
                                      (read-from-string cwfont-size)))
    (build-view-fonts) (resize-cells-cw)
    (setf *default-graph-size* (make-point dgs-x dgs-y))
    (setf *default-log-size* (make-point dls-x dls-y))
    (setf *char-file-delimiter*
          (cond ((radio-button-pushed-p tab-button) #\Tab)
                ((radio-button-pushed-p comma-button) #\,)
                (t
                 ;; Following case should never happen...
                 *char-file-delimiter*)))
    (setf *log-type*
          (cond ((radio-button-pushed-p file-button) :file)
                ((radio-button-pushed-p event-button) :event)
                (t :test)))
    (setf *bypass-dialogs* (check-box-checked-p bp-check-box-di))
    (setf *f-confirm-quit* (check-box-checked-p confirm-quit-check-box-di))
    (setf *f-include-file-labels* (check-box-checked-p include-labels-check-box-di))
    (setf *f-auto-open-graphs* (check-box-checked-p ao-graphs-check-box-di))
    (setf *auto-open-primary-tables* (check-box-checked-p ao-tables-check-box-di))
    (setf *vars-openable* (check-box-checked-p ov-check-box-di))
    (setf *confirm-deletes* (check-box-checked-p confirm-delete-di))
    )
  ;;
  ;; Update the file if necessary.
  ;;
  (when f-update-prefs-file
    (update-user-preferences-file)))

(defparameter *preferences-file-creator* "CLSP")
(defparameter *preferences-file-type* "PREF")

(defun update-user-preferences-file ()
  "Saves the current user preferences in the CLASP preferences file."
  ;;
  (let ((pn-user-preferences-file (pn-user-preferences-file :create))
        (sexp-preferences-reload-form (sexp-preferences-reload-form)))
    (with-open-file (stream pn-user-preferences-file :direction :output
                            :if-exists :supersede
                            :if-does-not-exist :create)
      (write `(in-package ,(clasp::form-keyword (package-name *clasp-interface-package*)))
             :stream stream :escape t)
      (terpri stream)
      (terpri stream)
      (write sexp-preferences-reload-form :stream stream :pretty t :escape t))
    (set-mac-file-creator pn-user-preferences-file *preferences-file-creator*)
    (set-mac-file-type pn-user-preferences-file *preferences-file-type*)
    ;; Return the pathname instead of the possibly long form.
    pn-user-preferences-file))


(defun load-user-preferences-file ()
  "Loads the current user preferences from the CLASP preferences file."
  ;;
  ;(ignore-errors 
   (let ((pn-user-preferences-file (pn-user-preferences-file :find))
         (*package* *clasp-interface-package*))
     (when pn-user-preferences-file
       #+MAYBE-LATER
       (with-open-file (stream pn-user-preferences-file :direction :input)
         (loop for form = (read stream nil :EOF-VALUE) 
               while (not (eq form :EOF-VALUE)) do
               (eval form)))
       (set-mac-file-type pn-user-preferences-file "TEXT")
       (prog1 (load pn-user-preferences-file)
         (set-mac-file-type pn-user-preferences-file *preferences-file-type*)))))
;)

(defun locate-preferences (&key (all-drives t) (directory "") (action :find))
  ;; Search through their list of drives and hope they do not
  ;; have a spurious system folder.
  (flet ((check-drive (drive)
           (case action
             (:find (probe-file (concatenate 'string 
                                             drive
                                             directory
                                             "CLASP Preferences")))
             ;; Just make sure the directory is there.
             (:create 
              (if (probe-file (concatenate 'string 
                                           drive
                                           directory))
                (pathname
                 (concatenate 'string 
                              drive
                              directory
                              "CLASP Preferences")))))))
    (if (not all-drives)
      (check-drive "")
      (loop for drive-num from 0
            for drive-name = (ignore-errors (drive-name drive-num))
            until (null drive-name)
            for file = (check-drive (namestring drive-name))
            until file
            finally (return (or file nil))))))

;; I think there is a built in CCL function that will return the pathname
;; of the system folder - we should use it.
(defvar *preferences-directory-path* "System Folder:Preferences:")

(defun pn-user-preferences-file (&optional (action :Find))
  "Returns the pathname to save the user preferences in."

  (or 
   ;; Look in the system preferences directory
   (locate-preferences :all-drives t   :directory *preferences-directory-path* :action :find)
   ;; Look in the local directory
   (locate-preferences :all-drives nil :directory ""                           :action :find)
   ;; And if they really need a new place to put it suggest these.
   (and (eq action :create)
        (or 
         (locate-preferences :all-drives t   :directory *preferences-directory-path* :action :create)
         (locate-preferences :all-drives nil :directory ""                           :action :create)))))

(defun sexp-preferences-reload-form ()
  "Returns a sexp that, when evaluated, will recreate current user
preferences."
  ;;
  (append
   `(setf *f-confirm-quit* ,*f-confirm-quit*
          *char-file-delimiter* ,*char-file-delimiter* 
          *f-include-file-labels* ,*f-include-file-labels*
          *f-auto-open-graphs* ,*f-auto-open-graphs*
          *auto-open-primary-tables* ,*auto-open-primary-tables*
          *log-type* ,*log-type*
          *vars-openable* ,*vars-openable*
          *bypass-dialogs* ,*bypass-dialogs*
          *space-delimit-tables* ,*space-delimit-tables*
          *confirm-deletes* ,*confirm-deletes*
          *use-ribbon-window* ,*use-ribbon-window*
          *default-graph-size* ,*default-graph-size*
          *default-log-size* ,*default-log-size*
          *default-number-of-bins* ,*default-number-of-bins*
          *table-entry-font* ,(cons 'list *table-entry-font*)
          *table-header-font* ,(cons 'list *table-header-font*)
          *table-title-font* ,(cons 'list *table-title-font*)
          *font-spec-variables* ,(cons 'list *font-spec-variables*)
          *font-spec-datasets* ,(cons 'list *font-spec-datasets*)
          *font-spec-history* ,(cons 'list *font-spec-history*)
          *font-spec-history-result* ,(cons 'list *font-spec-history-result*)
          *font-spec-history-label* ,(cons 'list *font-spec-history-label*)
          *font-spec-history-command* ,(cons 'list *font-spec-history-command*)
          *font-spec-history-comment* ,(cons 'list *font-spec-history-comment*)
          *caddy-font* ,(cons 'list *caddy-font*))
   (if *_licensed_*
     `(*_licensed_* ,*_licensed_*) NIL)))

;; automatically load the preferences file

;;(load-user-preferences-file)