;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                          Statistics Functions                          *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1996 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.
;;;
;;;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  05-30-95 File Created (schmill)
;;;  03-22-96 `pop-up-menus/ are no longer dialog-items so changed 
;;;           :dialog-item-enabled-p to :enabledp in `variable-pop-up-menu' and
;;;           :dialog-item-text to :menu-title `pop-up-menu' in initialization 
;;;           arguments (and did a bunch before as well).  - Westy
;;;
;;;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP-INTERFACE)

;;;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; here is the interface code, that should be moved to the test fns interface file

;;; start with the contrast-dialog-thingies

(defclass group-dropper (variable-stdi)
  ()
  (:documentation "A variable-stdi that will make things happen when it gets a drop."))

(defmethod receive-drop :around ((view group-dropper) (object clasp::variable) (pt-global integer))
  (if (eql (variable-vset-stdi view) object)
    (call-next-method)
    (let ((group-list (view-named 'group-lister (view-container view)))
          (cmen (view-named 'contrasts (view-container view))))
      (call-next-method)
      (deselect-cells group-list)
      (set-table-sequence group-list (clasp::extract-unique-values (clasp::simplify object)))
      ;; clear the contrasts that were already there
      (apply #'remove-menu-items cmen (menu-items cmen))
      (invalidate-view cmen t)
      (set-table-sequence (view-named 'contrast-g1b (view-container view)) NIL)
      (set-table-sequence (view-named 'contrast-g2b (view-container view)) NIL))))
  
(defun delete-contrast-items (di)
  (let* ((celts (mapcar #'point-v (selected-cells di))))
    (labels ((dismember (list bl count)
               (cond ((null list) '())
                     ((member count bl) (dismember (cdr list) bl (1+ count)))
                     (t (cons (car list) (dismember (cdr list) bl (1+ count)))))))
      (set-table-sequence di (dismember (table-sequence di) celts 0))
      (deselect-cells di))))

(defclass contrast (menu-item)
  ((neg-gps :initform NIL :initarg :neg-gps :accessor contrast-neg-gps)
   (pos-gps :initform NIL :initarg :pos-gps :accessor contrast-pos-gps)))

(defmethod print-object ((obj contrast) stream)
  (let ((pl (loop for m in (contrast-pos-gps obj) collect
                  (format nil "(~,2F)~A" (/ 1.0 (length (contrast-pos-gps obj))) m)))
        (nl (loop for m in (contrast-neg-gps obj) collect
                  (format nil "(~,2F)~A" (/ 1.0 (length (contrast-neg-gps obj))) m))))
    (cond
     ((not (or nl pl)) "Null Contrast")
     (t (format stream "~{~A~^ + ~}~:[ ~; - ~]~{~A~^ - ~}" pl nl nl)))))

(defmethod contrast->weight-alist ((obj contrast))
  (append (loop for m in (contrast-pos-gps obj) collect
                (cons m (/ 1.0 (length (contrast-pos-gps obj)))))
          (loop for m in (contrast-neg-gps obj) collect
                (cons m (/ -1.0 (length (contrast-neg-gps obj)))))))

(defmethod initialize-instance :after ((obj contrast) &key)
  (let ((pl (loop for m in (contrast-pos-gps obj) collect
                  (format nil "(~,2F)~A" (/ 1.0 (length (contrast-pos-gps obj))) m)))
        (nl (loop for m in (contrast-neg-gps obj) collect
                  (format nil "(~,2F)~A" (/ 1.0 (length (contrast-neg-gps obj))) m))))
    (set-menu-item-title obj
                         (cond
                          ((not (or nl pl)) "Null Contrast")
                          (t (format nil "~{~A~^ + ~}~:[ ~; - ~]~{~A~^ - ~}" pl nl nl))))))

(defclass contrast-toolbox-dialog (dialog statistic-dialog-mixin)
  ((current-contrast :initform (make-instance 'contrast) :accessor current-contrast))
  (:default-initargs
    :window-title "Contrast Toolbox"
    :window-type :document
    :help '*contrast-toolbox-help*
    :view-size #@(320 235)
    :view-position (best-spot 320 235)
    :view-subviews
    (list (make-instance 'static-text-dialog-item :dialog-item-text "GV"
                         :view-position #@(5 5))
          (make-instance 'group-dropper :view-position #@(30 5) :view-size #@(120 15)
                         :view-nick-name 'x-var-item)
          (make-instance 'sequence-dialog-item :view-position #@(0 40) :view-size #@(160 160) :selection-type :disjoint
                         :view-nick-name 'group-lister :table-sequence NIL)
          (make-instance 'static-text-dialog-item :dialog-item-text "DV"
                         :view-position #@(5 22))
          (make-instance 'variable-stdi :view-position #@(30 22) :view-size #@(120 12) :view-nick-name 'y-var-item)
          (make-instance 'pop-up-menu #+CCL-3 :menu-title #-CCL-3 :dialog-item-text "" :view-position #@(165 22) :view-size #@(150 20)
                         :view-nick-name 'contrasts)
          (make-instance 'button-dialog-item :view-position #@(185 45) :border-p t :dialog-item-text "Insert Contrast"
                         :dialog-item-action #'(lambda (button) (insert-contrast (view-container button))))
          (make-instance 'button-dialog-item :view-position #@(180 65) :border-p t :dialog-item-text "Replace Contrast"
                         :dialog-item-action #'(lambda (button) (replace-contrast (view-container button))))
          (make-instance 'button-dialog-item :view-position #@(185 85) :border-p t :dialog-item-text "Delete Contrast"
                         :dialog-item-action #'(lambda (button) (delete-contrast (view-container button))))
          (make-instance 'static-text-dialog-item :dialog-item-text "Contrast"
                         :view-position #@(205 105))
          (make-instance 'sequence-dialog-item :table-sequence NIL :view-position #@(165 122) :selection-type :disjoint
                         :view-size #@(75 60) :view-nick-name 'contrast-g1b)
          (make-instance 'button-dialog-item :border-p NIL :view-position #@(162 185) :view-size #@(30 15)
                         :dialog-item-text "Add" :dialog-item-action
                         #'(lambda (item)
                             (add-contrast-groups (view-container item) 'contrast-g1b)))
          (make-instance 'button-dialog-item :border-p NIL :view-position #@(195 185) :view-size #@(30 15)
                         :dialog-item-text "Del" :dialog-item-action
                         #'(lambda (item)
                             (delete-contrast-items (view-named 'contrast-g1b (view-container item)))))
          (make-instance 'sequence-dialog-item :table-sequence NIL :view-position #@(241 122) :selection-type :disjoint
                         :view-size #@(75 60) :view-nick-name 'contrast-g2b)
          (make-instance 'button-dialog-item :border-p NIL :view-position #@(234 185) :view-size #@(30 15)
                         :dialog-item-text "Add" :dialog-item-action
                         #'(lambda (item)
                             (add-contrast-groups (view-container item) 'contrast-g2b)))
          (make-instance 'button-dialog-item :border-p NIL :view-position #@(267 185) :view-size #@(30 15)
                         :dialog-item-text "Del" :dialog-item-action
                         #'(lambda (item)
                             (delete-contrast-items (view-named 'contrast-g2b (view-container item)))))
          (make-instance 'button-dialog-item :dialog-item-text "Test Significance"
                         :view-position #@(5 210) :dialog-item-action
                         #'(lambda (button) (compute-ab-significance (view-container button))))          
          )))

(defmethod delete-contrast ((dialog contrast-toolbox-dialog))
  (let* ((cm (view-named  'contrasts dialog)))
    (remove-menu-items cm (selected-item cm))
    (invalidate-view cm t)))

(defmethod insert-contrast ((dialog contrast-toolbox-dialog))
  (let* ((cm (view-named  'contrasts dialog))
         (pg (table-sequence (view-named 'contrast-g1b dialog)))
         (ng (table-sequence (view-named 'contrast-g2b dialog)))
         (nc (make-instance 'contrast :pos-gps pg :neg-gps ng)))
    (add-menu-items cm nc)
    (invalidate-view cm t)))

(defmethod replace-contrast ((dialog contrast-toolbox-dialog))
  (delete-contrast dialog)
  (insert-contrast dialog))

(defun tc () (setq mr* (make-instance 'contrast-toolbox-dialog)))

(defmethod add-contrast-groups ((dialog contrast-toolbox-dialog) which)
  (let ((dest (view-named which dialog))
        (sitems (map 'list #'(lambda (cell)
                               (elt (table-sequence (view-named 'group-lister dialog)) (point-v cell)))
                     (selected-cells (view-named 'group-lister dialog)))))
    (when sitems
      (set-table-sequence dest (remove-duplicates (append (table-sequence dest) sitems))))))
  
(defmethod compute-ab-significance ((dialog contrast-toolbox-dialog))
  (let ((contrasts (mapcar #'contrast->weight-alist (menu-items (view-named 'contrasts dialog)))))
    (multiple-value-bind (orth-p why) (apply #'clasp::orthogonal-contrasts-p contrasts)
      (unless orth-p
        (gui-warn why)
        ;; perhaps eventually we can go ahead with it
        (return-from compute-ab-significance))
      ;; perform the computation
      (let ((clasp-command (make-clasp-command `(apply #'clasp::contrast-significance-unequal-cells-args
                                                       (clasp::simplify ,(variable-vset-stdi (view-named 'x-var-item dialog)))
                                                       (clasp::simplify ,(variable-vset-stdi (view-named 'y-var-item dialog)))
                                                       ',contrasts)
                                               :format-function 'contrast-formatter
                                               :description (list (list 'command 'clasp::contrast-significance)
                                                                  (list 'x-args (variable-vset-stdi (view-named 'x-var-item dialog)))
                                                                  (list 'y-args (variable-vset-stdi (view-named 'x-var-item dialog)))
                                                                  (list 'contrasts contrasts)))))
        (run-clasp-command clasp-command :str-message "computing a/b contrast significance...."
                           :f-update-dataset-view t)))))

(defun contrast-formatter (clasp-command)
  (let ((resultlist (l-result-clasp-command clasp-command)))
    (destructuring-bind (cvals sscs dfcs mscs fcs pcs ssw dfw msw ssb dfb msb hm) resultlist
      (declare (ignore ssb dfb msb hm))
      (list "Contrast Results"
            (make-instance 'statistical-table
              :name (format nil "Contrast ~A within ~A" 
                            (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'y-args)))
                            (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag clasp-command 'x-args))))
              :row-names (append (mapcar #'princ-to-string (make-count-list 1 (length cvals))) '("Within"))
              :column-names (list "Contrast" "Sum Squares" "DF" "Mean Squares" "F" "P")
              :table-data (append (clasp::transpose (list cvals sscs dfcs mscs fcs pcs)) (list (list "" ssw dfw msw)))
              :primary-table t)))))

;;;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; the new ANOVA workshop

(defclass anova-toolbox-dialog (dialog statistic-dialog-mixin)
  ((current-contrast :initform (make-instance 'contrast) :accessor current-contrast))
  (:default-initargs
    :window-title "ANOVA Toolbox"
    :window-type :document
    :help '*anova-toolbox-help*
    :view-size #@(600 140)
    :view-position (best-spot 600 140)
    :view-subviews
    (list (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X1(S)"
                         :view-nick-name 'x-var-item :view-position #@(5 5)
                         :view-size #@(200 20))
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X2(S)"
                         :view-nick-name 'x2-var-item :view-position #@(5 28) #+CCL-3 :enabledp #-CCL-3 :dialog-item-enabled-p NIL
                         :view-size #@(200 20))
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X3(S)"
                         :view-nick-name 'x3-var-item :view-position #@(5 51) #+CCL-3 :enabledp #-CCL-3 :dialog-item-enabled-p NIL
                         :view-size #@(200 20))
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(S)"
                         :view-nick-name 'y-var-item :view-position #@(5 74) :allow-categorical-p NIL
                         :view-size #@(200 20))
          (make-instance 'disabler-checkbox-item :view-position #@(5 97) :view-size #@(158 20)
                         :dialog-item-text "Confidence Intervals" :watches-over '(confidence-value)
                         :view-nick-name 'confi-enable-cbdi)
          (make-instance 'number-entry-item :view-position #@(180 99) :view-size #@(40 16)
                         :allow-negatives NIL :allow-floats t
                         :view-nick-name 'confidence-value :dialog-item-text "0.95")
          (make-instance 'check-box-dialog-item :view-position #@(5 120) :dialog-item-text "Scheffe Tests?"
                         :view-nick-name 'scheffe-enabled-cbdi :check-box-checked-p t)
          (make-instance 'radio-button-dialog-item :dialog-item-text "One Way" :view-nick-name 'one-way
                         :view-position #@(290 5)
                         :dialog-item-action #'(lambda (button)
                                                 (fiddle-with (view-container button) 
                                                              '(x3-var-item x2-var-item)
                                                              '(x-var-item y-var-item scheffe-enabled-cbdi confi-enable-cbdi)
                                                              '((x-var-item "X1(S)")))))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Two Way Factorial" :view-nick-name 'two-way-fac
                         :view-position #@(290 25)
                         :dialog-item-action #'(lambda (button)
                                                 (fiddle-with (view-container button) 
                                                              '(x3-var-item scheffe-enabled-cbdi confi-enable-cbdi)
                                                              '(x-var-item x2-var-item y-var-item)
                                                              '((x-var-item "X1(S)") (x2-var-item "X2(S)")))))
          (make-instance 'radio-button-dialog-item :dialog-item-text "One Way Repeated Measures" :view-nick-name 'one-way-rep
                         :view-position #@(290 45)
                         :dialog-item-action #'(lambda (button)
                                                 (fiddle-with (view-container button) 
                                                              '(x3-var-item confi-enable-cbdi)
                                                              '(x-var-item x2-var-item y-var-item scheffe-enabled-cbdi)
                                                              '((x-var-item "F(S)") (x2-var-item "R(S)")))))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Two Way Mixed" :view-nick-name 'two-way-mixed
                         :view-position #@(290 65)
                         :dialog-item-action #'(lambda (button)
                                                 (fiddle-with (view-container button) 
                                                              '(x3-var-item scheffe-enabled-cbdi confi-enable-cbdi)
                                                              '(x-var-item x2-var-item y-var-item)
                                                              '((x-var-item "F(S)") (x2-var-item "R(S)")))))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Two Way w/Two Repeated Measures" :view-nick-name 'two-way-trep
                         :view-position #@(290 85)
                         :dialog-item-action #'(lambda (button)
                                                 (fiddle-with (view-container button) 
                                                              '(confi-enable-cbdi scheffe-enabled-cbdi)
                                                              '(x-var-item x2-var-item x3-var-item y-var-item)
                                                              '((x-var-item "F1(S)") (x2-var-item "F2(S)")
                                                                (x3-var-item "R(s)")))))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Three Way Factorial" :view-nick-name 'three-way-fac
                         :view-position #@(290 105)
                         :dialog-item-action #'(lambda (button)
                                                 (fiddle-with (view-container button)
                                                              '(confi-enable-cbdi scheffe-enabled-cbdi)
                                                              '(x-var-item x2-var-item x3-var-item y-var-item)
                                                              '((x-var-item "X1(S)") (x2-var-item "X2(S)")
                                                                (x3-var-item "X3(s)")))))
          (make-instance 'compute-button-di :view-position #@(500 115)))))

(defmethod f-can-compute-statistics-dialog ((dialog anova-toolbox-dialog))
  (let ((analysis (cond ((radio-button-pushed-p (view-named 'one-way dialog)) 'anova-one-way)
                        ((radio-button-pushed-p (view-named 'two-way-fac dialog)) 'anova-two-way-variables-unequal-cell-sizes)
                        ((radio-button-pushed-p (view-named 'one-way-rep dialog)) 'anova-one-way-repeated-measures)
                        ((radio-button-pushed-p (view-named 'two-way-trep dialog)) 'anova-two-way-repeated-measures)
                        ((radio-button-pushed-p (view-named 'two-way-mixed dialog)) 'anova-two-way-mixed)
                        ((radio-button-pushed-p (view-named 'three-way-fac dialog)) 'anova-three-way))))
    (case analysis
      (anova-one-way
       (let ((ci (or (and (check-box-checked-p (view-named 'confi-enable-cbdi dialog)) 
                          (number-item-value (view-named 'confidence-value dialog)))
                     0.0)))
       (values (and (l-variables-var-pum (view-named 'x-var-item dialog))
                    (l-variables-var-pum (view-named 'y-var-item dialog))
                    (>= ci 0.0) (<= ci 1.0))
               (if (or (>= ci 0.0) (<= ci 1.0)) "Invalid confidence intervals specified"
                   "One way ANOVA requires X and Y variables"))))
      (anova-one-way-repeated-measures
       (values (and (l-variables-var-pum (view-named 'x-var-item dialog)) (l-variables-var-pum (view-named 'x2-var-item dialog))
                    (l-variables-var-pum (view-named 'y-var-item dialog)))
               "One way RM ANOVA requires fixed, random, and Y variables"))
      (anova-two-way-variables-unequal-cell-sizes
       (values (and (l-variables-var-pum (view-named 'x-var-item dialog)) (l-variables-var-pum (view-named 'x2-var-item dialog))
                    (l-variables-var-pum (view-named 'y-var-item dialog)))
               "Two way factorial ANOVA requires two X factors and a Y variable"))
      (anova-two-way-repeated-measures
       (values (and (l-variables-var-pum (view-named 'x-var-item dialog)) (l-variables-var-pum (view-named 'x2-var-item dialog))
                    (l-variables-var-pum (view-named 'x3-var-item dialog)) (l-variables-var-pum (view-named 'y-var-item dialog)))
               "Two way ANOVA with two repeated measures requires two fixed factors, a random factor, and a Y variable"))
      (anova-two-way-mixed
       (values (and (l-variables-var-pum (view-named 'x-var-item dialog)) (l-variables-var-pum (view-named 'x2-var-item dialog))
                    (l-variables-var-pum (view-named 'y-var-item dialog)))
               "Two way mixed ANOVA requires two X factors and a Y variable"))
      (anova-three-way
       (values (and (l-variables-var-pum (view-named 'x-var-item dialog)) (l-variables-var-pum (view-named 'x2-var-item dialog))
                    (l-variables-var-pum (view-named 'x3-var-item dialog)) (l-variables-var-pum (view-named 'y-var-item dialog)))
               "Three way factorial ANOVA requires three X factors and a Y variable."))
      (otherwise (values NIL "This analysis is not yet supported")))))

;; we use this parameter as a trick to SUBSEQ out the arguments we need
(defparameter *anova-args-alist* '((clasp::anova-one-way . 2)
                                   (clasp::anova-one-way-repeated-measures . 3)
                                   (clasp::anova-two-way-variables-unequal-cell-sizes . 3)
                                   (clasp::anova-two-way-mixed . 3)
                                   (clasp::anova-two-way-repeated-measures . 4)
                                   (clasp::anova-three-way . 4)))

(defun scheffe-able (analysis)
  (member analysis '(clasp::anova-one-way clasp::anova-one-way-repeated-measures)))

(defun confi-able (analysis)
  (member analysis '(clasp::anova-one-way)))

(defparameter *analysis-formatter-alist* '((clasp::anova-one-way . anova-one-way-formatter)
                                           (clasp::anova-one-way-repeated-measures . anova-one-way-repeated-formatter)
                                           (clasp::anova-two-way-mixed . anova-two-way-mixed-formatter)
                                           (clasp::anova-two-way-variables-unequal-cell-sizes . anova-two-way-formatter)
                                           (clasp::anova-two-way-repeated-measures . anova-two-way-repeated-formatter)
                                           (clasp::anova-three-way . anova-three-way-formatter)))

(defmethod compute-statistics-dialog ((dialog anova-toolbox-dialog))
  (let* ((analysis (cond ((radio-button-pushed-p (view-named 'one-way dialog)) 'clasp::anova-one-way)
                         ((radio-button-pushed-p (view-named 'two-way-fac dialog)) 'clasp::anova-two-way-variables-unequal-cell-sizes)
                         ((radio-button-pushed-p (view-named 'one-way-rep dialog)) 'clasp::anova-one-way-repeated-measures)
                         ((radio-button-pushed-p (view-named 'two-way-trep dialog)) 'clasp::anova-two-way-repeated-measures)
                         ((radio-button-pushed-p (view-named 'two-way-mixed dialog)) 'clasp::anova-two-way-mixed)
                         ((radio-button-pushed-p (view-named 'three-way-fac dialog)) 'clasp::anova-three-way)))
         (x1s (l-variables-var-pum (view-named 'x-var-item dialog)))
         (x2s (l-variables-var-pum (view-named 'x2-var-item dialog)))
         (x3s (l-variables-var-pum (view-named 'x3-var-item dialog)))
         (ys  (l-variables-var-pum (view-named 'y-var-item dialog)))
         (req-args (cdr (assoc analysis *anova-args-alist*)))
         (scheffe (check-box-checked-p (view-named 'scheffe-enabled-cbdi dialog)))
         (confidence (and (check-box-checked-p (view-named 'confi-enable-cbdi dialog))
                          (number-item-value (view-named 'confidence-value dialog)))))
    (labels ((compute-anova-recur (a x1 x2 x3 y)
               (unless (some #'null (subseq (list x1 y x2 x3) 0 req-args))
                 (compute-anova-analysis a (car x1) (car x2) (car x3) (car y) req-args scheffe confidence)
                 (compute-anova-recur a (cdr x1) (cdr x2) (cdr x3) (cdr y)))))
      (compute-anova-recur analysis x1s x2s x3s ys))))

(defun compute-anova-analysis (analysis x1 x2 x3 y args scheffe confidence)
  (let ((clasp-command (make-clasp-command
                        (append (cons analysis
                                      (mapcar #'(lambda (vvn) `(clasp::simplify ,vvn))
                                              (append (subseq (list x1 x2 x3) 0 (1- args)) (list y))))
                                (if (scheffe-able analysis)
                                  (list scheffe) NIL)
                                (if (confi-able   analysis)
                                  (list confidence) NIL))
                        :format-function (cdr (assoc analysis *analysis-formatter-alist*))
                        :description (cons (list 'command analysis)
                                           (mapcar #'(lambda (tag var) (list tag var))
                                                   (append (subseq '(x-args x2-args x3-args) 0 (1- args)) '(y-args))
                                                   (append (subseq (list x1 x2 x3) 0 (1- args)) `(,y)))))))
    (run-clasp-command clasp-command :str-message "Computing analysis of variance..."
                       :f-update-dataset-view t)))

(defmethod fiddle-with ((dialog anova-toolbox-dialog) disables enables retitles &optional actions)
  (loop for vn in disables do
        (dialog-item-disable (view-named vn dialog)))
  (loop for vn in enables do
        (dialog-item-enable (view-named vn dialog)))
  (loop for rt in retitles do
        (let ((view (view-named (car rt) dialog)))
          (typecase view
            (pop-up-menu (set-menu-title view (second rt)))
            (t (set-dialog-item-text view (second rt))))
          (invalidate-view view)))
  (mapc #'eval actions))

;;;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; the 3d contingency-table analysis stuff

(defclass analyze-3d-dialog (dialog statistic-dialog-mixin)
  ()
  (:default-initargs 
    :window-title "3d Contingency Table Analysis"
    :window-type :document
    :help '*3d-ct-help*
    :view-size #@(500 115)
    :view-position (best-spot 500 115)
    :view-subviews
    (list (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(S)"
                         :view-nick-name 'x-var-item :view-position #@(5 5) :view-size #@(200 20))
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(S)"
                         :view-nick-name 'y-var-item :view-position #@(5 28) :view-size #@(200 20))
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Z(S)"
                         :view-nick-name 'z-var-item :view-position #@(5 51) :view-size #@(200 20))
          (make-instance 'disabler-checkbox-item  :view-position #@(305 5) :view-size #@(180 20)
                         :dialog-item-text "Complete Independence" :watches-over '(one-factor? conditional? homogenous? hom-reps hom-tag)
                         :check-box-checked-p t :view-nick-name 'complete?)
          (make-instance 'disabler-checkbox-item :view-position #@(305 25) :view-size #@(180 20)
                         :dialog-item-text "Singular Independence" :watches-over '(conditional? homogenous? hom-reps hom-tag) 
                         :check-box-checked-p t :view-nick-name 'one-factor?)
          (make-instance 'disabler-checkbox-item  :view-position #@(305 45) :view-size #@(190 20)
                         :dialog-item-text "Conditional Independence" :watches-over '(homogenous? hom-reps hom-tag) :check-box-checked-p t
                         :view-nick-name 'conditional?)
          (make-instance 'disabler-checkbox-item :view-position #@(305 65) :view-size #@(190 20)
                         :dialog-item-text "Homogenous Association" :watches-over '(hom-reps hom-tag) :check-box-checked-p t
                         :view-nick-name 'homogenous?)
          (make-instance 'static-text-dialog-item :dialog-item-text "HA Iterations" :view-position #@(325 88)
                         :view-nick-name 'hom-tag)
          (make-instance 'number-entry-item :dialog-item-text "6" :allow-negatives NIL :allow-floats NIL
                         :view-position #@(420 88) :view-size #@(50 16) :view-nick-name 'hom-reps)
          (make-instance 'static-text-dialog-item
                         :dialog-item-text "P Threshold" :view-position #@(120 88))
          (make-instance 'number-entry-item :dialog-item-text "0.10" :view-nick-name 'p-threshold 
                         :view-position #@(210 88) :view-size #@(50 16) :allow-negatives NIL)
          (make-instance 'compute-button-di :view-position #@(40 88)))))

(defmethod f-can-compute-statistics-dialog ((dialog analyze-3d-dialog))
  (let ((vstat (and (l-variables-var-pum (view-named 'x-var-item dialog))
                    (l-variables-var-pum (view-named 'y-var-item dialog))
                    (l-variables-var-pum (view-named 'z-var-item dialog))))
        (pstat (and (>= (number-item-value (view-named 'p-threshold dialog)) 0)
                    (>= (number-item-value (view-named 'p-threshold dialog)) 0))))
    (values (and vstat pstat) (if pstat "Must supply 3 dimensions of analysis" "Cutoff P value must be in the range 0..1"))))

(defmethod compute-statistics-dialog ((dialog analyze-3d-dialog))
  (let ((xs (l-variables-var-pum (view-named 'x-var-item dialog)))
        (ys (l-variables-var-pum (view-named 'y-var-item dialog)))
        (zs (l-variables-var-pum (view-named 'z-var-item dialog)))
        (pc (number-item-value (view-named 'p-threshold dialog)))
        (co (check-box-checked-p (view-named 'complete? dialog)))
        (of (check-box-checked-p (view-named 'one-factor? dialog)))
        (cn (check-box-checked-p (view-named 'conditional? dialog)))
        (ha (and (check-box-checked-p (view-named 'conditional? dialog)) (number-item-value (view-named 'hom-reps dialog)))))
    (loop for x in xs
          for y in ys
          for z in zs do
          (three-d-compute x y z pc co (and co of) (and co of cn) (and co of cn ha)))))

(defun three-d-compute (x y z pc co of cn ha)
  (let ((clasp-command (make-clasp-command
                        `(clasp::analyze-3way-relationship (clasp::simplify ,x) (clasp::simplify ,y) (clasp::simplify ,z)
                                                           :complete? ,co :one-way? ,of :conditional? ,cn
                                                           :cutoff-p ,pc :homogenous? ,ha)
                        :description (list (list 'command 'analyze-3way-relationship)
                                           (list 'x-args  x) (list 'y-args y) (list 'z-args z))
                        :format-function 'three-d-formatter)))
    (run-clasp-command clasp-command :f-update-dataset-view t :str-message "Analyzing 3 way dependencies...")))

(defmethod three-d-formatter ((cc clasp-command))
  (let* ((resultlist (l-result-clasp-command cc))
         (rll (length (first resultlist))))
    (setq *foo* resultlist)
    (list "Three Way Analysis"
          (make-instance 'statistical-table
            :name (format nil "Contingency Analysis ~A X ~A X ~A" 
                          (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag cc 'x-args)))
                          (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag cc 'y-args)))
                          (print-list-or-atom (mapcar #'name-and-dataset (get-descript-tag cc 'z-args))))
            :column-names '("G"  "P")
            :row-names (append (when (>= rll 1) '("Complete Ind."))
                               (when (>= rll 2) '("X" "Y" "Z"))
                               (when (>= rll 3) '("Y,Z : X" "X,Z : Y" "X,Y : Z"))
                               (when (>= rll 4) '("[X][Y][Z]")))
            :table-data (append (when (>= rll 1) (list (caar resultlist)))
                                (when (>= rll 2) (second (car resultlist)))
                                (when (>= rll 3) (third (car resultlist)))
                                (when (>= rll 4) (list (fourth (car resultlist)))))
            :primary-table t))))
