;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :CLASP-INTERFACE)

(define-clasp-command (com-uni-bootstrap :command-table clasp-tests :name t
                                         :menu ("Bootstrap Univariate"
                                                :documentation
                                                "Bootstrap a univariate statistic."))
    ((column 'column :prompt "Sample")
     (statistic 'string :prompt "Statistic")
     (resamples #-lucid '(integer 1 *) #+lucid 'integer :prompt "Resamples" :default 100)
     (shuffle '(member :sample-with-replacement :sample-without-replacement)
	      :prompt "Sampling Technique"
	      :default :sample-with-replacement))
   (let ((stat (eval (read-from-string (format nil "(function ~A)" statistic)))))
     (clasp::make-dataset-from-columns
      "BOOTSTRAP-DISTRIBUTION"
      (list
       (clasp::compute-bootstrap stat 
                                 :sampling-functions
                                 (case shuffle
                                   (:sample-with-replacement (list #'clasp::sample-with-replacement-from-sequence))
                                   (t #'clasp::sample-without-replacement-from-sequence))
                                 :sampling-function-arguments
                                 (list (list (simplify column)))
                                 :number-of-iterations resamples))
      (list statistic))))

(define-clasp-command (com-uni-monte-carlo :command-table clasp-tests :name t
                                           :menu ("Monte Carlo Univariate"
                                                  :documentation
                                                  "Monte Carlo testing on a univariate statistic."))
    ((statistic 'string :prompt "Statistic")
     (resamples #-lucid '(integer 1 *) #+lucid 'integer :prompt "Resamples" :default 100)
     (form #+clim-1 'expression
	   #+clim-2 '((expression) :auto-activate t) :prompt "Generation Form"))
   (let ((stat (eval (read-from-string (format nil "(function ~A)" statistic)))))
     (clasp::make-dataset-from-columns
      "MONTE-CARLO-DISTRIBUTION"
      (list
       (clasp::compute-bootstrap stat 
                                 :sampling-functions
                                 (list #'eval)
                                 :sampling-function-arguments
                                 (list (list form))
                                 :number-of-iterations resamples))
      (list statistic))))

(define-clasp-command (com-bootstrap-r :command-table clasp-tests :name t
				       :menu ("Bootstrap Bivariate"
					      :documentation "Sample by bootstrapping"))
    ((sample-1 'column :prompt "Sample-1")
     (sample-2 'column :prompt "Sample-2")
     (statistic 'string :prompt "Statistic")
     (resamples #-lucid '(integer 1 *) #+lucid 'integer :prompt "Resamples" :default 100)
     (shuffle '(member :joint-without-replace :swap-in-place :disjoint-with-replace
		       :joint-with-replace :disjoint-without-replace) 
	      :prompt "Sampling Technique"
	      :default :joint-with-replace))
   (let ((stat (eval (read-from-string (format nil "(function ~A)" statistic)))))
     (clasp::make-dataset-from-columns
      "BOOTSTRAP-DISTRIBUTION"
      (list
       (clasp::compute-bootstrap-special stat
				         (simplify sample-1)
				         (simplify sample-2)
				         resamples
				         :mode shuffle))
      (list statistic))))

(define-clasp-command (com-bi-monte-carlo :command-table clasp-tests :name t
				          :menu ("Monte Carlo Bivariate"
					         :documentation "Monte Carlo testing with a bivariate statistic."))
    ((statistic 'string :prompt "Statistic")
     (resamples #-lucid '(integer 1 *) #+lucid 'integer :prompt "Resamples" :default 100)
     (form1 #+clim-1 'expression
	    #+clim-2 '((expression) :auto-activate t) :prompt "Generation Form(X)")
     (form2 #+clim-1 'expression
	    #+clim-2 '((expression) :auto-activate t) :prompt "Generation Form(Y)"))
   (let ((stat (eval (read-from-string (format nil "(function ~A)" statistic)))))
     (clasp::make-dataset-from-columns
      "MONTE-CARLO-DISTRIBUTION"
      (list
       (clasp::compute-bootstrap stat 
                                 :sampling-functions
                                 (list #'eval #'eval)
                                 :sampling-function-arguments
                                 (list (list form1) (list form2))
                                 :number-of-iterations resamples))
      (list statistic))))
