;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :clasp-interface)

(defvar *last-command-result* nil)

(defmethod present-with-update :after ((self t) &key)
  (setf *last-command-result* self))

(defmethod present-with-update ((self cons)
				&key stream view type
				&allow-other-keys)
  (let (presentation)
    (cond
     ((eq (display-option self) :icon)
      (setf presentation
	    (present self (or type (list 'sequence (type-of (car self))))
		     :stream stream :view view)))
     ((eq (display-option self) :notebook)
      (let ((present-type (or type (list 'sequence (type-of (car self))))))
	(setf presentation
	      (with-output-as-presentation
		  #+clim-1 (:stream stream :object self :type (or type (list 'sequence (type-of (car self)))))
		  #+clim-2 (stream self present-type)
		  (mapcar #'(lambda (self-element)
			      (display self-element stream))
			  self)))))
     ((eq (display-option self) :window)
      (setf presentation
	(present self (or type (list 'sequence (type-of (car self))))
		 :stream stream :view view))
      (dolist (self-element self) (open-it self-element :refresh t)))
     ((eq (display-option self) :none) nil)
     (t (error "Illegal display option")))
    (terpri stream)
    presentation))


(defmethod present-with-update ((self result)
				&key stream view type
				&allow-other-keys)
  (declare (special mapping-p))
  (when mapping-p
    (with-text-style
	(#+clim-2 stream *label-text-style* #+clim-1 stream)
      (format stream "~a~%" (description self))))
  (let (presentation)
    (cond
     ((eq (display-option self) :icon)
      (setf presentation
	(present self (or type (type-of self)) :stream stream :view view)))
     ((eq (display-option self) :notebook)
      (setf presentation
	(with-output-as-presentation
	    #+clim-1 (:stream stream :object self :type (or type (type-of self)))
	    #+clim-2 (stream self (or type (type-of self)))
	  (display self stream))))
     ((eq (display-option self) :window)
      (setf presentation
	(present self (or type (type-of self)) :stream stream :view view))
      (open-it self :refresh t))
     ((eq (display-option self) :none) nil)
     (t (error "Illegal display option")))
    (terpri stream)
    presentation))

#+ignore
(defmethod present-with-update :after ((self cons)
				       &key stream view type
				       &allow-other-keys)
  (declare (ignore stream view type))
  (update-datasets-display)
  (update-results-display))

#+ignore
(defmethod present-with-update :after ((self result)
				       &key stream view type
				       &allow-other-keys)
  (declare (ignore stream view type))
  (update-results-display))

#+ignore
(defmethod present-with-update :after ((self dataset)
				       &key stream view type
				       &allow-other-keys)
  (declare (ignore stream view type))
  (update-datasets-display))

#+ignore
(defmethod present-with-update :after ((self column)
				       &key stream view type
				       &allow-other-keys)
  (declare (ignore stream view type))
  (update-datasets-display))

#+nil
(defmethod refresh-window (object)
  (when (window-open-p object)
    (window-clear (display-stream object))
    (with-output-as-presentation
	#+clim-1 (:stream (display-stream object) :object object :type (type-of object))
	#+clim-2 ((display-stream object) object (type-of object))
	(display object (display-stream object)))))

(defmethod refresh-window (object)
  (when (window-open-p object)
    (window-clear (display-stream object))
    (let ((s (display-stream object))
	  (type (type-of object)))
      (with-output-as-presentation (s object type)
	(display object (display-stream object))))))

#+ignore
(defmethod initialize-instance :after ((self result) &rest initargs)
  (declare (ignore initargs))
  (with-clasp-frame (clasp-frame)
   (update-results-display)))

#+ignore
(defmethod initialize-instance :after ((self dataset) &rest initargs)
  (declare (ignore initargs))
  (with-clasp-frame (clasp-frame)
   (update-datasets-display)))

#+ignore
(defmethod initialize-instance :after ((self column) &rest initargs)
  (declare (ignore initargs))
  (refresh-window (clasp::variable-dataset self)))

#+ignore
(defmethod (setf clasp::variable-value) :after (list (the-variable clasp::variable) &key
				                     (order-by 'row-number)
				                     where
				                     distinct)
  (declare (ignore list order-by where distinct))
  (with-clasp-frame (clasp-frame)
     (refresh-window the-variable)
     (refresh-window (clasp::variable-dataset the-variable))))

#+ignore
(defmethod (setf clasp::dataset-variables) :after (new-variables (self dataset))
  (declare (ignore new-variables))
  (with-clasp-frame (clasp-frame)
     (refresh-window self)))

#+ignore
(defmethod (setf title) :after (title (self result))
  (declare (ignore title))
  (with-clasp-frame (clasp-frame)
   (update-results-display)))

(defmethod (setf title) :after (title (self dataset))
  (declare (ignore title))
  (with-clasp-frame (clasp-frame)
   (refresh-window self)))

(defmethod (setf title) :after (title (self column))
  (declare (ignore title))
  (with-clasp-frame (clasp-frame)
   (refresh-window self)
   (refresh-window (clasp::variable-dataset self))))
  
#+ignore
(define-presentation-method present :after
  (result (type result) stream view &key)
  (declare (ignore stream view))
  (update-results-display)
  (auto-open result))

#+ignore
(define-presentation-method present :after
  (dataset (type dataset) stream view &key)
  (declare (ignore stream view))
  (update-datasets-display)
  (auto-open dataset))
