;;;; -*- Mode: Lisp; Package: CLASP-INTERFACE; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/frame.lisp *-*
;;;; *-* Last-Edit: Tuesday, April 4, 1995  15:56:35; Edited-By: westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                        CLASP APPLICATION FRAME                         *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  04-04-95 Header added.  Moved all the `display' and `open-it' methods to
;;;           the RESULT-METHODS file. Moved all the inferior-frame methods to
;;;           the INFERIOR-FRAME file. (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP-INTERFACE)

(export '(clasp))

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#|
To start up the frame:
   - Evaluate (CLASP-INTERFACE::clasp).

|#

;;;;----------------------------------------------------------------
;;;; Frame definition
;;;;----------------------------------------------------------------

#+clim-1
(define-gesture-name :multiple-select :button :left :shifts (:shift))
#+clim-1
(define-gesture-name :disjoint-select :button :left :shifts (:shift :control))
#+clim-1
(define-gesture-name :deselect        :button :left :shifts (:control))

#+clim-2
(define-gesture-name :multiple-select :pointer-button (:left :shift))
#+clim-2
(define-gesture-name :disjoint-select :pointer-button (:left :shift :control))
#+clim-2
(define-gesture-name :deselect        :pointer-button (:left :control))

;;;;----------------------------------------------------------------

(defvar *clasp-frame* nil)
(defvar *clasp-frame-main-window* nil)
(defvar *current-command* nil)
(defvar *current-arguments* nil)
(defvar mapping-p nil)

;;;;----------------------------------------------------------------

(clim::define-application-frame clasp-frame ()
  ((error-handling-preferences :initform '() :accessor error-handling-preferences)
   (datasets :initform (make-instance 'datasets-list :display-option :notebook) :accessor datasets)
   (results :initform (make-instance 'results-list :display-option :notebook) :accessor results)
   (about-window :accessor about-window)
   (inferiors :initform nil :accessor inferiors)
   (history :initform nil))
  (:command-table (clasp
		    :inherit-from
		    (user-command-table
		     clasp-file 
                     clasp-description
		     clasp-transform clasp-tests
		     clasp-data-manipulation 
                     clasp-graphing
		     clasp-sample
                     #+obsolete clasp-bootstrap
		     #+ignore clasp-map
		     user-command-table
		     ;;accept-values-pane
		     ;;icon-command-table
		     :graph
		     )))
  
  (:command-definer define-clasp-command)
  #+clim-1
  (:panes ((display :application
                    :incremental-redisplay t
                    :scroll-bars :both
                    :end-of-line-action :allow)
	   (mouse-doc :pointer-documentation)
	   (menu :command-menu)
	   (notebook :interactor
		       :default-text-style (clim::make-text-style :sans-serif :roman :normal))
	   (datasets :application
		     :default-text-style (clim::make-text-style :sans-serif :roman :small)
		     :incremental-redisplay t
		     :scroll-bars :both
		     :end-of-line-action :allow
		     :end-of-page-action :allow
		     :display-function 'draw-datasets-pane)
	   (results :application
		    :default-text-style (clim::make-text-style :san-serif :roman :small)
		    :incremental-redisplay t
		    :scroll-bars :both
		    :end-of-line-action :allow
		    :end-of-page-action :allow
		    :display-function 'draw-results-pane)
           ))
  #+clim-1
  (:layout ((notebook
	     (:column 1
	      (menu :compute)
	      (:row :rest  
	       (:column 1/5
		(datasets 1/2)
		(results :rest))
	       (notebook :rest))
	      (mouse-doc :compute)))
            #-MCL
            (window
	     (:column 1
	      (menu :compute)
	      (notebook :rest)
	      (mouse-doc :compute)))
            (none
	     (:column 1
	      (menu :compute)
	      (notebook :rest)
	      (mouse-doc :compute)))
	    ))
  #+clim-2
  (:panes
   (notebook :interactor
	     :scroll-bars t
	     :default-text-style (clim::make-text-style :sans-serif :roman :normal)
	     ;:height +fill+ :width +fill+
	     )
   (datasets :application
	     :incremental-redisplay t
	     :default-text-style (clim::make-text-style :sans-serif :roman :small)
             :display-time t
	     :scroll-bars t
	     :end-of-line-action :allow
	     :end-of-page-action :allow
	     :display-function 'draw-datasets-pane)
   (results  :application
	     :incremental-redisplay t
	     :default-text-style (clim::make-text-style :sans-serif :roman :small)
	     :display-time t
	     :scroll-bars t
	     :end-of-line-action :allow
	     :end-of-page-action :allow
	     :display-function 'draw-results-pane)
   (mouse-doc :pointer-documentation
              :text-style (clim:make-text-style :sans-serif :roman :small)
	      :height 12
	      :max-height 15))

  #+clim-2-old
  (:pointer-documentation t)
  #+clim-2
  (:documentation "CLASP")
  #+clim-2
  (:layouts
   (notebook+datasets+results
    (vertically ()
      (horizontally ()
	(1/5 (vertically () datasets results))
	(4/5 notebook))
	mouse-doc))
   (notebook+datasets
    (vertically ()
      (horizontally ()
	(1/5 datasets)
	(4/5 notebook))
      mouse-doc))
   (notebook+results
    (vertically ()
      (horizontally ()
	(1/5 results)
	(4/5 notebook))
      mouse-doc))
   (notebook
    (vertically ()
      (horizontally () notebook)
      mouse-doc)))
  (:top-level (clasp-top-level))
  )	; Added for lisp-listener -AC 7/23

;; This seems like a reasonable place to attempt to reload the translations.
(defmethod initialize-instance :after ((frame clasp-frame) &key)
  (let ((pathname (merge-pathnames "clasp.translations" 
				   #+Lispworks
				   (system::get-working-directory)
				   #+cmu
				   (ext:default-directory)
				   #-(or cmu Lispworks)
				   (dwim::working-directory)
				   )))
    (if (probe-file pathname)
        (load pathname)
        (warn "Could not locate translations file ~a - autoloading is disabled." pathname))))
      

(defun clasp-frame ()
  *clasp-frame*)

(defmacro with-clasp-frame ((frame-var) &body body)
  `(let ((,frame-var (clasp-frame)))
     (when ,frame-var
       . ,body)))

;; This fixes the scroll bar not workig bug in MCL.
#+MCL
(defun set-scroll-max (stream)
  (ccl:set-scroll-bar-max (slot-value stream 'ccl::v-scroller) 
                          (round (* 2.5 (window-inside-height stream))))
  (ccl:set-scroll-bar-max (slot-value stream 'ccl::h-scroller) 
                          (round (* 2.5 (window-inside-width stream))))
  )
 
(defun stream-of (pane-name)
  (with-clasp-frame (clasp-frame)
    (get-frame-pane clasp-frame pane-name)))

#+CLIM-2 
;; override the default behavior in CLIM-2 which is to use the first
;; application frame that it finds
(defmethod frame-standard-output ((the-frame clasp-frame))
  (get-frame-pane the-frame 'notebook))

;;; ---------------------------------------------------------------------------

(defmethod draw-datasets-pane ((application clasp-frame) stream)
  (display (datasets application) stream))

(defun draw-datasets-window ()
  (display (datasets (clasp-frame)) (display-stream (datasets (clasp-frame)))))

(defun update-datasets-display ()
  (with-clasp-frame (clasp-frame)
    (format *trace-output* "datasets layout = ~A~%" (frame-current-layout clasp-frame))
    (case (frame-current-layout clasp-frame)
      ((notebook+datasets+results notebook+datasets)
       (clim:redisplay-frame-pane clasp-frame 'datasets :force-p t))
      ((notebook+results notebook)
       (draw-datasets-window)))))

(defmethod draw-results-pane ((application clasp-frame) stream)
  (display (results application) stream))

(defun draw-results-window ()
  (display (results (clasp-frame)) (display-stream (results (clasp-frame)))))

(defun update-results-display ()
  (with-clasp-frame (clasp-frame)
    (case (frame-current-layout clasp-frame)
      ((notebook+datasets+results notebook+results)
       (clim:redisplay-frame-pane clasp-frame 'results :force-p t))
      ((notebook+datasets notebook)
       (draw-results-window)))))

;;; ----------------------------------------------------------------------------

(defvar *command-counter* 0)

(defmethod execute-frame-command :before ((frame clasp-frame) command)
  (fresh-line)
  (format *standard-output* "~s> " (incf *command-counter*))
  (present command 'command)
  (terpri)
  (format *standard-output* "~s< " *command-counter*)
  )

(defmethod execute-frame-command :around ((frame clasp-frame) command)
  (declare (ignore command))
  (flet
      ((do-it ()
	 (if clasp::*debug*
	     (handler-case (call-next-method)
	       (clasp::clasp-error (condition)
		 (format *standard-output* "~&CLASP ERROR: ~A~%" condition))
	       #+ignore
	       (clasp::clasp-warning (condition)
		 (format *standard-output* "~&CLASP WARNING: ~A~%" condition)))
	   (handler-case (call-next-method)
	     (clasp::clasp-error (condition)
	       (format *standard-output* "~&CLASP ERROR: ~A~%" condition))
	     #+ignore
	     (clasp::clasp-warning (condition)
	       (format *standard-output* "~&CLASP WARNING: ~A~%" condition))
	     (error (condition)
	       (format *standard-output* "~&ERROR: ~A~%" condition))))))
    #-MCL (do-it)
    #+MCL (catch :cancel
	    (do-it))))

(defmethod execute-frame-command :after ((frame clasp-frame) command)
  (declare (ignore command))
  #+MCL
  (set-scroll-max (get-frame-pane frame 'display)))


;;; ----------------------------------------------------------------------------
;;; Starting up the frame

#+clim-1
(defun clasp (&optional restart)
  (when (or restart (null *clasp-frame*))
    (setf *clasp-frame*
      (make-application-frame
       'clasp-frame
       :parent (clasp-root)
       :width 640
       :height 640)))
  (run-frame-top-level *clasp-frame*))

#+clim-1
(defvar *clasp-root* nil)

#+clim-1(defun clasp-root (&optional reinit)
  (or (and (not reinit) *clasp-root*)
      (setf *clasp-root* (open-root))))
  
#+clim-1
(defun open-root ()
  ;;thanks to Oliver Christ <oli@adler.ims.uni-stuttgart.de> for the code below
  #+Lucid (open-root-window
	   :clx
	   :host (lcl:environment-variable "DISPLAY"))
  #+Allegro-v4.1 (open-root-window
		  :clx
		  :host (system:getenv "DISPLAY"))
  #+MCL          (open-root-window :mcl)
  #+Genera       (open-root-window :sheet)
  ;; 11-3-93 Westy
  #+Lispworks    (open-root-window 
                  :clx
		  :host (system::getenv "DISPLAY"))
  ;;Please contact keunen@nrb.be if you modify this source code.
  #-(or Lucid Allegro-v4.1 MCL Genera Lispworks)
  (warning "Unknown CLIM/LISP combination.  Please modify the
`open-root' function to your needs."))

#+clim-2
(defun clasp (&optional restart)
  (when (or restart (null *clasp-frame*))
    (setf *clasp-frame*
	  (make-application-frame 'clasp-frame
            :width 640 :height 480
	    )))
  (run-frame-top-level *clasp-frame*)
  
  #+OLD
  (let* ((port (find-port))
	 (framem (typecase port
		   (frame-manager port)
		   (t (find-frame-manager :port port))))
	 (graft (typecase port
		  (frame-manager (graft port))
		  (t (find-graft :port port)))))
    (declare (ignore graft))
    (when (or restart (null *clasp-frame*))
      (setf *clasp-frame*
	(make-application-frame 'clasp-frame :frame-manager framem
                                :width width :height height
                                )))
    (run-frame-top-level *clasp-frame*))
  #+Lispworks
  (when (member :CLASP-STANDALONE *features*)
    (user::bye)))

;;;;----------------------------------------------------------------
;;;; For easy access

(defmethod run-frame-top-level :before ((the-frame clasp-frame)
					#+clim-2 &key #+clim-2 &allow-other-keys)
  (setf *clasp-frame-main-window*
	(get-frame-pane the-frame 'notebook)))

(defmethod run-frame-top-level :around ((the-frame clasp-frame)
					#+clim-2 &key #+clim-2 &allow-other-keys)
  (let ((*print-length* 10)
	(*clasp-readtable* (copy-readtable)))
    (set-macro-character #\! #'clasp::clasp-variable-reader-macro
			 nil *clasp-readtable*)
    (let ((*readtable* *clasp-readtable*))
      (call-next-method))))

;;; ----------------------------------------------------------------------------
;;; ----------------------------------------------------------------------------
;;; Application building code.

#+MCL
(defun clasp-top-level-function (&optional restart)
  (ccl::%set-toplevel nil) ; set this to nil MCL will return after clasp exits
  (handler-case   
    (clasp restart)
    (error (condition)
           (unless (ccl::y-or-n-dialog 
		    (format nil "Do not touch the close box.~%~
                                 Use the QUIT command to quit.~2%~
                                 Do you really want to quit?")
				       :cancel-text nil)
             (clasp-top-level-function t)))))

#+MCL
(defun build-app (&optional (name "CLASP 1.2"))
  (ccl::save-application name :toplevel-function #'clasp-top-level-function :creator "CLSP"))

;;; ----------------------------------------------------------------------------

#+Lucid
(defun clasp-top-level-function (&optional restart)
  (clasp restart)
  (lcl::quit))

#+Lucid
(defun build-app (&optional (name "clasp"))
  (lcl::disksave name :full-gc t :restart-function 'clasp-top-level-function :verbose t))

;;; ----------------------------------------------------------------------------

#+Lispworks
(defun clasp-top-level-function (&optional restart)
  (clasp restart))

#+Lispworks
(defun build-app (&optional (name "clasp"))
  (pushnew :CLASP-STANDALONE *features*)
  (lw::save-image
   name
   :restart-function
   #'(lambda ()
       (tools:start-lispworks
        :gcmon nil
	:start-functions '(clasp-interface::clasp-top-level-function)))))


;;; ----------------------------------------------------------------------------

#+Allegro
(defun clasp-top-level-function (&optional restart)
  (clasp restart)
  (excl::exit))

;;; ----------------------------------------------------------------------------

#|  This might be necessary for lisp-listener -AC 7/23
(defmethod clasp-top-level :before
		((the-frame clasp-frame))
  (setf *clasp-frame* the-frame)
  (setf *clasp-frame-main-window*
	(get-frame-pane *clasp-frame* 'display)))

(defmethod clasp-top-level :around ((the-frame clasp-frame))
  (format *standard-output* "~&; Changing *package* to ~a~%" (find-package 'clasp-interface))
  (let ((*package* (find-package 'clasp-interface)))
    (call-next-method)))

|#
;;; ----------------------------------------------------------------------------
  
#|

(defclass graph-icon (result)
	  ((graph :initarg :graph :reader graph)))

(defun make-graph-icon (&key graph x-label y-label title)
  (make-instance 'graph-icon
    :graph (or graph
               (make-graph
		:title (or title
			   (format nil "Graph Number ~D"
				   (incf graph::graph-number))) ;; was graph-user:: AC
		:x-label (or x-label "Fortnights")
		:y-label (or y-label "Furlongs")
		:present-self-p nil))))

(defmethod print-object ((the-graph-icon graph-icon) stream)
  (print-unreadable-object (the-graph-icon stream)
    (princ (clasp::name the-graph-icon) stream)))

;(define-presentation-type graph-icon ())

(defmethod display ((the-graph-icon graph-icon) stream)
  (graph::display-graph ;; was graph-user:: -AC
   (graph the-graph-icon)
   :width 440 :height 390
   :scroll-if-necessary t
   :stream stream))

#+OLD
(defmethod execute-on-target (icon-1 icon-2)
  nil)

#+OLD
(defmethod execute-on-target ((the-graph-icon graph-icon) (the-data-icon graph-data-icon))
  (add-dataset (graph the-graph-icon) (graph-data the-data-icon))
  (open-it the-graph-icon :refresh t))

|#
;;; ----------------------------------------------------------------------------


;;; ***************************************************************************
;;; *                              End of File                                *
;;; ***************************************************************************
