;;;; -*- Mode:Common-Lisp; Package:CLASP-INTERFACE; Base:10 -*-
;;;; *-* Last-edit: Friday, July 9, 1993  12:15:43; Edited-By: carlson *-* 


;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu


(in-package :clasp-interface)

;;; ---------------------------------------------------------------------------
;;; Matrix

(defclass matrix-result (modifiable-display-option-mixin result)
  ((display-option :allocation :class :initform :notebook)
   (contents :initarg :contents)
   (dimensions :initarg :dimensions)
   (top-left :initarg :top-left :initform "")
   (top-right :initarg :top-right :initform "")
   (bottom-left :initarg :bottom-left :initform "")
   (bottom-right :initarg :bottom-right :initform "")
   (top :initarg :top :initform nil)
   (bottom :initarg :bottom :initform nil)
   (left :initarg :left :initform nil)
   (right :initarg :right :initform nil)))

(defun make-matrix-result (&rest options)
  (apply #'make-instance 'matrix-result options))

(defun make-matrix-result-from-list (list &rest initargs)
  (let ((dimensions (list (length (car list)) (length list))))
    (apply #'make-matrix-result
           :dimensions dimensions
           :contents (make-array dimensions list)
           initargs)))

(defmethod display ((self matrix-result) stream)
  (flet ((format-item (value)
	              (formatting-cell (stream :align-x :right)
			               (present value 'nice-real :stream stream)))
         (format-label (label)
	               (formatting-cell
	                (stream :align-x :right)
	                (with-text-style
		         (#+clim-2 stream *label-text-style* #+clim-1 stream)
	                 (format stream "~a" label)))))
    (with-slots (dimensions contents top bottom left right top-left top-right bottom-left bottom-right) self
      (fresh-line stream)
      (formatting-table
       (stream #+clim-1 :inter-column-spacing #+clim-2 :x-spacing '(2 :character))
       (formatting-row 
        (stream)
        (format-label top-left)
        (when top
          (dotimes (n (first dimensions))
            (format-label (nth n top))))
	(format-label top-right))
       (dotimes (j (second dimensions))
         (formatting-row
          (stream)
          (format-label (or (nth j left) ""))
          (dotimes (i (first dimensions))
            (format-item (aref contents i j)))
          (format-label (or (nth j right) ""))))
       (formatting-row 
        (stream)
        (format-label bottom-left)
        (when bottom
          (dotimes (n (first dimensions))
            (format-label (nth n bottom))))
        (format-label bottom-right))))))

(define-clasp-command (com-correlation-matrix :command-table clasp-description
				 :name "Corr. Matrix" :menu "Corr. Matrix")
    ((variables '(sequence numbers) :prompt "Variables"))
   (present-with-update (make-instance 'matrix-result
				       :title (format nil "Correlation matrix for ~{~a~^, ~}"
						      (mapcar #'title variables))
				       :contents (apply #'clasp::correlation-matrix (mapcar #'simplify variables))
				       :dimensions (list (length variables) (length variables))
				       :top (mapcar #'title variables)
				       :left (mapcar #'title variables)
				       :description (format nil "~@(correlation matrix for ~{~a~^, ~}~)"
							    (mapcar #'description variables)))
			:stream *standard-output* :view +textual-dialog-view+))

(defun clasp::correlation-matrix (&rest variables)
  (let* ((dimensions (list (length variables) (length variables)))
         (matrix (make-array dimensions :element-type 'float)))
    (dotimes (i (first dimensions))
      (dotimes (j (second dimensions))
        (setf (aref matrix i j) (if (<= i j) (correlation (nth i variables) (nth j variables))
                                  (aref matrix j i)))))
    matrix))

(define-clasp-command (com-covariance-matrix :command-table clasp-description
				 :name "Cov. Matrix" :menu "Cov. Matrix")
    ((variables '(sequence numbers) :prompt "Variables"))
   (present-with-update (make-instance 'matrix-result
				       :title (format nil "Correlation matrix for ~{~a~^, ~}"
						      (mapcar #'title variables))
				       :contents (apply #'clasp::covariance-matrix (mapcar #'simplify variables))
				       :dimensions (list (length variables) (length variables))
				       :top (mapcar #'title variables)
				       :left (mapcar #'title variables)
				       :description (format nil "~@(correlation matrix for ~{~a~^, ~}~)"
							    (mapcar #'description variables)))
			:stream *standard-output* :view +textual-dialog-view+))

(defun clasp::covariance-matrix (&rest variables)
  (let* ((dimensions (list (length variables) (length variables)))
         (matrix (make-array dimensions :element-type 'float)))
    (dotimes (i (first dimensions))
      (dotimes (j (second dimensions))
        (setf (aref matrix i j) (if (<= i j) (correlation (nth i variables) (nth j variables))
                                  (aref matrix j i)))))
    matrix))

