;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :CLASP-INTERFACE)

(define-command-table clasp-sample)

(add-menu-item-to-command-table 'clasp "Sample" :menu 'clasp-sample
				:errorp nil :after "Transform"
				:documentation
				"Sample from theoretical distributions")

(define-command (com-sample-uniform :command-table clasp-sample :name t
				    :menu ("Sample from Uniform Distribution"
					   :documentation
					   "Sample from Uniform Distribution"))
    ((minimum 'integer :prompt "Minimum" :default 0)
     (maximum 'integer :prompt "Maximum" :default 1)
     (size-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Size of samples")
     (number-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Number of samples"))
  (present-with-update
   (sample-uniform-distribution-to-dataset minimum maximum size-of-samples number-of-samples)
   #+OLD
   (funcall
    #'(lambda (size-of-samples number-of-samples)
	(sample-to-dataset (make-instance 'uniform-distribution
			     :minimum minimum
			     :maximum maximum)
			   size-of-samples
			   number-of-samples))
    size-of-samples number-of-samples)
   :stream *standard-output*
   :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))

(define-command (com-sample-normal :command-table clasp-sample :name t
				   :menu ("Sample from Normal Distribution"
					  :documentation
					  "Sample from Normal Distribution"))
    ((mean    'real :prompt "Mean")
     (std-dev 'real :prompt "Standard deviation")
     (size-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		  :prompt "Size of samples")
     (number-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		  :prompt "Number of samples"))
  (present-with-update
   (sample-normal-distribution-to-dataset mean std-dev size-of-samples number-of-samples)
   #+OLD
   (funcall
    #'(lambda (mean std-dev size-of-samples number-of-samples)
	(sample-to-dataset (make-instance 'normal-distribution :mean mean
					  :standard-deviation std-dev)
			   size-of-samples
			   number-of-samples))
    mean std-dev size-of-samples number-of-samples)
   :stream *standard-output*
   :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))

(define-command (com-sample-binomial :command-table clasp-sample :name t
				     :menu
				     ("Sample from Binomial Distribution"
				      :documentation
				      "Sample from Binomial Distribution"))
    ((p 'real :prompt "P")
     (n #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "N")
     (size-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Size of samples")
     (number-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Number of samples"))
  (present-with-update
   (sample-binomial-distribution-to-dataset p n size-of-samples number-of-samples)
   #+OLD
   (funcall
    #'(lambda (p n size-of-samples number-of-samples)
	(sample-to-dataset (make-instance 'binomial-distribution :p p :n n)
			   size-of-samples
			   number-of-samples))
    p n size-of-samples number-of-samples)
   :stream *standard-output*
   :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))

(define-command (com-sample-poisson :command-table clasp-sample :name t
				    :menu ("Sample from Poisson Distribution"
					   :documentation
					   "Sample from Poisson Distribution"))
    ((mean 'real :prompt "Mean")
     (size-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Size of samples")
     (number-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Number of samples"))
  (present-with-update
   (sample-poisson-distribution-to-dataset mean size-of-samples number-of-samples)
   #+OLD
   (funcall
    #'(lambda (mean size-of-samples number-of-samples)
	(sample-to-dataset (make-instance 'poisson-distribution :mean mean)
			   size-of-samples
			   number-of-samples))
    mean size-of-samples number-of-samples)
   :stream *standard-output*
   :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))
 
(define-command (com-sample-gamma :command-table clasp-sample :name t
				  :menu ("Sample from Gamma Distribution"
					 :documentation
					 "Sample from Gamma Distribution"))
    ((ia #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Integer order")
     (size-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Size of samples")
     (number-of-samples #-lucid '(integer 1 *) #+lucid 'integer
		:prompt "Number of samples"))
  (present-with-update
   (sample-gamma-distribution-to-dataset ia size-of-samples number-of-samples)
   #+OLD
   (funcall
    #'(lambda (ia size-of-samples number-of-samples)
	(sample-to-dataset (make-instance 'gamma-distribution :ia ia)
			   size-of-samples
			   number-of-samples))
    ia size-of-samples number-of-samples)
   :stream *standard-output*
   :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))


;;; ---------------------------------------------------------------------------
;;; EOF