;;;; -*- Mode:Common-Lisp; Package:CLASP-INTERFACE; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/test-commands.lisp *-*
;;;; *-* Last-edit: Friday, July 9, 1993  12:15:36; Edited-By: carlson *-* 
;;;; *-* Machine: Miles (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                       Data Manipulation Commands                       *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-18-93 File Created.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :CLASP-INTERFACE)

;;; --*--
;;; ***************************************************************************

(define-command-table clasp-tests)

(add-menu-item-to-command-table 'clasp "Test" 
                                :menu 'clasp-tests 
				:documentation "Statistical hypothesis testing"
                                :errorp nil :after "Describe")

;;; ---------------------------------------------------------------------------
;;; Command definition

(define-clasp-command (com-confidence-interval-z
		       :command-table clasp-tests
		       :name "Confidence Interval Using z Statistic"
		       :menu "Confidence Interval Using z Statistic")
    ((column-1 'numbers :prompt "X" :mapping-option :map)
     (real-2 #-lucid'(real 0 1) #+lucid 'real :prompt "Confidence level"))
  (let* ((values
	  (multiple-value-list (funcall
				#'confidence-interval-z
				(simplify column-1)
				real-2)))
	 (result (funcall 'make-confidence-interval-result-from-list values)))
    (setf (description result)
      (format nil "~A % Confidence interval of ~a"
	      (* real-2 100) (description column-1)))
    (assert (= (length values) 3) nil
      "`confidence-interval-z' returned ~s values, we were expecting 3"
      (length values))
    (present-with-update result :stream *standard-output*
			 :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-confidence-interval-t
		       :command-table clasp-tests
		       :name "Confidence Interval Using t Statistic"
		       :menu "Confidence Interval Using t Statistic")
    ((column-1 'numbers :prompt "X" :mapping-option :map)
     (real-2 #-lucid '(real 0 1)  #+lucid 'real :prompt "Confidence level"))
  (let* ((values
	  (multiple-value-list (funcall
			       #'confidence-interval-t
			       (simplify column-1)
			       real-2)))
	 (result (funcall 'make-confidence-interval-result-from-list values)))
    (setf (description result)
      (format nil "~A % Confidence interval of ~a"
	      (* real-2 100) (description column-1)))
    (assert (= (length values) 3) nil
      "`confidence-interval-t' returned ~s values, we were expecting 3"
      (length values))
    (present-with-update result :stream *standard-output*
			 :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-confidence-interval-proportion
		       :command-table clasp-tests
		       :name "Confidence Interval of a Proportion"
		       :menu "Confidence Interval of a Proportion")
    ((integer-1 #-lucid '(integer 0 *) #+lucid 'integer :prompt "Successes")
     (integer-2 #-lucid '(integer 1 *) #+lucid 'integer :prompt "Trials")
     (real-3 #-lucid '(real 0 1) #+lucid 'real :prompt "Confidence level"))
  (let* ((values
	  (multiple-value-list (funcall
				#'confidence-interval-proportion
				integer-1
				integer-2
				real-3)))
	 (result (funcall 'make-confidence-interval-result-from-list values)))
    (setf (description result)
      (format nil "~A % Confidence interval of ~a success in ~a trials"
	      (* real-3 100) integer-1 integer-2))
    (assert (= (length values) 3) nil
      "`confidence-interval-proportion' returned ~s values, we were expecting 3"
      (length values))
    (present-with-update result :stream *standard-output*
			 :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-t-test-one-sample :command-table clasp-tests
					     :name "t-test One Sample"
					     :menu "t-test One Sample")
    ((column-1 'numbers :prompt "X" :mapping-option :map)
     (mean 'number :prompt "Mean" :mapping-option :map)
     (member-2 '(member :both :positive :negative)
		 :prompt "Tails: [Both, Positive or Negative]"))
  (let* ((values
	  (multiple-value-list (funcall
				#'t-test-one-sample
				(simplify column-1)
				member-2 mean)))
	 (result (funcall 'make-t-test-result-from-list values)))
    (setf (description result)
      (format nil "t-test of ~a with ~a tail(s)"
	      (description column-1) member-2))
    (assert (= (length values) 4) nil
      "`t-test-one-sample' returned ~s values, we were expecting ~s"
      (length values))
    (present-with-update result :stream *standard-output*
			 :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-t-test-two-samples :command-table clasp-tests
					      :name "t-test Two Samples"
					      :menu "t-test Two Samples")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map)
     (member-3 '(member :both :positive :negative)
		 :prompt "Tails: [Both, Positive or Negative]"))
  (let* ((values
	  (multiple-value-list (funcall
				#'t-test
				(simplify column-1)
				(simplify column-2)
				member-3)))
	 (result (funcall 'make-t-test-result-from-list values)))
    (setf (description result)
      (format nil "t-test of ~a and ~a with ~a tail(s)"
	      (description column-1) (description column-2) member-3))
    (assert (= (length values) 4) nil
      "`t-test' returned ~s values, we were expecting ~s"
      (length values) 4)
    (present-with-update result :stream *standard-output*
			 :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-t-test-matched :command-table clasp-tests
					  :name "t-test Matched Pairs"
					  :menu "t-test Matched Pairs")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map)
     (member-3 '(member :both :positive :negative)
		 :prompt "Tails: [Both, Positive or Negative]"))
  (let* ((values
	  (multiple-value-list (funcall
				#'t-test-matched
				(simplify column-1)
				(simplify column-2)
				member-3)))
	 (result (funcall 'make-t-test-result-from-list values)))
    (setf (description result)
      (format nil "matched pairs t-test of ~a and ~a with ~a tail(s)"
	      (description column-1) (description column-2) member-3))
    (assert (= (length values) 4) nil
      "`t-test-matched' returned ~s values, we were expecting ~s"
      (length values) 4)
    (fresh-line)
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-d-test :command-table clasp-tests :name t
			    :menu "D test")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map)
     (member-3 '(member :both :positive :negative)
		 :prompt "Tails: [Both, Positive or Negative]"))
  (let* ((values
	  (multiple-value-list (funcall
				#'d-test
				(simplify column-1)
				(simplify column-2)
				member-3)))
	 (result (funcall 'make-d-test-result-from-list values)))
    (setf (description result)
      (format nil "D test of ~a and ~a with ~a tail(s)"
	      (description column-1) (description column-2) member-3))
    (assert (= (length values) 3) nil
      "`d-test' returned ~s values, we were expecting 3"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-anova-one-way-variables :command-table clasp-tests
						   :name "Anova - One Way"
						   :menu "Anova - One Way")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map)
     (confidence-intervals '(null-or-type #-lucid (real 0 1) #+lucid real) :prompt "Confidence intervals" :default nil))
  (let* ((iv-values (simplify-and-sort column-1))
         (values
	  (multiple-value-list (funcall
				#'anova-one-way-variables
				iv-values
				(simplify-and-sort-by column-2 column-1)
				t
				confidence-intervals)))
	 (result (funcall 'make-anova-one-way-result-from-list values
			  :iv column-1 :dv column-2 :groups (remove-duplicates iv-values))))
    (setf (description result)
      (format nil "anova of ~a on ~a"
	      (description column-1) (description column-2)))
    (assert (= (length values) 5) nil
      "`anova-one-way-variables' returned ~s values, we were expecting 5"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-anova-two-way-variables :command-table clasp-tests
						   :name "Anova - Two Way"
						   :menu "Anova - Two Way")
    ((column-1 'numbers :prompt "Y" :mapping-option :map)
     (column-2 'numbers :prompt "X 1" :mapping-option :map)
     (column-3 'numbers :prompt "X 2" :mapping-option :map))
   (let* ((values
	  (multiple-value-list (funcall
				#'anova-two-way-variables-unequal-cell-sizes
				(simplify column-1)
				(simplify column-2)
				(simplify column-3))))
	 (result (funcall 'make-anova-two-way-table-from-list (nth 0 values)
			  :iv1 (name column-2)
			  :iv2 (name column-3)
			  :dv (name column-1)
			  :cell-means (nth 1 values)
			  :row-totals (nth 2 values)
			  :column-totals (nth 3 values)
			  :grand-total (nth 4 values)
			  :a-labels (nth 5 values)
			  :b-labels (nth 6 values))))
    (setf (description result)
      (format nil "anova of ~a and ~a on ~a"
	      (description column-2) (description column-3)
	      (description column-1)))
    (assert (= (length values) 7) nil
      "`~a' returned ~s values, we were expecting ~s"
      'anova-two-way-variables (length values) 7)
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-chi-square-rxc :command-table clasp-tests
					  :name "Chi-Square RxC"
					  :menu "Chi-Square RxC")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map))
  (let* ((values
	  (multiple-value-list (funcall
				#'chi-square-rxc
				(simplify column-1)
				(simplify column-2))))
	 (result (funcall 'make-chi-square-result-from-list values)))
    (setf (description result)
      (format nil "chi-square of ~a and ~a"
	      (description column-1) (description column-2)))
    (assert (= (length values) 8) nil
      "`chi-square-rxc' returned ~s values, we were expecting 8"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-chi-square-counts
		       :command-table clasp-tests
		       :name "Chi-Square Counts"
		       :menu "Chi-Square Counts")
    ((a 'number :prompt "A")
     (b 'number :prompt "B")
     (c 'number :prompt "C")
     (d 'number :prompt "D"))
  (let* ((table (make-array '(2 2) :initial-contents
                                   (list (list a c) (list b d))))
         (values
	  (append
            (multiple-value-list (funcall #'chi-square-2x2-counts a b c d))
            (multiple-value-list (funcall #'clasp::g-test table))
	    (list table)
            '(("c1" "c2") ("r1" "r2"))))
	 (result (funcall 'make-chi-square-result-from-list values)))
    (setf (description result)
      (format nil "chi-square of ~a, ~a, ~a, ~a" a b c d))
    (assert (= (length values) 8) nil
      "`chi-square-2x2' returned ~s values, we were expecting 8"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-linear-regression-brief
		       :command-table clasp-tests
		       :name "Linear Regression - brief"
		       :menu "Linear Regression - brief")
    ((column-1 'numbers :prompt "Y" :mapping-option :map)
     (column-2 'numbers :prompt "X" :mapping-option :map))
  (let* ((values
	  (multiple-value-list (funcall
				#'linear-regression-brief
				(simplify column-1)
				(simplify column-2))))
	 (result (funcall 'make-linear-regression-brief-result-from-list
			  values)))
    (setf (description result)
      (format nil "linear regression of ~a on ~a"
	      (description column-1) (description column-2)))
    (assert (= (length values) 5) nil
      "`linear-regression-brief' returned ~s values, we were expecting 5"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-linear-regression-verbose
		       :command-table clasp-tests
		       :name "Linear Regression - verbose"
		       :menu "Linear Regression - verbose")
    ((column-1 'numbers :prompt "Y" :mapping-option :map)
     (column-2 'numbers :prompt "X" :mapping-option :map))
  (let* ((values
	 (multiple-value-list (funcall
			       #'linear-regression-verbose
			       (simplify column-1)
			       (simplify column-2))))
	 (result (funcall 'make-linear-regression-verbose-result-from-list
			  values :dv column-1 :iv column-2)))
    (setf (description result)
      (format nil "linear regression of ~a on ~a"
	      (description column-1) (description column-2)))
    (assert (= (length values) 7) nil
      "`linear-regression-verbose' returned ~s values, we were expecting 7"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-multiple-linear-regression
		       :command-table clasp-tests
		       :name "Multiple Linear Regression"
		       :menu "Multiple Linear Regression")
    ((dv 'numbers :prompt "Y")
     (ivs '(sequence numbers) :prompt "X's"))
  (let* ((values
	  (multiple-value-list (apply
				#'clasp::multiple-linear-regression-verbose
				(simplify dv)
				(mapcar #'simplify ivs))))
	 (result (funcall 'make-multiple-linear-regression-verbose-result-from-list
			  values
			  :dv dv
			  :ivs ivs)))
    (setf (description result)
      (format nil "multiple linear regression of ~a on ~{~a~^ and ~}"
	      (description dv) (mapcar #'description ivs)))
    (assert (= (length values) 14) nil
      "`multiple-linear-regression' returned ~s values, we were expecting 14"
      (length values))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

;;; ---------------------------------------------------------------------------
;;; EOF
