;;;; -*- Mode: Lisp; Package: USER; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/dev/load-common.lisp *-*
;;;; *-* Last-Edit: Wednesday, May 29, 1996  14:57:12; Edited-By: Westy *-*
;;;; *-* Machine: Lindy (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                           Load File for CLASP                          *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and
;;; its documentation is hereby granted without fee for non-commercial
;;; uses only (not for resale), provided that the above copyright
;;; notice of EKSL, this paragraph and the one following appear in all
;;; copies and in supporting documentation.

;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

(in-package :COMMON-LISP-USER)

;;; --*--
;;; ***************************************************************************

(defvar *clasp-suppress-loading-interface* #+(or CLIM MCL) nil #-(or CLIM MCL) t)

;; Set these to T if you want to load any of them.
(defvar *clasp-load-fbd* nil) 
(defvar *clasp-load-msdd* nil)
(defvar *clasp-load-imsdd* nil)
(defvar *clasp-load-layers* #+MCL nil #-MCL (or *clasp-load-fbd* *clasp-load-msdd* *clasp-load-imsdd*))

(defvar *clasp-defer-load* nil
  "Used to communicate to the layered systems that they should defer loading themselves.")

;; Load the the .translations and .system files for CLASP and CLASP-INTERFACE.
(load (merge-pathnames "clasp-translations.lisp" (load-pathname)))
(load (merge-pathnames "clasp-system.lisp" (load-pathname)))
(unless *clasp-suppress-loading-interface*
  (load (merge-pathnames "clasp-interface-system.lisp" (load-pathname))))

;; Version naming crap.
(defparameter *clasp-next-version* "1.6")
(defparameter *clasp-development-version* (search "dev" (namestring (eksl-translate-logical-pathname "CLASP:SOURCE;"))))
(defparameter *clasp-demo-version* (search "demo" (namestring (eksl-translate-logical-pathname "CLASP:SOURCE;"))))

#+MCL
(when *clasp-development-version*
  (pushnew :CLASP-MCL-INTERFACE-DEBUG *features*))

;; Define a subset of the EKSL Utilities to load for CLASP.
(defparameter *utils-files-to-load-for-clasp*
  `("package"
    "provide-require"
    "spy"
    "feature-case"
    "macros"
    "timeit"
    "anaphoric"
    "utilities"
    "menus"
    "debugging-utils"
    "day-and-time"
    "clos-utils"
    "clos-mixins"
    "indentation"
    #+MCL
    "mcl-menus"))

(proclaim '(special *utils-files-to-load*))

;; Define a function that can be called from the various load, compile, build files that does all the right stuff.
(defun clasp-do-load (&rest options)
  (cond (*clasp-demo-version*
         (format *standard-output* "~&; Building demo version for unreleased version ~a...~%" *clasp-next-version*))
        (*clasp-development-version*
         (format *standard-output* "~&; Building development version for unreleased version ~a...~%" *clasp-next-version*))
        (t
         (format *standard-output* "~&; Building version ~a...~%" *clasp-next-version*)))
  
  (let* ((operation (cond ((member :recompile options) :recompile)
                          ((member :compile options) :compile)
                          (t :load)))
         (propagate (member :propagate options))
         (opfun (ecase operation
                 (:load 'eksl-load-system)
                 (:compile 'eksl-compile-system)
                 (:recompile 'eksl-recompile-system))))
    
    (bind-implementation-dependent-parameters-for-compile (:development t ; keep everything around
                                                           :load-verbose t)
      ;; Load EKSL utilities
      (let ((*utils-files-to-load* *utils-files-to-load-for-clasp*))
        (if (and propagate (or (eq operation :compile) (eq operation :recompile)))
          (load (eksl-translate-logical-pathname "CLASP:EKSL-UTILS;compile-eksl-utils"))
          (load (eksl-translate-logical-pathname "CLASP:EKSL-UTILS;load-eksl-utils"))))
      
      ;; CLASP main stuff
      (funcall opfun 'clasp)
      
      (pushnew :CLASP *features*)
      (when *clasp-demo-version*
        (pushnew :DEMO *features*))
      
      
      ;; Scigraph or Plotter required by Interface
      (unless *clasp-suppress-loading-interface*
        #-MCL
	(unless (find-package "DWIM")
	  #+cmu
	  (require :scigraph)
	  #-cmu
	  (load (eksl-translate-logical-pathname "CLASP:DWIM;load-dwim")))
        #-MCL
	(unless (find-package "GRAPH")
	  (load (eksl-translate-logical-pathname "CLASP:SCIGRAPH;load-scigraph")))
        #+MCL
        (if (and propagate (or (eq operation :compile) (eq operation :recompile)))
          (load (eksl-translate-logical-pathname "CLASP:PLOTTER;compile-plotter"))
          (load (eksl-translate-logical-pathname "CLASP:PLOTTER;load-plotter"))))
      
      ;; Interface
      (unless *clasp-suppress-loading-interface*
        (funcall opfun 'clasp-interface))
      
      ;; Layered Systems
      ;; What we doing here is possibly defering loading the layered systems
      ;; based on the values of the *clasp-load-????* variables. We do this by
      ;; setting the variable *clasp-defer-load* to T
      ;; if the system is not supposed to be loaded and then loading the load
      ;; file for that system. The system should check *clasp-defer-load* and
      ;; if it is non-nil it should not do a `eksl-load-system', but it should
      ;; load the files necessary to allow us to load it later (ie., the
      ;; .translations and .system files).
      
      
      ;; Possibly load layers system if anybody else is to be loaded now.
      (when *clasp-load-layers*
        (if (and propagate (or (eq operation :compile) (eq operation :recompile)))
          (load (eksl-translate-logical-pathname "CLASP:CLASP-LAYERS;compile-layers"))
          (load (eksl-translate-logical-pathname "CLASP:CLASP-LAYERS;load-layers"))))
      
      
      ;; All of these require the layers system to be loaded before a clasp frame is built.
      (when *clasp-load-layers*
        ;; FBD
        (let ((*clasp-defer-load* (not *clasp-load-fbd*)))
          (if (and propagate (or (eq operation :compile) (eq operation :recompile)))
            (load (eksl-translate-logical-pathname "CLASP:FBD;compile-fbd"))
            (load (eksl-translate-logical-pathname "CLASP:FBD;load-fbd"))))
        
        ;; MSDD
        (let ((*clasp-defer-load* (not *clasp-load-msdd*)))
          (if (and propagate (or (eq operation :compile) (eq operation :recompile)))
            (load (eksl-translate-logical-pathname "CLASP:MSDD;compile-msdd"))
            (load (eksl-translate-logical-pathname "CLASP:MSDD;load-msdd"))))
        
        ;; IMSDD
        (let ((*clasp-defer-load* (not *clasp-load-imsdd*)))
          (if (and propagate (or (eq operation :compile) (eq operation :recompile)))
            (load (eksl-translate-logical-pathname "CLASP:IMSDD;compile-imsdd"))
            (load (eksl-translate-logical-pathname "CLASP:IMSDD;load-imsdd"))))))
    
    ;; Set the CLASP::*CLASP-VERSION* variable to something appropriate.
    (setf (symbol-value (intern "*CLASP-VERSION*" (find-package 'clasp)))
	  (concatenate 'string "Version " *clasp-next-version*
		       (cond (*clasp-development-version*
			      " (Development)")
			     (*clasp-demo-version*
			      " (Demo)")
			     (t
                              ""))))
    
    (pushnew "clasp:binary;patches" *eksl-all-patch-files* :test #'string=)
    (unless *clasp-suppress-loading-interface*
      (pushnew "clasp:interfacebinary;patches" *eksl-all-patch-files* :test #'string=))

    (eksl-copyright-notice "CLASP" "clasp-support@cs.umass.edu")))

;;; ***************************************************************************
;;; EOF




