;;;; -*- Mode:Common-Lisp; Package:utils; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/advise.lisp *-*
;;;; *-* Last-edit: Wednesday, March 23, 1994  13:44:45; Edited-By: anderson *-* 
;;;; *-* Machine: Chuck (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                              Advise Macros                             *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Experimental Knowledge Systems Laboratory
;;;             Paul R. Cohen, Principal Investigator
;;;             David L. Westbrook, Systems Manager
;;;             David M. Hart, Laboratory Manager
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  03-22-94 File Created.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(advise advised-functions unadvise))

;;; --*--
;;; ***************************************************************************


(defvar *advised-functions* nil
  "An association list from symbols naming advised function to symbols naming
original definitions.")

(defmacro advise (name &body forms)
  "Modifies function to do other stuff around the normal execution.  The normal
execution is done by calling the function DO-IT, which is created by macrolet."
  (let ((saved-name (gentemp (string name)))
	(doit-name  (intern "DO-IT"))
	(vals-name  (gentemp "VALS")))
    `(progn (setf (symbol-function ',saved-name)
		  (symbol-function ',name))
	    (setf (symbol-function ',name)
		  #'(lambda (&rest args)
		      (macrolet ((,doit-name () '(setf ,vals-name (multiple-value-list (apply (symbol-function ',saved-name) args)))))
				(let (,vals-name)
				  ,@forms
				  (values-list ,vals-name)))))
	    (push (cons ',name ',saved-name)
		  *advised-functions*)
	    ',name)))

(defun advised-functions ()
  (mapcar #'car *advised-functions*))

(defun unadvise (&optional name)
  (if name
      (let ((pair (assoc name *advised-functions*)))
	(when pair
	  (setf (symbol-function (car pair))
		(symbol-function (cdr pair)))
	  (setf *advised-functions*
		(delete pair *advised-functions*))
	  (unadvise name)))
      (progn (dolist (pair *advised-functions*)
	       (setf (symbol-function (car pair))
		     (symbol-function (cdr pair))))
	     (setf *advised-functions* nil))))

#+test
(defun my-load (fn)
  (load fn :verbose t))

#+test
(defun test-advise ()
  (let ((file "~/delete-me.lisp"))
    (with-open-file (out file :direction :output)
      (format out "~%"))
    (format t "~&First call shouldn't be advised.~%")
    (my-load file)
    (advise my-load
	    (format t "~&Hello there!~%")
	    (do-it))
    (format t "~&Second call should be advised.~%")
    (my-load file)
    (unadvise 'my-load)
    (format t "~&Third call should not be advised.~%")
    (my-load file)))

;;; ***************************************************************************
;;; EOF
