;;;; -*- Mode: Common-Lisp; Package: EKSL-UTILITIES; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/clim-colors.lisp *-*
;;;; *-* Last-Edit: Thursday, January 4, 1996  14:58:56; Edited-By: Westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                      CLIM Special Effects Package                      *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Matthew Schmill
;;;             Department of Computer and Information Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;; 04-16-95 created (schmill)
;;;
;;; ---------------------------------------------------------------------------
(in-package :u)

(export '(defrainbow draw-rainbow))

(defparameter +rainbow-groups+ (make-hash-table))

(defun defrainbow (name &rest colors)
  (setf (gethash name +rainbow-groups+)
    (apply #'vector 
	   (loop for col in colors append
		 (if (and (symbolp col) (color-named col))
		     (list (color-named col))
		   (list col))))))

(defun getrainbow (name)
  (or (gethash name +rainbow-groups+) (error "No rainbow named ~S." name)))

(defparameter +tan30+ 0.0524)

(defun draw-rainbow* (medium sx sy dx dy rainbow-group &key (pixelarity 20) (thickness 1))
  (let* ((distance (sqrt (+ (expt (- dy sy) 2) (expt (- dx sx) 2))))
	 (segments (/ distance pixelarity))
	 (xs (- dx sx))
	 (ys (- dy sy))
	 (rb (getrainbow rainbow-group))
	 (cc (length rb)))
    (loop for i from 0 to (1- (truncate segments)) do
	  (draw-line* medium (+ sx (* i (/ xs segments)))
		      (+ sy (* i (/ ys segments)))
		      (+ sx (* (1+ i) (/ xs segments)))
		      (+ sy (* (1+ i) (/ ys segments)))
		      :ink (aref rb (mod i cc))
		      :line-thickness (max 1 thickness)))
    (unless (zerop (- (truncate segments) segments))
      (draw-line* medium (+ sx (* (truncate segments) (/ xs segments)))
		  (+ sy (* (truncate segments) (/ ys segments)))
		  dx dy
		  :ink (aref rb (mod (ceiling segments) cc))
		  :line-thickness thickness))))

(defun draw-rainbow (medium spoint dpoint rainbow-group &rest keys)
  (apply #'draw-rainbow* medium (clim::point-x spoint) (clim::point-y spoint) (clim::point-x dpoint) (clim::point-y dpoint) 
	 rainbow-group keys))
  
#+TEST
(defpulsing-group ()
    :g3 (:12%-gray-color :50%-gray-color :88%-gray-color)
    :g2 (:50%-gray-color :88%-gray-color :12%-gray-color)
    :g1 (:88%-gray-color :12%-gray-color :50%-gray-color))

#+TEST
(defrainbow :rb1 :g1 :g2 :g3)

#+TEST
(defun draw-chain (medium &optional (pts '((20 20) (145 180) (300 400) (275 175) (250 50) (20 20))))
  (labels ((zit (remain &optional source)
	     (when remain
	       (when source
		 (draw-rainbow medium (first source) (second source)
			       (first (car remain)) (second (car remain)) :rb1 :arrow 3 :thickness 2))
	       (zit (cdr remain) (car remain)))))
    (zit pts)))
