;;;; -*- Mode:Common-Lisp; Package:EKSL-UTILITIES; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/geometry-primitives.lisp *-*
;;;; *-* Last-edit: Wednesday, June 21, 1995  16:25:15; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                        Geometry Primitives                             *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Mike Greenberg, Scott D. Anderson, David L. Westbrook
;;;             Department of Computer and Information Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  10-24-89 Fixed a coordinate orientation bug in `point-rotate-origin' and
;;;           its friends. Also fixed `point-rotate-around-point' to call
;;;           `point-rotate-origin*' with the correct number of arguments.
;;;           These weren't used until now.  (Westy)
;;;  03-20-90 Collapsed the separate `segment&parameter->point' functions to two:
;;;           segment&parameter->unrounded-point and segment&parameter->point.
;;;           Made the others obsolete, and removed them from functions that 
;;;           call them.  Further collapsing is possible, but I think the extra
;;;           typing in order to get unrounded points is a good thing, since 
;;;           most of Phoenix can't deal with unrounded points.  (Anderson)
;;;  06-24-90 Discovered anomalous behavior in `point-rotate-around-point' and
;;;           wrote `rotate-vector' to replace it.  Wrote a test function to prove
;;;           that `rotate-vector' is correct.  Left `point-rotate-around-point'
;;;           in case someone can prove to me that it is correct.  Almost no one 
;;;           uses it.  Also added documentation strings.  (Anderson)
;;;  11-12-90 Altered `average-point' to allow new point to be nil.  (Westy)
;;;  03-27-91 Removed GWIN package dependencies.  (Westy)
;;;  02-11-92 Put `map-pixels-on-line' back to its original form, in which it
;;;           returns early if the user's function returns :return.  Wrote a 
;;;           test function to show that lexical closures and `return-from' are
;;;           slower than functions and the use of `:return.'  (Anderson)
;;;  02-11-92 Modified `map-pixels-on-line' so that the user's function can 
;;;           return multiple values.  (Anderson)
;;;  02-18-92 Added use of `with-temp-points' to `map-pixels-on-line'.  (Westy)
;;;  04-12-94 Copied file from Phoenix to MESS system.  Changed package to 
;;;           :eksl-utilities.   (anderson)
;;;  04-12-94 Simplified point-separation for portability.  See the original
;;;           file for a possibly more efficient implementation.  (anderson)
;;;  04-12-94 Removed feature-edge functions.  Those will go in a different 
;;;           file, if anywhere.   (anderson)
;;;  04-12-94 Removed point-clip functions; should go elsewhere.  (anderson)
;;;  04-12-94 Removed polyline-intersects-cell-p.  (anderson)
;;;  04-12-94 Removed map-pixels-on-line and related functions.  (anderson)
;;;  04-13-94 Removed point-distance-from-rectangle because of dependency on
;;;           w: (Explorer) code.  (anderson)
;;;  05-07-95 Removed commented-out functions.  (anderson)
;;;  07-03-95 added macro-dispatch-character for point as #$(x y) (schmill)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(proclaim '(optimize (speed 3) (safety 1)))

(export '(square 
	   point-separation-squared*
	   point-separation-squared 
	   double-point-separation-squared 
	   point-separation 
	   point-separation-lessp 
	   point-separation-lessp-1 
	   point-difference* 
	   point-difference 
	   point-sum* 
	   point-sum 
	   point-round* 
	   point-round 
	   point-average 
	   point-rotate-origin* 
	   point-rotate-origin 
	   point-rotate-around-point 
	   rotate-vector 
	   angle-between-points 
	   cos-angle-among-points 
	   angle-among-points 
	   point-separation-and-sin-cos 
	   points-in-same-cell-p 
	   xy-segment-to-implicit-line 
	   segment-to-implicit-line 
	   segment-to-parametric-line 
	   same-side-p 
	   extend-segment 
	   inline-point-wrt-line 
	   point-wrt-line 
	   point-on-line-p 
	   point-left-of-line-p 
	   point-right-of-line-p 
	   parameter-of-nearest-point-on-line 
	   segment&parameter->unrounded-point 
	   segment&parameter->point 
	   nearest-point-on-segment 
	   point-distance-from-line-squared 
	   point-distance-from-segment-squared 
	   squared-distance-between-segments 
	   segment-intersection 
	   check-segment 
	   point-on-segment->parameter 
	   segment-intersection-parameters 
	   segment-intersection-parameter 
	   segments-intersect-p 
	   xy-segments-intersect-p 
	   half-line-intersects-segment-p 
	   quick-segment-intersects-cell-p 
	   segment-intersects-cell-p 
	   segment-box-intersection-parameter 
	   segment-cell-intersection-parameter 
	   segment-cell-interior-intersection-parameter 
	   segment-cell-exterior-intersection-parameter 
	   segment-cell-intersection 
	   segment-cell-interior-intersection 
	   segment-cell-exterior-intersection
	   sector-code
	   dist-from-rectangle
	   point-distance-from-square
	   point-distance-from-extent
	   line-circle-intersection-parameter 
	   segment-circle-intersection-parameter 
	   vector-end-point-unrounded 
	   vector-end-point 
	   find-z-coordinate-in-plane 
	   point-in-cell-p 
	   point-distance-from-square 
	   point-distance-from-extents 
	   segment-side-segments 
	   extent-intersection 
	   area-of-triangle))

;;; --*--
;;; ***************************************************************************

(proclaim '(inline square))
(defun square (x)
  "(* x x)"
  (* x x))

;;; ---------------------------------------------------------------------------
;;; point macro-dispatcher

(defun point-reader (stream subchar arg)
  (declare (ignore subchar arg))
  (cons 'point (read stream t NIL t)))

(set-dispatch-macro-character #\# #\$ #'point-reader)

;;;----------------------------------------------------------------------------
;;;
;;; Start: Point Functions
;;;

(defsubst point-separation-squared* (x1 y1 x2 y2)
  "Return the square of the distance between the two points.  The result is a
single-precision floating point number."
  (+ (square (coerce (- x1 x2) 'single-float))
     (square (coerce (- y1 y2) 'single-float))))

(defsubst point-separation-squared (p1 p2)
  "Return the square of the distance between the two points.  The result is a
single-precision floating point number."
  (declare (inline point-separation-squared*))
  (point-separation-squared* (point-x p1) (point-y p1) (point-x p2) (point-y p2)))

(defun double-point-separation-squared (p1 p2)
  "Return the square of the distance between the two points.  The result is a
double-precision floating point number."
  (+ (square (coerce (- (point-x p1) (point-x p2)) 'double-float))
     (square (coerce (- (point-y p1) (point-y p2)) 'double-float))))

(defun point-separation (p1 p2)
  "Return the distance between the two points +/- 1 meter."
  (declare (optimize speed (space 0)))
  (let ((dx (- (point-x p1) (point-x p2)))
	(dy (- (point-y p1) (point-y p2))))
    (isqrt (+ (square dx) (square dy)))))

(defconstant *dont-need-float* (isqrt (truncate most-positive-fixnum 2)))

(defun point-separation-lessp (p1 p2 distance)
  "Return T if distance between  `p1' and `p2' < distance."
  (let ((x (abs (- (point-x p1) (point-x p2))))
	(y (abs (- (point-y p1) (point-y p2)))))
    (when (and (< x distance) (< y distance))
      (if (< distance *dont-need-float*)
	  ;; x*x + y*y < most-positive-fixnum
	  (and (< (+ (square x) (square y)) (square distance)))
	  (and (< (+ (square (float x)) (square (float y)))
		  (square (float distance))))))))

(defun point-separation-lessp-1 (p1 p2 distance)
  "Return nil if separation between `p1' and `p2' > distance.  If
it is < distance, return the new distance.  This should be made obsolete."
  (let ((x (abs (- (point-x p1) (point-x p2))))
	(y (abs (- (point-y p1) (point-y p2))))
	d)
    (when (and (< x distance) (< y distance))
      (setf d (sqrt (+ (square (float x)) (square (float y)))))
      (and (< d distance) d))))


(proclaim '(inline point-difference* point-difference
		   point-sum* point-sum
		   point-round point-round*
		   ))

(defun point-difference* (p1 p2)
  "Return p1 = p1 - p2."
  (decf (point-x p1) (point-x p2))
  (decf (point-y p1) (point-y p2))
  p1)

(defun point-difference (p1 p2)
  "Return p1 - p2."
  (point (- (point-x p1) (point-x p2)) (- (point-y p1) (point-y p2))))

(defun point-sum* (p1 p2)
  "Return P1 = P1 + P2."
  (incf (point-x p1) (point-x p2))
  (incf (point-y p1) (point-y p2))
  p1)

(defun point-sum (p1 p2)
  "Return P1 + P2."
  (point (+ (point-x p1) (point-x p2)) (+ (point-y p1) (point-y p2))))

(defun point-round* (point)
  "Rounds `point'.  Destructively modifies 'point'."
  (setf (point-x point) (round (point-x point))
	(point-y point) (round (point-y point)))
  point)

(defun point-round (point)
  "Rounds `point'.  Returns a new point."
  (point (round (point-x point)) (round (point-y point))))

(defun point-average (new-point &rest points)
  "`new-point' is set to be the rounded arithmetic mean of `points'.
`new-point' is not included in the average.  If `new-point' is nil the point is
simple returned."
  (let ((n  (length points)))
    ;; more space efficient and a hell of a lot faster than using `reduce'
    ;; twice, if not as clean
    (do* ((p      (pop points) (pop points))
	  (totalx (point-x p)  (+ totalx (point-x p)))
	  (totaly (point-y p)  (+ totaly (point-y p))))
	 ((null points)
          (if new-point
            (point (round totalx n)
                   (round totaly n)
                   new-point)
            (point (round totalx n)
                   (round totaly n)))))))

;;;----------------------------------------------------------------------------
;;; Given a point and an orientation - return the point rotated around the orgin
;;; by the orientation (theta)

(proclaim '(inline point-rotate-origin*))
(defun point-rotate-origin* (theta pt)
  "Rotate the vector `pt' \(that is, a vector from the origin to `pt'\) through
an angle of `theta' radians COUNTERCLOCKWISE \(this is different from most
Explorer code\).  The `pt' argument is destructively modified, and the returned
point is not rounded."
  (let ((sin (sin theta))
	(cos (cos theta)))
    (psetf (point-x pt) (- (* (point-x pt) cos) (* (point-y pt) sin))
	   (point-y pt) (+ (* (point-y pt) cos) (* (point-x pt) sin)))
    pt))

(defun point-rotate-origin (theta pt)
  "Rotate the vector `pt' \(that is, a vector from the origin to `pt'\) through
an angle of `theta' radians COUNTERCLOCKWISE \(this is different from most
Explorer code\).  A more general function is `rotate-vector.' The returned point
is not rounded."
  (declare (inline point-rotate-origin*))
  (point-rotate-origin* theta (copy-point pt)))

(defun point-rotate-around-point (theta rotate-point around-point)
  "Just like `rotate-vector,' except: \(1\) theta is off by half a circle,
meaning that giving `theta' of 0 rotates by pi radians; \(2\) it destructively
modifies `around-point' to be the result; and \(3\) the result is not rounded."
  (point-sum* around-point
	      (point-rotate-origin* theta
				    (point-difference around-point rotate-point))))

(defun rotate-vector (hub point theta new-point)
   "Rotate the vector from `hub' to `point' around `hub' by an angle of `theta'
radians, CLOCKWISE.  For example, if `hub' is \(point 0 0\), `point' is \(point
1 0\), and `theta' is `fpi/2,' the result is \(point 0 1\).  In other words, a
90 degree rotation of an east vector is a south vector.  Remember, the vector
from \(point 0 0\) to \(point 0 1\) is a south vector because of the
``handedness'' of the coordinate system: X increases to the East and Y increases
to the South \(in ``normal'' coordinate systems, Y increases to the North.\).
The function has been thoroughly tested and can be relied upon.  See
`test-rotate-vector.'

The result is rounded and returned in `new-point,' which may be eq to either
`hub' or `point,' if you want.  To get counterclockwise rotation, you can negate
the `theta' argument."
   (declare (optimize speed (space 0)))
   ;; Works by translating the vector so that hub is at the origin, rotating it,
   ;; then translating back.  The clockwise rotation matrix is:
   ;; |  cos theta   -sin theta |
   ;; |  sin theta    cos theta |
   (let* ((hub-x (point-x hub))
	  (hub-y (point-y hub))
	  (pt-x  (point-x point))
	  (pt-y  (point-y point))
	  (x     (- pt-x hub-x))
	  (y     (- pt-y hub-y))
	  ;; I did some timings, and using the `cis' function is worse in both
	  ;; time and number-consing.  The `cis' function is in fact faster, but
	  ;; the compiler puts in enough type-checking code to more than lose
	  ;; the advantage.
	  (cos	(cos theta))
	  (sin	(sin theta))
	  (rot-x (round (- (* cos x) (* sin y))))
	  (rot-y (round (+ (* sin x) (* cos y)))))
      (point (+ hub-x rot-x)
	     (+ hub-y rot-y)
	     new-point)))

#+test
(defun test-rotate-vector (&optional (seconds-for-sweep 10))
  "The user picks the hub of a circle via `spum,' and this function demonstrates
that `rotate-vector' will sweep a vector around the circle.  The vector is 2
kilometers long and initially points East."
  (let* ((hub   (hspum))
	 ;; starting point is 2km due East of hub.
	 (start (point-sum hub (point 2000 0)))
	 (end   (point)))
    (dotimes (a 360)
      (let ((end-point (rotate-vector hub start (degrees->radians a) end)))
	(with-highlight
	    (highlight-line hub end-point)
	  (sleep (/ seconds-for-sweep 360)))))))

;;;----------------------------------------------------------------------------

(defun angle-between-points (p1 p2)
  "Return the angle of the vector from `p1' to `p2' (in radians).  Note that the
result will depend on the handedness and such of the coordinate system."
  (atan (- (point-y p2) (point-y p1))
	(- (point-x p2) (point-x p1))))

(defun cos-angle-among-points (A O B)
  "Return the cosine of the angle AOB, where A, O, and B are all points.  Uses
the law of cosines to avoid any trigonometric functions.  See also the function
`angle-among-points.'"
  (let ((Ax (point-x A))
	(Ay (point-y A))
	(Ox (point-x O))
	(Oy (point-y O))
	(Bx (point-x B))
	(By (point-y B)))
    ;; Translate to origin first
    (let ((A1x (- Ax Ox))
	  (A1y (- Ay Oy))
	  (B1x (- Bx Ox))
	  (B1y (- By Oy)))
      ;; squared lengths of triangle sides
      (let ((OA (+ (* A1x A1x) (* A1y A1y)))
	    (OB (+ (* B1x B1x) (* B1y B1y)))
	    (AB (+ (square (- A1x B1x))
		   (square (- A1y B1y)))))
	;; Finally, law of cosines
	(/ (- AB OA OB)
	   (* -2 (sqrt (* OA OB))))))))

#+test
(defun test-cos-angle-among-points ()
  (flet ((test (theta)
	   (let* ((alpha (random 2fpi))
		 (beta  (+ alpha theta)))
	     (let ((A (point (cos alpha) (sin alpha)))
		   (O (point 0 0))
		   (B (point (cos beta) (sin beta))))
	       (spy (- (cos theta) (cos-angle-among-points A O B)))))))
    (loop for i from 0.0 to 2fpi by .1 do (test i))))

(defun angle-among-points (A O B)
  "Return the angle AOB, where A, O, and B are all points.  See also the
function `cos-angle-among-points,' which is faster if all you want is the cosne
of the angle."
  (acos (cos-angle-among-points A O B)))

(defun point-separation-and-sin-cos (p1 p2)
  "Find the distance between points.  Return the sine and cosine of the angle
between them.  Equivalently, return the magnitude, y-component and x-component
of the vector from p1 to p2.  This is cheaper than trigonometric functions."
  (declare (values distance sine cosine))
  (let* ((x1 (point-x p1))
	 (y1 (point-y p1))
	 (x2 (point-x p2))
	 (y2 (point-y p2))
	 (dx (float (- x2 x1)))
	 (dy (float (- y2 y1)))
	 (dist (sqrt (+ (square dx) (square dy))))
	 )
    
    (if (zerop dist)
	(values 0 0 0)
	(values dist (/ dy dist) (/ dx dist)))))

#+test
(defun test-point-separation-and-sin-cos ()
  (spy (point-separation-and-sin-cos (point 1 0) (point 0 1)))
  (spy (point-separation-and-sin-cos (point 0 1) (point 1 0))))

(defun points-in-same-cell-p (p1 p2 resolution)
  "Return T/NIL if the two points share the same cell at the given resolution."
  (let ((mask (- (exp2 resolution) 1)))
    (and (= (logand mask (point-x p1)) (logand mask (point-x p2)))
	 (= (logand mask (point-y p1)) (logand mask (point-y p2))))))

;;;
;;; End: Point Functions
;;;
;;;----------------------------------------------------------------------------
;;;----------------------------------------------------------------------------
;;;
;;; Start: Segment -> Segment Functions
;;;

(proclaim '(inline xy-segment-to-implicit-line))
(defun xy-segment-to-implicit-line (xk yk xl yl)
  "Given a line segment, return the implict form Ax + By + C = 0."
  (declare (values a b c))
  (let* ((x0 xk)
	 (f  (- xl x0))
	 (y0 yk)
	 (g  (- yl y0)))
    (values (- g) f (- (* x0 g) (* y0 f)))))

(defun segment-to-implicit-line (pk pl)
  "Given a line segment, return the implict form Ax + By + C = 0."
  (declare (values a b c)
	   (inline xy-segment-to-implicit-line))
  (xy-segment-to-implicit-line (point-x pk) (point-y pk) (point-x pl) (point-y pl)))

;; Just for future reference...
;;<< While compiling SEGMENT-TO-PARAMETRIC-LINE >> The inline
;;function SEGMENT-TO-PARAMETRIC-LINE was used by
;;POLYLINE->SMALL-SEGMENT-POLYLINE before it was defined.

(proclaim '(inline segment-to-parametric-line))
(defun segment-to-parametric-line (p0 p1)
  "Given a segment, return the parameters of the line
x = Xo + Fs
y = Yo + Gs."
  (declare (values f g))
  (values (- (point-x p1) (point-x p0))
	  (- (point-y p1) (point-y p0))))


(defun same-side-p (line-pt1 line-pt2 point1 point2)
  "Return T if `point1' and `point2' are both in the same half plane determined
by `linep1' and `linep2'."
  (let* ((dx  (- (point-x line-pt2) (point-x line-pt1)))
	 (dy  (- (point-y line-pt2) (point-y line-pt1)))
	 (dx1 (- (point-x point1) (point-x line-pt1)))
	 (dy1 (- (point-y point1) (point-y line-pt1)))
	 (dx2 (- (point-x point2) (point-x line-pt2)))
	 (dy2 (- (point-y point2) (point-y line-pt2))))
    (< 0 (* (- (* dx dy1) (* dy dx1)) (- (* dx dy2) (* dy dx2))))))

(defun extend-segment (p0 p1 extend-amount &optional (extend-to (point)))
  "Extend the segment from `p0' to `p1' by `extend-amount'.  Returns the new
segment end [p1] rounded to meters."
  (multiple-value-bind (f g) (segment-to-parametric-line p0 p1)
    (when (and (zerop f) (zerop g))
      (error "P0 and P1 are the same point:  ~s" p0))
    (let* ((distance (point-separation p0 p1))
	   (f (/ f distance))
	   (g (/ g distance)))
      (point (+ (point-x p0) (round (* f (+ distance extend-amount))))
	     (+ (point-y p0) (round (* g (+ distance extend-amount))))
	     extend-to))))

;;;
;;; End: Segment -> Segment Functions
;;;
;;;----------------------------------------------------------------------------



;;;----------------------------------------------------------------------------
;;;
;;; Start: Point/Segment Functions
;;;

(proclaim '(inline inline-point-wrt-line))
(defun inline-point-wrt-line (point p0 p1)
  "Return 0 if `point' is on the line, < 0 if left, > 0 if right."
  (let ((a (- (point-x p1) (point-x p0)))
	(b (- (point-y p1) (point-y p0))))

    (- (* b (- (point-x point) (point-x p0)))
       (* a (- (point-y point) (point-y p0))))))

(defun point-wrt-line (point p0 p1)
  "Return 0 if `point' is on the line, < 0 if left, > 0 if right."
  (inline-point-wrt-line point p0 p1))

(defun point-on-line-p (point p0 p1)
  "Determine if `point' lies along line defined by segment."
  (= 0 (inline-point-wrt-line point p0 p1)))

(defun point-left-of-line-p (point p0 p1)
  "Determine if `point' is in left half-plane defined by the directed line segment."
  (< 0 (inline-point-wrt-line point p0 p1)))

(defun point-right-of-line-p (point p0 p1)
  "Determine if `point' is in right half-plane defined by the directed line segment."
  (< (inline-point-wrt-line point p0 p1) 0))

(proclaim '(inline parameter-of-nearest-point-on-line))
(defun parameter-of-nearest-point-on-line (J K L)
  "Converts the segment [K,L] to a parametric line and returns the parameter of
the point on the line nearest to point `J.' Adapted from Bowyer and Woodark,
section 3.2, page 47--48.  Bowyer and Woodark solve for the distance of `J' from
the line.  An intermediate value is the parameter of the nearest point on the
line, so this function simple returns that parameter."
  (declare (inline segment-to-parametric-line))
  (multiple-value-bind (f g)
      (segment-to-parametric-line K L)
    
    (let* ((xk    (point-x K))
	   (yk    (point-y K))
	   (xkj   (float (- xk (point-x J))))
	   (ykj   (float (- yk (point-y J))))
	   
	   (denom (+ (square f) (square g))))
      (if (zerop denom)
	  ;; denom=0 means K=L, so we can really give either 0.0 or 1.0
	  0.0
	  (/ (- (+ (* xkj f) (* ykj g))) 
	     denom)))))

(proclaim '(inline segment&parameter->unrounded-point))
(defun segment&parameter->unrounded-point (p0 p1 parameter p2)
  "Given a segment and a parameter, return a point on the segment.  The new
point is not rounded and is returned in `p2'."
  ;; The result of this calculation will typically not be a pair of integers,
  ;; but we use integer-specified points throughout.  The most ``accurate''
  ;; results are obtained if POINT-ROUND is subsequently used to convert the
  ;; coordinates to integers.  In this case, ``accuracy'' means that the
  ;; function POINT-ON-SEGMENT->PARAMETER is the inverse of this function.
  ;; Given the range of our points, those functions are not perfect inverses,
  ;; but the remaining error is due to floating-point approximation.  It's
  ;; possible to go to double-precision floating point, which helps.
  (multiple-value-bind (f g) (segment-to-parametric-line p0 p1)
    (point (+ (point-x p0) (* parameter f))
	   (+ (point-y p0) (* parameter g))
	   p2)))

(proclaim '(inline segment&parameter->point))
(defun segment&parameter->point (p0 p1 parameter p2)
  "Given a segment and a parameter, return a point on the segment.  The point is
rounded and is returned in `p2,' which may be eq to p0 or p1 if you wish."
  (declare (inline segment&parameter->unrounded-point))
  (point-round* (segment&parameter->unrounded-point p0 p1 parameter p2)))

(defun nearest-point-on-segment  (J K L)
  "Find the point on the segment [K,L] nearest to `J'.  The returned point may
be `eq' to either K or L."
  (declare (inline parameter-of-nearest-point-on-line
		   segment&parameter->point))
  ;; This could be slightly more efficient by eliminating redundant calculations
  ;; (e.g.  segment-to-parametric-line gets called twice), but I don't think
  ;; it's worth it.
  (let ((tt (parameter-of-nearest-point-on-line J K L)))
    (cond ((<= tt 0.0) K)
	  ((<= 1.0 tt) L)
	  (t
	   (segment&parameter->point K L tt (point))))))

(defun point-distance-from-line-squared (J K L)
  (declare (inline point-separation-squared))
  ;; As with `nearest-point-on-segment,' this could be made more efficient
  (let ((tt  (parameter-of-nearest-point-on-line J K L)))
    (let ((tmp (segment&parameter->point K L tt (point))))
      (point-separation-squared tmp J))))

(defun point-distance-from-segment-squared (J K L)
  "The distance squared from point J to the line segment K-L.  Note that this
function conses one point and discards it."
  ;; As with `nearest-point-on-segment,' this could be made more efficient
  (declare (inline point-separation-squared))
  (let ((tt  (parameter-of-nearest-point-on-line j k l)))
    ;; the following is the only difference between this function and
    ;; point-distance-from-line-squared
    (cond ((> tt 1.0) (setf tt 1.0))
	  ((< tt 0.0) (setf tt 0.0)))
    (let ((tmp (segment&parameter->point K L tt (point))))
      (point-separation-squared tmp j))))

(defun squared-distance-between-segments (J K L M)
  "Returns the squared distance between the segments [J,K] and [L,M]."
  (if (segment-intersection J K L M)
      0
      (min (point-distance-from-segment-squared J L M)
	   (point-distance-from-segment-squared K L M)
	   (point-distance-from-segment-squared L J K)
	   (point-distance-from-segment-squared M J K))))
       
;;;
;;; End: Point/Segment Functions
;;;
;;;----------------------------------------------------------------------------



;;;----------------------------------------------------------------------------
;;;
;;; Start: Segment x Segment Functions
;;;

(defun segment-intersection (pk pl pm pn)
  "Return the point of intersection or nil.  The returned point is rounded.
Copied from Bowyer & Woodwark."
  (when (or (point= pk pm) (point= pk pn))
    (return-from segment-intersection pk))
  
  (when (or (point= pl pm) (point= pl pn))
    (return-from segment-intersection pl))
  
  (let* ((xk (point-x pk))
	 (yk (point-y pk))
	 (xl (point-x pl))
	 (yl (point-y pl))
	 (xm (point-x pm))
	 (ym (point-y pm))
	 (xn (point-x pn))
	 (yn (point-y pn))
	 (xlk (- xl xk))
	 (ylk (- yl yk))
	 (xnm (- xn xm))
	 (ynm (- yn ym))
	 (xmk (- xm xk))
	 (ymk (- ym yk))
	 (det (- (* xnm ylk) (* ynm xlk)))
	 detinv s tt)
    
    ;; Parallel
    (when (zerop det)
      (return-from segment-intersection nil))
    
    (setf detinv (/ 1.0 det))
    (setf s (* (- (* xnm ymk) (* ynm xmk)) detinv))
    (when (<= 0.0 s 1.0)
      (setf tt (* (- (* xlk ymk) (* ylk xmk)) detinv))
      (if (<= 0.0 tt 1.0)
	  (point (round (+ xk (* xlk s))) (round (+ yk (* ylk s))))))))


(defmacro check-segment (p0 p1)
  "Error-checking macro to ensure that [P0,P1] is a possible segment; namely, P0
and P1 are different."
  `(assert (not (point= ,p0 ,p1)) () "Need two different points to specify a segment."))

(defun point-on-segment->parameter (p0 p1 point &optional error-check?)
  "If `point' is on the segment [P0,P1], returns the parametric specification of
`point'.  Actually, this works for any point on the half-infinite line [ray]
from `p0' through `p1' to infinity.  If `point' is on the line but not on the
ray, the parameter returned is the negative of the correct answer; that is, the
correct answer is negative, but the answer returned is the absolute value of the
correct answer."
  ;; Based on the observation that Param : 1 :: distance(p0,point) :
  ;; distance(p0,p1).  Since this needs two expensive square root operations, we
  ;; use the following code, which requires only one.

  ;; Centered points are used to avoid round-off errors.  Also, the use of a
  ;; single SQRT turns out to reduce round-off as well.
  (check-segment p0 p1)
  (let ((param  (sqrt (/ (double-point-separation-squared p0 point)
			 (double-point-separation-squared p0 p1)))))
    (when (and error-check?
	       ;; a distance of one is the smallest resolution of rounded points
	       (< 1 (point-distance-from-line-squared point p0 p1)))
      (error "POINT = ~s is not on the line [P0,P1] = [~s,~s]." point p0 p1))
    param))

#+test
(defun test-point-on-segment->parameter (&optional (range 80))
  "Tests that the function POINT-ON-SEGMENT->PARAMETER works correctly."
  (let ((tmp1  (point))
	(tmp2  (point))
	(tmp3  (point))
	(tmp4  (point)))
    (dotimes (y 1000)
      (let ((p0  (point (random range) (random range) tmp1))
	    (p1  (point (random range) (random range) tmp2)))
	(dotimes (x 1000)
	  (let* ((param       (sys:double-float (random 1.0
                                                        #+ignore (/ (square range)
                                                                    (point-separation-squared p0 p1)))
                                                ))
		 (point       (segment&parameter->centered-point p0 p1 param tmp3))
		 (param-again (point-on-segment->parameter p0 p1 point t))
		 (point-again (segment&parameter->centered-point p0 p1 param-again tmp4)))
	    ;; We re-compute a point and compare those because parameters, being
	    ;; real numbers, can specify points in between the rounded points.
	    ;; Essentially, two parameters are "equal" iff they specify the same
	    ;; rounded point, even though they are not "=".
	    (if (not (point= (point-round* point) (point-round* point-again)))
		(error "failed:  p0 = ~s  p1 = ~s, ~%point = ~20t~s ~%point-again = ~20t~s~%param = ~20t~s~%param again = ~20t~s~%~
                                 distance^2(p0,point) = ~s ;  distance^2(p0,p1) = ~s"
		       p0 p1 point point-again param param-again
		       (point-separation-squared p0 point)
		       (point-separation-squared p0 p1)))))))))

#+test
(defun time-point-on-segment->parameter ()
  "Times how long and how much storage the function POINT-ON-SEGMENT->PARAMETER
takes.  The result is 500us on average.  SQRT is 151us.  Floating divide is 9us.
POINT-CENTER* is 69us, and POINT-ROUND* is 6us [converting to floating point
costs a lot].  POINT-SEPARATION-SQUARED is 44us.

Total is: 151+9+3[69]+3[6]+2[44]=473.  The remainder is function-call overhead
for POINT-SEPARATION-SQUARED and SQRT [all the rest are inline functions.
Believe it or not, CHECK-SEGMENT and \(WHEN \(AND nil ...\)\) take only about
3us."
  (let ((sum  0))
    (dotimes (x 1000)
      (incf sum 
	    (let ((p0     (point (random 8000) (random 8000)))
		  (p1     (point (random 8000) (random 8000)))
		  (point  (point (random 8000) (random 8000))))
	      (time:timeit (:repeat 100 :values :cpu :units :microseconds)
		(point-on-segment->parameter p0 p1 point nil)))))
    (format t "~&Average cpu time of ~s = ~d~%" 'point-on-segment->parameter (/ sum 1000)))
  (let ((sum  0))
    (dotimes (x 1000)
      (incf sum 
	    (let ((p0     (point (random 8000) (random 8000)))
		  (p1     (point (random 8000) (random 8000))))
	      (time:timeit (:repeat 100 :values :cpu :units :microseconds)
		(point-separation-squared p0 p1)))))
    (format t "~&Average cpu time of ~s = ~d~%" 'point-separation-squared (/ sum 1000)))
  (let ((sum 0))
    (dotimes (x 1000)
      (incf sum
	    (let ((n  (random (square 8000))))
	      (time:timeit (:repeat 100 :values :cpu :units :microseconds)
		(sqrt n)))))
    (format t "~&Average cpu time of ~s = ~d~%" 'sqrt (/ sum 1000)))
  (let ((sum 0))
    (dotimes (x 1000)
      (incf sum
	    (let ((num   (random (coerce (square 8000) 'float)))
		  (denom (random (coerce (square 8000) 'float))))
	      (time:timeit (:repeat 100 :values :cpu :units :microseconds)
		(/ num denom)))))
    (format t "~&Average cpu time of ~s = ~d~%" '/ (/ sum 1000)))
  (let ((sum 0))
    (dotimes (x 1000)
      (incf sum
	    (let ((point  (point (random 8000) (random 8000))))
	      (time:timeit (:repeat 100 :values :cpu :units :microseconds)
		(point-center* point)))))
    (format t "~&Average cpu time of ~s = ~d~%" 'point-center*  (/ sum 1000)))
  (let ((sum 0))
    (dotimes (x 1000)
      (incf sum
	    (let ((point  (point (random 8000.0) (random 8000.0))))
	      (time:timeit (:repeat 100 :values :cpu :units :microseconds)
		(point-round* point)))))
    (format t "~&Average cpu time of ~s = ~d~%" 'point-round* (/ sum 1000)))) 

(defun segment-intersection-parameters (pk pl pm pn)
  "Return the parameters of the intersection of the lines determined by
segments [Pk,Pl] and [Pm,Pn].  The first value is the parameter on
[Pk,Pl], the second on [Pm,Pn].  If the segments intersect, both
parameters will be between 0 and 1, inclusive.  If the lines do not
intersect, NIL is returned.  If the lines are coincident, the correct
result is calculated."
  (declare (values parameter-on-pk-pl parameter-on-pm-pn))
  (when (point= pk pm)
    (return-from segment-intersection-parameters (values 0.0 0.0)))
  (when (point= pk pn)
    (return-from segment-intersection-parameters (values 0.0 1.0)))
  (when (point= pl pm)
    (return-from segment-intersection-parameters (values 1.0 0.0)))
  (when (point= pl pn)
    (return-from segment-intersection-parameters (values 1.0 1.0)))
  
  (let* ((xk (point-x pk))
	 (yk (point-y pk))
	 (xl (point-x pl))
	 (yl (point-y pl))
	 (xm (point-x pm))
	 (ym (point-y pm))
	 (xn (point-x pn))
	 (yn (point-y pn))
	 (xlk (- xl xk))
	 (ylk (- yl yk))
	 (xnm (- xn xm))
	 (ynm (- yn ym))
	 (xmk (- xm xk))
	 (ymk (- ym yk))
	 (det (- (* xnm ylk) (* ynm xlk))))
    (when (zerop det)
      ;; They have the same slope, so they're either parallel (possibily
      ;; colinear but non-intersecting) or coincident, which means there are
      ;; infinitely many points of intersection.  (There cannot be just one
      ;; point, because that would necessarily be an endpoint, and we've taken
      ;; care of that case with the POINT= code above.) We just return one,
      ;; namely an endpoint of one segment that is on the other segment,
      ;; specified twice---as parameters on each line.
      (cond ((point-on-line-p pk pm pn)
	     (return-from segment-intersection-parameters
                (values 0.0 (point-on-segment->parameter pm pn pk))))
	    ((point-on-line-p pl pm pn)
	     (return-from segment-intersection-parameters
                (values 1.0 (point-on-segment->parameter pm pn pl))))
	    ((point-on-line-p pm pk pl)
	     (return-from segment-intersection-parameters
                (values (point-on-segment->parameter pk pl pm) 0.0)))
	    ((point-on-line-p pn pk pl)
	     (return-from segment-intersection-parameters
                (values (point-on-segment->parameter pk pl pn) 1.0)))
	    (t
	     (return-from segment-intersection-parameters nil))))
    (let* ((detinv         (/ 1.0 det))
	   (parameter-on-pk-pl (* (- (* xnm ymk) (* ynm xmk)) detinv))
	   (parameter-on-pm-pm (* (- (* xlk ymk) (* ylk xmk)) detinv)))
      (values parameter-on-pk-pl parameter-on-pm-pm))))

#+test
(defun test-segment-intersection-parameters (&optional (times 1000))
  (labels ((random-sign ()
	     ;; returns either +1 or -1
	     (if (zerop (random 2)) -1 +1))
	   (random-between (low high)
	     (+ low (random high)))
	   (four-points-on-a-line (slope p1 p2 p3 p4)
	     (let ((x   (random 1000)))
	       (point x (random 1000) p1)
	       (let ((delta-x  (random-between 100 200)))
		 (point (+ x delta-x) (+ x (round (* delta-x slope))) p2))
	       (let ((delta-x  (random-between 300 400)))
		 (point (+ x delta-x) (+ x (round (* delta-x slope))) p3))
	       (let ((delta-x  (random-between 500 600)))
		 (point (+ x delta-x) (+ x (round (* delta-x slope))) p4)))))
    (let ((pk  (point))
	  (pl  (point))
	  (pm  (point))
	  (pn  (point)))
      (dotimes (x times)
	;; 10% of all tests will be same-slope tests
	(if (= 9 (random 10))
	    (let ((slope  (* (random 100.0) (random-sign))))
	      (case (random 4)
		(0  (four-points-on-a-line slope pm pk pn pl)
		    (if (null (segment-intersection-parameters pk pl pm pn))
			(error  "Colinear, intersecting segments PM PK PN PL doesn't work.")))
		(1  (four-points-on-a-line slope pm pk pl pn)
		    (if (null (segment-intersection-parameters pk pl pm pn))
			(error  "Colinear, intersecting segments PM PK PL PN doesn't work.")))
		(2  (four-points-on-a-line slope pk pm pl pn)
		    (if (null (segment-intersection-parameters pk pl pm pn))
			(error  "Colinear, intersecting segments PK PM PL PN doesn't work.")))
		(3  (four-points-on-a-line slope pk pm pn pl)
		    (if (null (segment-intersection-parameters pk pl pm pn))
			(error  "Colinear, intersecting segments PK PM PN PL doesn't work.")))))
	    (if (null (segment-intersection-parameters
			(point (random 1000) (random 1000) pk)
			(point (random 1000) (random 1000) pl)
			(point (random 1000) (random 1000) pm)
			(point (random 1000) (random 1000) pn)))
		(error "some damn thing.")))))))

(defun segment-intersection-parameter (pk pl pm pn)
  "Return the parameter of intersection on the segment [Pk,Pl].  If the
segments don't intersect, return NIL.  If the segments are coincident,
an endpoint of [Pm,Pn] that is on [Pk,Pl] is returned."
  (multiple-value-bind (parameter-on-pk-pl parameter-on-pm-pn)
      (segment-intersection-parameters pk pl pm pn)
    (if (and (not (null parameter-on-pk-pl))
	     (<= 0 parameter-on-pk-pl 1)
	     (<= 0 parameter-on-pm-pn 1))
	parameter-on-pk-pl)))

#+test
(defun test-segment-intersection-parameter (&optional (pk (hspum)) (pl (hspum)) (pm (hspum)) (pn (hspum)))
  (highlight-line pk pl)
  (highlight-line pm pn)
  (let ((param (time:timeit (:cpu :cons :number-cons)
		 (segment-intersection-parameter pk pl pm pn))))
    (spy param)
    (highlight-point (segment&parameter->point pk pl param (point)))
    param))

;; The following version is slightly more efficient, since it avoids computing
;; Tt=parameter-on-pk-pl when they don't intersect.  I think the savings is
;; minimal and doesn't justify the loss of perspicuity.  --SDA 9/10/89
#+comment
(defun segment-intersection-parameter (pk pl pm pn)
  "Return the parameter of intersection on the segment Pk Pl.  If the segments
don't intersect, return NIL.  (bug: if the lines are parallel and intersect, nil
is returned)"
  (when (or (point= pk pm) (point= pk pn))
    (return-from segment-intersection-parameter 0.0))
  
  (when (or (point= pl pm) (point= pl pn))
    (return-from segment-intersection-parameter 1.0))
  
  (let* ((xk (point-x pk))
	 (yk (point-y pk))
	 (xl (point-x pl))
	 (yl (point-y pl))
	 (xm (point-x pm))
	 (ym (point-y pm))
	 (xn (point-x pn))
	 (yn (point-y pn))
	 (xlk (- xl xk))
	 (ylk (- yl yk))
	 (xnm (- xn xm))
	 (ynm (- yn ym))
	 (xmk (- xm xk))
	 (ymk (- ym yk))
	 (det (- (* xnm ylk) (* ynm xlk)))
	 detinv s tt)
    
    ;; Parallel
    (when (zerop det)
      (return-from segment-intersection-parameter nil))
    
    (setf detinv (/ 1.0 det))
    (setf s (* (- (* xnm ymk) (* ynm xmk)) detinv))
    (when (<= 0.0 s 1.0)
      (setf tt (* (- (* xlk ymk) (* ylk xmk)) detinv))
      (if (<= 0.0 tt 1.0)
	  s))))

(defun segments-intersect-p (pk pl pm pn)
  "Return the point of intersection or nil. Copied from  Bowyer & Woodwark."
  (when (or (point= pk pm) (point= pk pn))
    (return-from segments-intersect-p pk))
  
  (when (or (point= pl pm) (point= pl pn))
    (return-from segments-intersect-p pl))
  
  (let* ((xk (point-x pk))
	 (yk (point-y pk))
	 (xl (point-x pl))
	 (yl (point-y pl))
	 (xm (point-x pm))
	 (ym (point-y pm))
	 (xn (point-x pn))
	 (yn (point-y pn))
	 (xlk (- xl xk))
	 (ylk (- yl yk))
	 (xnm (- xn xm))
	 (ynm (- yn ym))
	 (xmk (- xm xk))
	 (ymk (- ym yk))
	 (det (- (* xnm ylk) (* ynm xlk)))
	 detinv s tt)
    
    ;; Parallel
    (when (zerop det)
      (return-from segments-intersect-p nil))
    
    (setf detinv (/ 1.0 det))
    (setf s (* (- (* xnm ymk) (* ynm xmk)) detinv))
    (when (<= 0.0 s 1.0)
      (setf tt (* (- (* xlk ymk) (* ylk xmk)) detinv))
      (<= 0.0 tt 1.0))))

(defun xy-segments-intersect-p (xk yk xl yl xm ym xn yn)
  "Return the point of intersection or nil. Copied from  Bowyer & Woodwark."
  (when (or (and (= xk xm) (= yk ym))
	    (and (= xk xn) (= yk yn))
	    (and (= xl xm) (= yl ym))
	    (and (= xl xn) (= yl yn)))
    (return-from xy-segments-intersect-p t))
  
  (let* ((xlk (- xl xk))
	 (ylk (- yl yk))
	 (xnm (- xn xm))
	 (ynm (- yn ym))
	 (xmk (- xm xk))
	 (ymk (- ym yk))
	 (det (- (* xnm ylk) (* ynm xlk)))
	 detinv s tt)
    
    ;; Parallel
    (when (zerop det)
      (return-from xy-segments-intersect-p nil))
    
    (setf detinv (/ 1.0 det))
    (setf s (* (- (* xnm ymk) (* ynm xmk)) detinv))
    (when (<= 0.0 s 1.0)
      (setf tt (* (- (* xlk ymk) (* ylk xmk)) detinv))
      (<= 0.0 tt 1.0))))

(defun half-line-intersects-segment-p (l0 l1 s0 s1)
  "Returns T iff the half-infinite line [aka the ray] starting at L0 and going
through L1 intersects the segment [S0,S1]."
  ;; T1 is the position of the intersection on the half-infinite line
  ;; T2 is the position of the intersection on the segment
  (multiple-value-bind (t1 t2) (segment-intersection-parameters l0 l1 s0 s1)
    ;; T1 is null if they don't intersect and negative if it's the wrong half of
    ;; the line.  T2 is between 0 and 1 iff the line actually crosses the
    ;; segment.
    (and (not (null t1))
	 (plusp t1)
	 (<= 0.0 t2 1.0))))

(defun quick-segment-intersects-cell-p (p0 p1 cell-point resolution)
  "True iff the segment [P0,P1] passes through the cell containing `cell-point'
at `resolution'.  Returns the point [rounded] where the segment intersects a
diagonal of the cell.  [BUG: fails if the segment ends inside the cell without
intersecting a diagonal.]"
  ;; This algorithm uses the fact that if a line passes through a cell, it
  ;; intersects at least one of the diagonals
  (let* ((min-x (trunc2 (point-x cell-point) resolution))
	 (min-y (trunc2 (point-y cell-point) resolution))
	 (cell-size (exp2 resolution))
	 (max-x (+ min-x cell-size))
	 (max-y (+ min-y cell-size))
	 (tmp1 (point))
	 (tmp2 (point)))
    (let ((intersection  (or (segment-intersection-parameter
			       p0 p1
			       ;; upper left 
			       (point min-x min-y tmp1)
			       ;; lower right
			       (point max-x max-y tmp2))
			     (segment-intersection-parameter
			       p0 p1
			       ;; upper right
			       (point max-x min-y tmp1)
			       ;; lower left
			       (point min-x max-y tmp2)))))
      (and intersection
	   (segment&parameter->point p0 p1 intersection tmp1)))))

(defun segment-intersects-cell-p (p0 p1 point resolution)
  "Returns T iff the segment [P0,P1] goes through the cell of size `resolution'
containing `point'."
  (assert (not (point= p0 p1)) () "Need two different points for a segment.")
  (and (segment-cell-intersection-parameter p0 p1 point resolution) t))

(defun segment-box-intersection-parameter (p0 p1 box-north box-south box-east box-west)
  "Returns the parameter on [P0,P1] of the intersection point of the segment
[P0,P1] with the rectangle defined by `box-north' `box-south' `box-east', and
`box-west'.  The first two are y-coordinates, while the latter two are
x-coordinates."
  (let* ((tmp1 (point))
	 (tmp2 (point))
	 (min-param 100)
	 (this-param))
    (macrolet ((intersect-edge (edge-point1 edge-point2)
		 `(when (and (setf this-param
				   (segment-intersection-parameter
				     p0 p1 ,edge-point1 ,edge-point2))
			     (< this-param min-param))
		    (setf min-param this-param))))
      ;; North edge of box
      (intersect-edge (point box-west box-north tmp1)
		      (point box-east box-north tmp2))
      ;; East edge of box
      (intersect-edge (point box-east box-north tmp1)
		      (point box-east box-south tmp2))
      ;; South edge of box
      (intersect-edge (point box-west box-south tmp1)
		      (point box-east box-south tmp2))
      ;; West edge of box
      (intersect-edge (point box-west box-north tmp1)
		      (point box-west box-south tmp2)))
    ;; Finally, return MIN-PARAM if it indicated a valid intersection
    (if (<= 0.0 min-param 1.0)
	min-param
	nil)))

(defun segment-cell-intersection-parameter (p0 p1 cell-point resolution)
  "Return the parameter corresponding to the first intersection point of the
segment with the border of the cell.  This point is not guaranteed to be in the
cell, since the North and West borders of a cell are in the cell, while the
South and East borders are not---they belong to the cells to the South and East,
respectively.  See `segment-cell-interior-intersection-parameter' for an
alternative function.

In more detail: The border of a cell is defined by four segments, the North,
South, East and West edges.  Points on the North and West edges are IN the cell,
while points on the South and East edges are NOT in the cell; they are in the
cells to the South and East, because those same segments are the borders of
other cells, and each point can only be in one cell.  Essentially, the points IN
a cell are \(x,y\), such that

                CELL-SIZE = 2^RESOLUTION
                FLOOR[x/CELL-SIZE] <= x < FLOOR[x/CELL-SIZE]+CELL-SIZE

and similarly for y.  Note the <= versus the <."
  
  (let* ((cell-size (exp2 resolution))
	 (cell-west  (trunc2 (point-x cell-point) resolution))
	 (cell-north (trunc2 (point-y cell-point) resolution))
	 (cell-east  (+ cell-west cell-size))
	 (cell-south (+ cell-north cell-size)))
    (segment-box-intersection-parameter p0 p1 cell-north cell-south cell-east cell-west)))

(defun segment-cell-interior-intersection-parameter (p0 p1 cell-point resolution)
  "Return the parameter corresponding to the first intersection point of the
segment with the points IN the cell.  This point IS guaranteed to be in the
cell.  See `segment-cell-intersection-parameter' for an alternative function."
  
  (let* ((cell-size (- (exp2 resolution) 1))
	 (cell-west  (trunc2 (point-x cell-point) resolution))
	 (cell-north (trunc2 (point-y cell-point) resolution))
	 (cell-east  (+ cell-west cell-size))
	 (cell-south (+ cell-north cell-size)))
    (segment-box-intersection-parameter p0 p1 cell-north cell-south cell-east cell-west)))

(defun segment-cell-exterior-intersection-parameter (p0 p1 cell-point resolution)
  "Return the parameter corresponding to the first intersection point of the
segment with the points just outside the cell.  This point is guaranteed not to
be in the cell.  See `segment-cell-intersection-parameter' or
`segment-cell-interior-intersection-parameter' for an alternative function."
  (let* ((cell-size (+ (exp2 resolution) 1))
	 (cell-west  (- (trunc2 (point-x cell-point) resolution) 1))
	 (cell-north (- (trunc2 (point-y cell-point) resolution) 1))
	 (cell-east  (+ cell-west cell-size))
	 (cell-south (+ cell-north cell-size)))
    (segment-box-intersection-parameter
      p0 p1 cell-north cell-south cell-east cell-west)))

(defun segment-cell-intersection (p0 p1 cell-point resolution)
  "Return the first intersection point of the segment [P0,P1] with the cell
containing `cell-point' at `resolution'.  If there is no intersection, returns
NIL.  The returned point is rounded.

Caution: The border of a cell is defined by four segments, the North, South,
East and West edges.  Points on the North and West edges are IN the cell, while
points on the South and East edges are NOT in the cell; they are in the cells to
the South and East, because those same segments are the borders of other cells,
and each point can only be in one cell.  Essentially, the points IN a cell are
\(x,y\), such that

          CELL-SIZE = 2^RESOLUTION
          FLOOR[x/CELL-SIZE] <= x < FLOOR[x/CELL-SIZE]+CELL-SIZE

and similarly for y.  Note the <= versus the <.  See
`segment-cell-interior-intersection' for an alternative function."
  (let ((param (segment-cell-intersection-parameter p0 p1 cell-point resolution)))
    (when param
      (segment&parameter->point p0 p1 param (point)))))

(defun segment-cell-interior-intersection (p0 p1 cell-point resolution)
  "Return the first intersection point of the segment [P0,P1] with the cell
containing `cell-point' at `resolution'.  If there is no intersection, returns
NIL.  The returned point is rounded.  The returned point IS guaranteed to be IN
the cell.  See `segment-cell-intersection' for an alternative function."
  (let ((param (segment-cell-interior-intersection-parameter
		 p0 p1 cell-point resolution)))
    (when param
      (segment&parameter->point p0 p1 param (point)))))

(defun segment-cell-exterior-intersection (p0 p1 cell-point resolution)
  "Return the first intersection point of the segment [P0,P1] with the cell
containing `cell-point' at `resolution'.  If there is no intersection, returns
NIL.  The returned point is rounded.  The returned point is guaranteed not to be
in the cell, but next to it.  See `segment-cell-intersection' or
`segment-cell-interior-intersection' for an alternative function."
  (let ((param (segment-cell-exterior-intersection-parameter
		 p0 p1 cell-point resolution)))
    (when param
      (segment&parameter->point p0 p1 param (point)))))

#+test
(defun test-segment-cell-intersection (&optional interior?)
  (do ((cell  (spum) (spum)))
      (nil)
    (highlight-cell cell :resolution *fire-cell-resolution*)
    (let ((p0  (hspum))
	  (p1  (hspum)))
      (highlight-line p0 p1)
      (let ((i-point   (if interior?
			   (segment-cell-interior-intersection p0 p1 cell *fire-cell-resolution*)
			   (segment-cell-intersection          p0 p1 cell *fire-cell-resolution*))))
	(if (null i-point)
	    (format t "~&no intersection point.~%")
	    (point-round* i-point)
	    (format t "~&intersection point ~s ~:[IS NOT~;IS~] in the cell.~%"
		    i-point (points-in-same-cell-p cell i-point *fire-cell-resolution*))
	    (highlight-point i-point :color *highlight-cyan*))))))

;;;
;;; End: Segment x Segment Functions
;;;
;;;----------------------------------------------------------------------------
;;;----------------------------------------------------------------------------
;;;
;;; Start: Misc Functions
;;;

;;; From Explorer window macros code:
(defsubst sector-code (x y left top right bottom)
  "This returns a bit encoded value for the location of a point in relation to
an orthogonal rectangle.  The rectangle is the center box of the following
diagram and the returned value is indicated for each sector.

                    |        |
              1001  |  1000  |  1010		The first number is the
                9   |    8   |   10		binary value of the code 
                    |        |			if the point lies in that

            --------------------------		area.  The second number 
                    |        |			is the decimal value of
              0001  |  0000  |  0010		the code.
                1   |    0   |    2
                    |        |

            --------------------------
                    |        |
              0101  |  0100  |  0110
                5   |    4   |    6
                    |        |
                                 "
  
  (let ((code 0))
    (when (< x left)   (setq code (+ code 1)))
    (when (> x right)  (setq code (+ code 2)))
    (when (> y bottom) (setq code (+ code 4)))
    (when (< y top)    (setq code (+ code 8)))
    code))

; From file SYS:GWIN;UTILITY.LISP
(defun dist-from-rectangle (from-x from-y left top right bottom &optional (solid? nil))
  "This returns the distance between a point and a rectangle.  The rectangle may
be solid and if it is and the point lies inside of the rectangle then the
returned distance is negative and is the distance from the point to the edge of
the rectangle."
  (case (sector-code from-x from-y left top right bottom)
    (0 (cond  (solid? (max (- left from-x)
			   (- top from-y)
			   (- from-x right)
			   (- from-y bottom)))
              (t      (min (- from-x left)
			   (- from-y top)
			   (- right from-x)
			   (- bottom from-y)))))
    (1 (- left from-x))
    (2 (- from-x right))
    (4 (- from-y bottom))
    (5 (sqrt (point-separation-squared* from-x from-y left bottom)))
    (6 (sqrt (point-separation-squared* from-x from-y right bottom)))
    (8 (- top from-y))
    (9 (sqrt (point-separation-squared* from-x from-y left top)))
    (10 (sqrt (point-separation-squared* from-x from-y right top)))
    (otherwise 9999)))

(defun point-distance-from-square (test-point square-origin size &optional (solid? t))
  "This returns the distance between a point and a rectangle specified as an origin and a size.  The rectangle
may be solid; if it is and the point lies inside it, the returned distance is negative and is the distance from the
point to the edge of the rectangle."
  (dist-from-rectangle
    (point-x test-point)             (point-y test-point)
    (point-x square-origin)          (point-y square-origin)
    (+ (point-x square-origin) size) (+ (point-y square-origin) size)
    solid?))

(defun point-distance-from-extents (test-point upper-left lower-right &optional (solid? t))
  "This returns the distance between a point and a rectangle specified by `upper-left' and `lower-right'.  The
rectangle may be solid; if it is and the point lies inside it, the returned distance is negative and is the distance
from the point to the edge of the rectangle."
  (dist-from-rectangle 
    (point-x test-point)  (point-y test-point)
    (point-x upper-left)  (point-y upper-left)
    (point-x lower-right) (point-y lower-right)
    solid?))

;;;
;;; End: Misc Functions
;;;
;;;----------------------------------------------------------------------------
;;;----------------------------------------------------------------------------
;;;
;;; Start: Segment x Circle Functions
;;;

(defun line-circle-intersection-parameter (p0 p1 center radius)
  "Returns the parameter of the intersection point between the line from `p0'
through `p1,' and the circle defined by `center' and `radius.' Nil is returned
if segment does not intersect circle.  If the segment is tangential, only one
parameter is returned.  If there are two intersections, both are returned: the
smaller is the first value, the larger is the second value.  Implemented
following Bowyer and Woodward, section 2.2, pages 25--27."
  (check-type p0 rounded-point)
  (check-type p1 rounded-point)
  (check-type center rounded-point)
  (check-type radius (integer 1 *))
  (multiple-value-bind (f g) (segment-to-parametric-line p0 p1)
    (let ((xj  (coerce (point-x center) 'single-float))
	  (yj  (coerce (point-y center) 'single-float))
	  (x0  (coerce (point-x p0) 'single-float))
	  (y0  (coerce (point-y p0) 'single-float))
	  (r   (coerce radius 'single-float)))
      ;; Bowyer & Woodwark code starts here
      (let* ((fsq  (square (coerce f 'single-float)))
	     (gsq  (square (coerce g 'single-float)))
	     (fgsq (+ fsq gsq))
	     (xj0  (- xj x0))
	     (yj0  (- yj y0))
	     (fygx (- (* f yj0) (* g xj0)))
	     (root (- (* r r fgsq) (* fygx fygx))))
	(if (< root 0)
	    ;; line does not intersect circle (imaginary roots)
	    (values nil)
	    (let ((fxgy (+ (* f xj0) (* g yj0))))
	      (if (zerop root)
		  ;; line is tangential.  Return one parameter
		  (/ fxgy fgsq)
		  ;; line intersects twice.  Return two parameters.
		  (let* ((root  (sqrt root))
			 (fginv (/ 1.0 fgsq))
			 (t1    (* (- fxgy root) fginv))
			 (t2    (* (+ fxgy root) fginv)))
		    (if (< t1 t2)
			(values t1 t2)
			(values t2 t1))))))))))

(defun segment-circle-intersection-parameter (p0 p1 center radius)
  "Returns the parameter of the intersection point between the directed segment
from `p0' to `p1,' and the circle defined by `center' and `radius.' Nil is
returned if segment does not intersect circle.  If the segment is tangential,
only one parameter is returned.  If there are two intersections, both are
returned: the smaller is the first value, the larger is the second value.
Implemented following Bowyer and Woodward, section 2.2, pages 25--27."
  ;; return those elements of t1 and t2 which are within 0.0 and 1.0, but the
  ;; following `delete-if-not' is too inefficient
  #+ignore
  (values-list 
    (delete-if-not #'(lambda (x) (<= 0.0 x 1.0))
		   (multiple-value-list (line-circle-intersection-parameter p0 p1 center radius))))
  (multiple-value-bind (t1 t2) (line-circle-intersection-parameter p0 p1 center radius)
    (cond ((null t1) nil)
	  ((null t2)
	   (if (<= 0.0 t1 1.0) t1 nil))
	  ((<= 0.0 t1 1.0)
	   (values t1 (if (<= 0.0 t2 1.0) t2 nil)))
	  ((<= 0.0 t2 1.0)
	   t2)
	  (T nil))))

#+test 
(defun test-segment-circle-intersection-parameter (&optional (radius 100) (center (spum)))
  (highlight-point
    center :radius radius :color *highlight-white* :type :outline :units :meters)
  (loop (let ((p0  (spum))
	      (p1  (spum)))
	  (highlight-point p0 :color *highlight-yellow*)
	  (highlight-point p1 :color *highlight-cyan*)
	  (highlight-line p0 p1)
	  (timeit (:cpu :cons :number-cons)
		  (segment-circle-intersection-parameter p0 p1 center radius))
	  (multiple-value-bind (t1 t2)
	      (segment-circle-intersection-parameter p0 p1 center radius)
	    (spy t1 t2)
	    (when t1
	      (highlight-point
		(segment&parameter->point p0 p1 t1 (point))
		:color *highlight-yellow*))
	    (when t2
	      (highlight-point
		(segment&parameter->point p0 p1 t2 (point))
		:color *highlight-cyan*))
	    (y-or-n-p)))))
	    
;;;
;;; End: Segment x Circle Functions
;;;
;;;----------------------------------------------------------------------------
;;;----------------------------------------------------------------------------
;;;
;;; Start: Misc Functions
;;;

(defun vector-end-point-unrounded (start-point angle radius end-point)
  "Given a vector specified by `start-point,' `angle' and `radius', calculate
and return the endpoint.  `Angle' is in radians.  The returned point is not
rounded."
  (setf (point-x end-point) (+ (point-x start-point) (* radius (cos angle)))
	(point-y end-point) (+ (point-y start-point) (* radius (sin angle))))
  end-point)

(defun vector-end-point (start-point angle radius &optional (end-point (point 0 0)))
  "Given a vector specified by `start-point,' `angle' and `radius', calculate
and return the endpoint.  `Angle' is in absolute radians: clockwise from due
East.  Radius is in meters.  The returned point is rounded."
  (point-round* (vector-end-point-unrounded start-point angle radius end-point)))

#+test
(defun test-vector-end-point (&optional (center (hspum)))
  "Shows arrows from `center' every 45 degrees for one circle.  Each arrow is of
length 100 meters plus the number of degrees in the angle.  This function makes
it easy to see how `vector-end-point' works."
  (let ((ep (point)))
    (do ((angle 0 (+ angle 45)))
	((>= angle 360))
      (vector-end-point center (degrees->radians angle) (+ 100 angle) ep)
      (highlight-line center ep :arrowp t))))

(defun find-z-coordinate-in-plane (x1 y1 z1 x2 y2 z2 x3 y3 z3 x y)
  "Function to find the z coordinate given x and y and 3 points that define a
plane.  Round to meter."
  
  (let* ((a1 (- x3 x1))
	 (a2 (- y3 y1))
	 (a3 (- z3 z1))
	 (b1 (- x3 x2))
	 (b2 (- y3 y2))
	 (b3 (- z3 z2))
	 (a (- (* a2 b3) (* a3 b2)))
	 (b (- (* a3 b1) (* a1 b3)))
	 (c (- (* a1 b2) (* a2 b1))))
    
    (+ z1 (round (+ (* a (- x1 x)) (* b (- y1 y))) c))))

(defun point-in-cell-p (pt cell-ul cell-size)
  "Returns T iff `pt' is inside the cell specified by its upper-left point
`cell-ul' and its size in meters, `cell-size.'"
  (let* ((ulx (point-x cell-ul))
	 (uly (point-y cell-ul))
	 (lrx (+ ulx cell-size -1))
	 (lry (+ uly cell-size -1)))
    (and (<= ulx (point-x pt) lrx)
	 (<= uly (point-y pt) lry))))

;;;
;;; End: Misc Functions
;;;
;;;----------------------------------------------------------------------------
;;;----------------------------------------------------------------------------
;;;
;;; Start: Segments with widths
;;;

(defun segment-side-segments (p1 p2 width
				 &optional
				 (s10 (point))
				 (s11 (point))
				 (s20 (point))
				 (s21 (point)))
                                 
  "Return the two segments on the boundary of the argument segment (rounded).
One segment is [s10,s11] and the other is [s20,s21]."
  (declare (values s11 s12 s21 s22))
  (multiple-value-bind (ignore sin cos) (point-separation-and-sin-cos p1 p2)
    (declare (ignore ignore))
    (let* ((w/2 (* width .5))
	   (x1 (point-x p1))
	   (y1 (point-y p1))
	   (x2 (point-x p2))
	   (y2 (point-y p2))
	   ;; + 90 degrees
	   (dx1 (round (* w/2 (- sin))))   ; cos(x+90) = -sin(x)
	   (dy1 (round (* w/2 cos)))       ; sin(x+90) = (cos x)

	   ;; -90 degrees
	   (dx2 (round (* w/2 sin)))	   ; cos(x-90) = sin(x)
	   (dy2 (round (* w/2 (- cos))))   ; sin(x-90) = -cos(x)
	   )
      
      (values
	(point (+ x1 dx1) (+ y1 dy1) s10)
	(point (+ x2 dx1) (+ y2 dy1) s11)
	
	(point (+ x1 dx2) (+ y1 dy2) s20)
	(point (+ x2 dx2) (+ y2 dy2) s21)))))

#+test
(defun test-seg-sides (width &optional edge)
  (let ((p1 (if edge (feature-edge-from-point edge) (spum)))
	(p2 (if edge (feature-edge-to-point edge) (spum))))
  (multiple-value-bind (p11 p12 p21 p22)
      (segment-side-segments p1 p2 width)
    (highlight-line p1 p2)
    (highlight-line p11 p12)
    (highlight-line p21 p22))))


;;;
;;; End: Segments with widths
;;;
;;;----------------------------------------------------------------------------
;;;----------------------------------------------------------------------------
;;;
;;; Start: Misc. Functions
;;;

(defun extent-intersection (ul1 lr1 ul2 lr2)
  "Given two extents, return their intersection."
  (let ((x0 (max (point-x ul1) (point-x ul2)))
	(y0 (max (point-y ul1) (point-y ul2)))
	(x1 (min (point-x lr1) (point-x lr2)))
	(y1 (min (point-y lr1) (point-y lr2))))
    (when (and (<= x0 x1) (<= y0 y1))
      (values (point x0 y0) (point x1 y1)))))


(defun area-of-triangle (p0 p1 p2)
  "Return the area of the triangle p0 p1 p2."
  (let ((x01 (- (point-x p0) (point-x p1)))
	(x21 (- (point-x p2) (point-x p1)))
	(y01 (- (point-y p0) (point-y p1)))
	(y21 (- (point-y p2) (point-y p1))))
    (abs (* .5 (- (* x01 y21) (* x21 y01))))))

;;;
;;; End: Misc. Functions
;;;
;;; ***************************************************************************
;;; EOF
