;;
;; clim mailreader
;;

(defpackage :mail-reader (:use ekslmail clim clim-lisp))
(in-package :mail-reader)

(export '(browse author-mail))

;;

(define-application-frame mail-reader ()
  ((mailbox :initform NIL :initarg :mailbox :accessor mr-mailbox)
   (cmsg :initform NIL :accessor mr-cmsg))
  (:panes
   (text :application :end-of-line-action :allow :end-of-page-action :allow
         :incremental-redisplay t :display-function 'mr-display-msg)
   (messages list-pane :scroll-bars :vertical :width 600 :min-width 600
             :value-changed-callback 'mr-newmess)
   (mail-file text-field :width 100 :min-height 30 :height 30 :max-height 30
              :value (format nil "/usr/spool/mail/~A" (first (last (pathname-directory (user-homedir-pathname))))))
   (read-file push-button :activate-callback #'(lambda (b) (mr-readmail (pane-frame b)))
	      :label "READ" :width 80 :max-width 80 :max-height 30 :height 30)
   (write-file push-button :activate-callback #'(lambda (b) (mr-savemail (pane-frame b)))
	      :label "WRITE" :width 80 :max-width 80 :max-height 30 :height 30)
   (header toggle-button :label "Headers" :min-height 22 :height 22 :max-height 22
           :value-changed-callback #'(lambda (b n) (declare (ignore n)) (redisplay-frame-pane (pane-frame b) 'text)))
   (reply push-button :label "REPLY" :min-height 22 :height 22 :max-height 22
          :activate-callback #'(lambda (b) (mr-reply (pane-frame b))))
   (mail  push-button :label "MAIL" :min-height 22 :height 22 :max-height 22
          :activate-callback #'(lambda (b) (mr-mail (pane-frame b))))
   (print push-button :label "PRINT"  :min-height 22 :height 22 :max-height 22
          :activate-callback #'(lambda (b) (mr-print (pane-frame b))))
   (delete push-button :label "DELETE" :min-height 22 :height 22 :max-height 22
           :activate-callback #'(lambda (b) (mr-delete (pane-frame b))))
   (undelete push-button :label "UNDELETE" :min-height 22 :height 22 :max-height 22
           :activate-callback #'(lambda (b) (mr-undelete (pane-frame b))))
   (exit push-button :activate-callback #'(lambda (b) (mr-quit (pane-frame b)))
         :label "QUIT" :max-height 22 :height 22 :min-height 22)
   (command :interactor :max-height 150))
  (:layouts (default (vertically ()
		       (3/6 text)
		       (2/6 (vertically () messages
                              (horizontally () header reply mail print delete undelete exit)
                              (horizontally () read-file mail-file write-file)))
                       (1/6 command)))))

(defmethod mr-print ((mr mail-reader))
  (notify-user mr "Print it yourself, dude."))

(defmethod mr-delete ((mr mail-reader))
  (let ((pane (find-pane-named mr 'messages)))
    (mark-message (mr-cmsg mr) :delete)
    (setf (set-gadget-items pane) (set-gadget-items pane))
    (repaint-sheet pane (sheet-region pane))))

(defmethod mr-undelete ((mr mail-reader))
  (let ((pane (find-pane-named mr 'messages)))
    (unmark-message (mr-cmsg mr) :delete)
    (setf (set-gadget-items pane) (set-gadget-items pane))
    (repaint-sheet pane (sheet-region pane))))

(defmethod mr-mail ((mr mail-reader))
  (author-mail))

(defmethod mr-reply ((mr mail-reader))
  (let ((mesg (mr-cmsg mr)))
    (when mesg
      (author-mail :quote (mapcar #'(lambda (l) (format nil "> ~A" l)) (msg-body mesg))
                   :to (princ (or (hdr-reply-to (msg-header mesg)) (hdr-from-address (msg-header mesg))) *error-output*)
                   :subject (format nil (if (search "Re:" (hdr-subject (msg-header mesg)))
                                            "~A" "Re: ~A") (hdr-subject (msg-header mesg)))))))

(defun mr-newmess (button new-item)
  (let ((fr (pane-frame button)))
    (unless (eql new-item (mr-cmsg fr))
      (setf (mr-cmsg fr) new-item)
      (redisplay-frame-pane fr 'text :force-p t))))

(defmethod mr-display-msg ((mr mail-reader) stream)
  (when (mr-cmsg mr)
    (when (gadget-value (find-pane-named mr 'header))
      (with-text-face (stream :italic)
	(loop for l in (msg-header-text (mr-cmsg mr)) do
	      (present (format nil "~A~%" l) 'string :stream stream)))
      (terpri stream))
    (with-text-face (stream :bold)
      (loop for l in (msg-body (mr-cmsg mr)) do
	    (present (format nil "~A~%" l) 'string :stream stream)))))

(defmethod mr-readmail ((mr mail-reader))
  (let* ((mailfile (gadget-value (find-pane-named mr 'mail-file)))
         (new-mb (read-mailbox mailfile))
         (msgpane (find-pane-named mr 'messages)))
    (when new-mb
      (setf (mr-mailbox mr) new-mb)
      (setf (clim:set-gadget-items msgpane) (mb-messages new-mb))
      (repaint-sheet msgpane (sheet-region msgpane)))))

;; THIS IS KIND OF SCREWED. IT DOES SCARY THINGS.
(defmethod mr-savemail ((mr mail-reader))
  (let* ((mailfile (gadget-value (find-pane-named mr 'mail-file)))
         (mb (mr-mailbox mr))
         (msgpane (find-pane-named mr 'messages)))
    (when mb
      ;; we should really test the waters here for new mail
      ;; archive the old mailbox
      (archive-mailbox mailfile)
      ;; delete the marked messages
      (setf (mb-messages mb) (delete-if #'(lambda (m) (member :delete (msg-markings m))) (mb-messages mb)))
      ;; dump the mailfile
      (dump-mailbox mb mailfile)
      ;; set & redraw the mailbox
      (setf (clim:set-gadget-items msgpane) (mb-messages mb))
      (repaint-sheet msgpane (sheet-region msgpane)))))

(defmethod mr-quit ((mr mail-reader))
  ;; clean up the mail stuff... i.e. delete, etc.
  (frame-exit mr))

(defun browse ()
  (let* ((port (find-port))
         (framem (typecase port
                   (frame-manager port)
                   (t (find-frame-manager :port port))))
         (new-frame (make-application-frame 'mail-reader
                           :frame-manager framem
                           :width 600 :height 600)))
    (mp:process-run-function "Mail Reader"
                             '(:run-reasons :enable)
                             #'run-frame-top-level new-frame)
    (mr-readmail new-frame)))

;;
;; Mail sender
;;

(define-application-frame mail-editor ()
  ((text :initform "" :initarg :text :accessor me-text)
   (quote :initform NIL :initarg :quote :accessor me-quote)
   (info :initform NIL :accessor me-info))
  (:panes
   (to   text-field  :max-height 30 :height 30 :min-height 30 :value "To: ")
   (cc   text-field  :max-height 30 :height 30 :min-height 30 :value "CC: ")
   (subject   text-field  :max-height 30 :height 30 :min-height 30 :value "Subject: ")
   (edit text-editor :word-wrap t :ncolumns 79 :width 400)
   (send push-button :label "SEND" :max-height 30 :min-height 30 :height 30
         :activate-callback #'(lambda (b)
				(me-sendmail (pane-frame b))
				(frame-exit (pane-frame b))))
   (quote push-button :label "QUOTE" :max-height 30 :min-height 30 :height 30
          :activate-callback #'(lambda (b) (me-quotemail (pane-frame b))))
   (abort push-button :label "ABORT" :max-height 30 :min-height 30 :height 30
          :activate-callback #'(lambda (b) (frame-exit (pane-frame b)))))
  (:layouts (default (vertically ()
                       to cc subject
                       edit
                       (horizontally () send quote abort)))))

(defmethod me-sendmail ((me mail-editor))
  (let ((to (gadget-value (find-pane-named me 'to)))
        (cc (gadget-value (find-pane-named me 'cc)))
        (subject (gadget-value (find-pane-named me 'subject)))
        (body (gadget-value (find-pane-named me 'edit))))
    (sendmail to (format nil "~A~%~A~%~%~A" subject cc body))))

(defmethod me-quotemail ((me mail-editor))
  (let ((pane  (find-pane-named me 'edit)))
    (setf (gadget-value pane)
          (format nil "~{~A~%~}~%~A" (me-quote me) (or (gadget-value pane) "")))
    (repaint-sheet pane (sheet-region pane))))

(defun author-mail (&key quote to subject cc)
  (let* ((port (find-port))
         pane-buffer
         (framem (typecase port
                   (frame-manager port)
                   (t (find-frame-manager :port port))))
         (new-frame (make-application-frame 'mail-editor
		      :frame-manager framem
		      :width 400 :height 400
                      :quote quote)))
    (mp:process-run-function "Mail Editor"
                             '(:run-reasons :enable)
                             #'run-frame-top-level new-frame)
    ;; set up the text fields
    (when to
      (setq pane-buffer (find-pane-named new-frame 'to))
      (setf (gadget-value pane-buffer) (format nil "To: ~A" to))
      (repaint-sheet pane-buffer (sheet-region pane-buffer)))
    (when cc
      (setq pane-buffer (find-pane-named new-frame 'cc))
      (setf (gadget-value pane-buffer) (format nil "CC: ~A" cc))
      (repaint-sheet pane-buffer (sheet-region pane-buffer)))
    (when subject
      (setq pane-buffer (find-pane-named new-frame 'subject))
      (setf (gadget-value pane-buffer) (format nil "Subject: ~A" subject))
      (repaint-sheet pane-buffer (sheet-region pane-buffer)))
    ))

    
