;;;; -*- Mode: Common-Lisp; Package: EKSL-UTILITIES; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/menus.lisp *-*
;;;; *-* Last-Edit: Monday, July 24, 1995  11:53:50; Edited-By: Westy *-*
;;;; *-* Machine: Sphinx (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                            Menu Definitions                            *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: David L. Westbrook

;;; Department of Computer Science
;;; University of Massachusetts
;;; Amherst MA 01003-4610

;;; Copyright (c) 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  07-24-95 File Created. Moved this code from DEBUGGING-UTILS.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :EKSL-UTILITIES)

;;; --*--
;;; ***************************************************************************

(provide-utilities
  (defun eksl-multiple-menu-choose
	 (items
	  &rest options
	  &key label original-selections default-selections
	  add-item-groups delete-item-groups
          #+MCL
          item-print-function
	  #-Explorer
	  (stream *standard-input*))
    
    "A facility for selecting a subset from a group of items.  A list of items
is provided, from which any number may be selected.  The function returns 2
values: the list of selected items, and an exit condition \(T when exited
safely; NIL when aborted\).

`label' is a string that provides the label for the menu; `ORIGINAL-SELECTIONS'
specifies the items to be initially selected; `DEFAULT-SELECTIONS' specifies a
set of default items \(accessible through a menu option\); `ADD-ITEM-GROUPS' is
a list of string and items pairs describing groups of items that can be selected
through a menu option; `DELETE-ITEM- GROUPS' is a list of string and items pairs
describing items that can be unselected through a menu option.

Additional unhandled window options will be passed on to system menu function if
appropriate."
    #-Explorer
    (declare (ignore options))
    #+MCL
    (mcl-multiple-menu-choose items 
                              :label label
                              :original-selections original-selections
                              :default-selections default-selections
                              :add-item-groups add-item-groups
                              :delete-item-groups delete-item-groups
                              :item-print-function item-print-function)

    #+Explorer
    (flet ((split-items (items divisor)
	     (let ((start 0)
		   (end divisor)
		   (result nil)
		   (stop (length items)))
	       (loop
		 (push
		   `("======== ITEMS ========"
		     '(("" :value nil :no-select)
		       ,@(subseq items start end))
		     :font fonts:tr12) result)
		 (setf start end)
		 (incf end divisor)
		 (when (> start stop)
		   (return (nreverse result)))))))
      
      (apply #'w:menu-choose
	     `(,@(split-items items 40)
	       ("============ META CHOICES ============"
		'(("" :value nil :no-select)
		  ("Original Selections"
		   :execute (zlc:funcall-self :set-highlighted-items ',original-selections))
		  ("Original Defaults"
		   :execute (zlc:funcall-self :set-highlighted-items ',default-selections))
		  ("Select All"
		   :execute (zlc:funcall-self :set-highlighted-items ',items))
		  ("DeSelect All"
		   :execute (zlc:funcall-self :set-highlighted-items nil))
		  ,@(when add-item-groups
		      `(("" :value nil :no-select)
			("======== GROUPS OF ITEMS TO TURN ON ========"
			 :value nil :font fonts:tr12 :no-select)
			("" :value nil :no-select)
			,@(mapcar
                            #'(lambda (group-list)
                                `(,(first group-list)
				  :execute (zlc:funcall-self
                                             :set-highlighted-items
                                             (union (zlc:funcall-self :highlighted-items)
                                                    ',(second group-list)))))
                            add-item-groups)))
		  ,@(when delete-item-groups
		      `(("" :value nil :no-select)
			("======== GROUPS OF ITEMS TO TURN OFF ========"
			 :value nil :font fonts:tr12 :no-select)
			("" :value nil :no-select)
			,@(mapcar
                            #'(lambda (group-list)
                                `(,(first group-list)
				  :execute (zlc:funcall-self
                                             :set-highlighted-items
                                             (set-difference
					       (zlc:funcall-self :highlighted-items)
					       ',(second group-list)))))
                            delete-item-groups))))
		:font fonts:tr12))
	     :multicolumn t
	     :highlighted-items original-selections
	     :highlighting t
	     :item-alignment :center
	     :label `(:centered :string ,(format nil "~A" (or label "Select Items")))
	     :menu-margin-choices '(:doit)
	     :allow-other-keys t
	     options))
    
    #-(or Explorer MCL)
    (let* ((selection-array (make-array (length items) :element-type 'bit :initial-element 0))
	   (status-list (mapcar #'(lambda (item) (member item original-selections)) items))
						;(separator (make-array 80 :element-type 'character :initial-element #\-))
	   (number-of-items         (length items))
	   (number-of-delete-groups (length delete-item-groups))
	   (number-of-add-groups    (length add-item-groups))
	   (select-all (+ 1 number-of-items number-of-delete-groups number-of-add-groups))
	   (deselect-all (+ 2 number-of-items number-of-delete-groups number-of-add-groups))
	   (select-originals (when original-selections
			       (+ 3 number-of-items number-of-delete-groups number-of-add-groups)))
	   (select-defaults  (when default-selections
			       (+ 4 number-of-items number-of-delete-groups number-of-add-groups)))
	   (exit?)
	   (abort?))
      
      ;; Initialize the `selection-array':
      (dotimes (index number-of-items)
	(when (nth index status-list) (setf (aref selection-array index) 1))) 
      
      (loop until (or exit? abort?) do
	    (flet ((READ-DELIMITED-NUMBERS ()
		     
		     "READ-DELIMITED-NUMBERS nil

Returns the sequence of integers found in a string read in using `read-line'."
		     
		     (do* ((number-string (read-line stream))
			   (start 0)
			   (string number-string (subseq string start))
			   (result))
			  ((string= "" string) (nreverse result))
		       (multiple-value-bind (integer index)
			   (parse-integer string :JUNK-ALLOWED t)
			 (cond ((integerp integer)
				(push integer result)
				(setf start index))
			       ((and (null integer) (zerop index))
				(setf start 1)))))))
	      
	      ;; Draw the menu:
	      (format t "~|~80,,,'-<~>~%~80:@<~A~>~%~80,,,'-<~>~%~80:@<Items~>~2%~:{~&~C (~2D) ~A~}~%~80,,,'-<~>~%"
						;separator
		      (or label "Select Items")
						;separator
		      (let ((result))
			(dotimes (index number-of-items (nreverse result))
			  (push `(,(if (zerop (aref selection-array index)) #\Space #\X)
                                  ,(1+ index)
                                  ,(if (zerop (aref selection-array index))
                                       (string-downcase (format nil "~S" (nth index items)))
                                       (string-upcase (string (format nil "~S" (nth index items))))))
				result)))
						;separator
		      )
	      (when add-item-groups
		(format t "~80:@<Groups of Items to Turn ON~>~2%~:{~&  (~2D) ~A~}~%~80,,,'-<~>~%"
			(let ((result))
			  (dotimes (index (length add-item-groups) (nreverse result))
			    (push `(,(+ number-of-items index 1)
                                    ,(string-capitalize (format nil "~S" (first (nth index add-item-groups)))))
				  result)))
						;separator
			))
	      (when delete-item-groups
		(format t "~80:@<Groups of Items to Turn OFF~>~2%~:{~&  (~2D) ~A~}~%~80,,,'-<~>~%"
			(let ((result))
			  (dotimes (index (length delete-item-groups) (nreverse result))
			    (push `(,(+ number-of-items number-of-add-groups index 1)
                                    ,(string-capitalize (format nil "~S" (first (nth index delete-item-groups)))))
				  result)))
						;separator
			))
	      (format t "~80:@<Meta Choices~>~2%~:{~&  (~2D) ~A~}~%~80,,,'-<~>~%~
                 Enter selection numbers (include -1 to abort; 0 to exit): "
		      `((,select-all "Select All")
			(,deselect-all "DeSelect All")
			,@(when select-originals
			    `((,select-originals "Original Selections")))
			,@(when select-defaults
			    `((,select-defaults "Original Defaults"))))
						;separator
		      )
	      
	      ;; Process commands:
	      (mapc #'(lambda (selection)
			(unless exit?
			  (cond ((zerop selection)
				 (setf exit? t))
				((= -1 selection)
				 (setf abort? t))
				((< number-of-items selection (+ number-of-items number-of-add-groups 1))
				 (dotimes (index number-of-items)
				   (when (member (nth index items)
						 (second (nth (- selection number-of-items 1) add-item-groups)))
				     (setf (aref selection-array index) 1))))
				((< (+ number-of-items number-of-add-groups) selection select-all)
				 (dotimes (index number-of-items)
				   (when (member (nth index items)
						 (second (nth (- selection number-of-items number-of-add-groups 1)
							      delete-item-groups)))
				     (setf (aref selection-array index) 0))))
				((= selection select-all)
				 (dotimes (index number-of-items)
				   (setf (aref selection-array index) 1)))
				((= selection deselect-all)
				 (dotimes (index number-of-items)
				   (setf (aref selection-array index) 0)))
				((and select-originals (= selection select-originals))
				 (let ((status-list (mapcar #'(lambda (item)
								(member item original-selections))
							    items)))
				   (dotimes (index number-of-items)
				     (if (nth index status-list)
					 (setf (aref selection-array index) 1)
					 (setf (aref selection-array index) 0)))))
				((and select-defaults (= selection select-defaults))
				 (let ((status-list (mapcar #'(lambda (item)
								(member item default-selections))
							    items)))
				   (dotimes (index number-of-items)
				     (if (nth index status-list)
					 (setf (aref selection-array index) 1)
					 (setf (aref selection-array index) 0)))))
				(t
				 (decf selection)
				 (when (and (integerp selection) (<= 0 selection number-of-items))
				   (setf (aref selection-array selection)
					 (abs (- (aref selection-array selection) 1))))))))
		    (read-delimited-numbers))))
      
      (values
	(let ((result))
	  (dotimes (index number-of-items result)
	    (when (= 1 (aref selection-array index))
	      (push (nth index items) result))))
	exit?))))

;; We define this function for choosing a single item from a menu instead
;; of using clim:menu-choose because clim:menu-choose insists on creating
;; menus with scroll bars, which looks stupid for small menus.

;; We have a choice between using as :PARENT either *clim-root* or
;; (clim:frame-top-level-window clim:*application-frame*).  In the former
;; case the menu is not positioned at the pointer, but it does get a label;
;; in the latter case it is positioned at the pointer, but it does not get
;; a label.  Arg!  Furthermore, in the latter case Franz CLIM 1.1 barfs on us.
;; We make one choice for Franz and one choice for Lucid because the SRI
;; preference is to have the menu positioned at the pointer, whereas the
;; Franz choice is to not have it break.

(defvar *application-root* nil
  "Set this variable to the root application frame, such as the value of
`open-root-window'.")

#+CLIM-1
(defun clim-single-choice-menu (item-list &key label)
  #+clim-1
  (assert *application-root* () "Should be bound to an application-frame")
  (let (menu-window
	#+clim-1
	(parent (or (clim:frame-top-level-window clim:*application-frame*)
		    *application-root*)))

    #+(and Franz-Inc clim-1)
    (setq parent (clim:window-root parent))

    (multiple-value-bind (x y)
			 (#+clim-1 clim:stream-pointer-position*
			  #+clim-2 clim:stream-pointer-position
			  parent)

      (setq menu-window
	    (clim:open-window-stream #+clim-1 :parent #+clim-1 parent
				     :scroll-bars (if (> 25 (length item-list))
						      nil
						    :vertical)
				     :left x
				     :top y
				     :label label))

      (multiple-value-bind (item gesture)
			   (clim::menu-choose-from-drawer
			    menu-window
			    'clim::menu-item
			    #'(lambda(stream type)
				(dolist (menu-item item-list)
                                  (format stream " ")
				  (clim:with-text-style ((or (getf (cdr menu-item) :style)
							     nil)
							 stream)
                                    ;; Print the current item.  If we print "" this
				    ;; causes the CLIM window to grow much bigger with
				    ;; lots of white space at the borders, so we
				    ;; print " " in place of "".

                                    (clim:present (format nil "~A" (if (string-equal "" (car menu-item))
								       " "
								       (car menu-item) ))
						  type
						  :stream stream) )
				  (format stream " ~%"))
				)
			    :x-position x
			    :y-position y)
			   (declare (ignore gesture))

        (getf (cdr (assoc item item-list :test #'equal))
	      :value)
	) ) ) )

(provide-utilities
 (defun eksl-menu-choose (items &rest options &key label (stream *standard-input*))
   #+Explorer
   (declare (ignore label stream))
   #+Explorer
   (apply #'w:menu-choose items options)
   #-Explorer
   (if *application-root*
       (apply #'clim-single-choice-menu items options)
       (progn
         ;; Draw the menu:
         (format t "~|~80,,,'-<~>~%~80:@<~A~>~%~80,,,'-<~>~%~80:@<Items~>~2%~:{~&(~2D) ~A~}~%~80,,,'-<~>~%~
Enter selection numbers (0 to exit):"
	         (or label "Select Items")
	         (loop for item in items
		       for cnt from 0
		       for name = (if (consp item) (first item) item)
		       collect `(,(1+ cnt) ,name)))
         #-Explorer
         (loop for integer = (parse-integer (read-line stream) :JUNK-ALLOWED t) do
	       (when (integerp integer)
		 (if (zerop integer)
		     (return nil)
		     (let ((selection (nth (1- integer) items)))
		       (return
		        (if (consp selection)
			    (or (getf (rest selection) :value)
			        (second selection))
			    selection))))))))))

#+MCL
(defun mcl-multiple-menu-choose (items &key label original-selections default-selections
	                               add-item-groups delete-item-groups
                                       (item-print-function #'princ-to-string))
  (let* ((selected-items original-selections)
         (row-width 14)
         (rows (round 300 row-width))
         (column-width (+ 10
                           (loop for item in items
                                 maximize (string-width 
                                           (funcall item-print-function item)
                                           '("Agaramond Semibold" 10 :SRCOR)
                                           ))))
         (columns (ceiling (length items) rows))
         (width (+ 20 (* column-width columns)))
         (height (+ 50 (* row-width rows)))
         (finish-button-margin 50)
         (dialog (MAKE-INSTANCE 'COLOR-DIALOG
                  :WINDOW-TYPE :TOOL 
                  :WINDOW-TITLE (or label "Choose Item")
                  :VIEW-POSITION '(:top 40)
                  :VIEW-SIZE (make-point width height)
                  :CLOSE-BOX-P NIL
                  :window-show nil
                  :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
                  :VIEW-SUBVIEWS
                  (append
                   (list
                    (make-dialog-item
                    'button-dialog-item
                    (make-point finish-button-margin (- height 20))
		    ;#@(140 114)
                    ;;;;;;#@(55 14)
                    "Done"
                    #'(lambda (item)
                        (declare (ignore item))
                        (return-from-modal-dialog (values selected-items t)))
                    :view-font '("Agaramond Semibold" 10 :SRCOR)
                    )
                    (make-dialog-item
                     'button-dialog-item
                     (make-point (- width 55 finish-button-margin)  (- height 20))
                     ;#@(10 114)
                    ;;;;;;#@(55 14)
                    "Abort"
                    #'(lambda (item)
                        (declare (ignore item))
                        (return-from-modal-dialog (values original-selections nil)))
                    :view-font '("Agaramond Semibold" 10 :SRCOR)
                    ))
                   (loop for item in items
                         for cnt from 0
                         with x = (- 10 column-width)
                         for y from 10 by 15
                         when (zerop (nth-value 1 (floor cnt rows)))
                        do (setf x (incf x column-width) y 10)
                        collect
                        (make-dialog-item
                         'check-box-dialog-item
                         (make-point x (- y 2))
                         (make-point column-width 14)
                         ;#@(55 14)
                         (funcall item-print-function item)
                         #'(lambda (box)
                             (if (check-box-checked-p box)
                               (pushnew (view-nick-name box) selected-items)
                               (setf selected-items 
                                     (remove (view-nick-name box) 
                                             selected-items))))
                         :view-nick-name item
                         :view-font '("Agaramond Semibold" 10 :SRCOR)
                         :check-box-checked-p (member item original-selections)
                         ))))))
                   
    (modal-dialog dialog)))

;;; ***************************************************************************
;;; EOF